@@ -3,7 +3,6 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-import warnings
 # THIRD-PARTY
 import numpy as np
 
@@ -33,7 +32,7 @@ def test_blackbody_scipy():
 
     """
     flux_unit = u.Watt / (u.m ** 2 * u.um)
-    wave = np.logspace(0, 8, 1e6) * u.AA
+    wave = np.logspace(0, 8, 100000) * u.AA
     temp = 100. * u.K
     with np.errstate(all='ignore'):
         bb_nu = blackbody_nu(wave, temp) * u.sr
@@ -122,4 +121,3 @@ def test_blackbody_array_temperature():
 
     flux = blackbody_nu(np.ones((3, 4)) * u.mm, np.ones(4) * u.K)
     assert flux.shape == (3, 4)
-
