@@ -10,6 +10,10 @@ import numpy as np
 from .core import Kernel, Kernel1D, Kernel2D, MAX_NORMALIZATION
 from ..utils.exceptions import AstropyUserWarning
 from ..utils.console import human_file_size
+from .. import units as u
+from ..nddata import support_nddata
+
+from ..extern.six.moves import range, zip
 
 
 # Disabling all doctests in this module until a better way of handling warnings
@@ -17,26 +21,30 @@ from ..utils.console import human_file_size
 __doctest_skip__ = ['*']
 
 
+@support_nddata(data='array')
 def convolve(array, kernel, boundary='fill', fill_value=0.,
-             normalize_kernel=False):
+             normalize_kernel=False, mask=None):
     '''
     Convolve an array with a kernel.
 
-    This routine differs from `scipy.ndimage.filters.convolve` because
+    This routine differs from `scipy.ndimage.convolve` because
     it includes a special treatment for ``NaN`` values. Rather than
-    including ``NaN``s in the convolution calculation, which causes large
-    ``NaN`` holes in the convolved image, ``NaN`` values are replaced with
-    interpolated values using the kernel as an interpolation function.
+    including ``NaN`` values in the array in the convolution calculation, which
+    causes large ``NaN`` holes in the convolved array, ``NaN`` values are
+    replaced with interpolated values using the kernel as an interpolation
+    function.
 
     Parameters
     ----------
-    array : `numpy.ndarray`
+    array : `numpy.ndarray` or `~astropy.nddata.NDData`
         The array to convolve. This should be a 1, 2, or 3-dimensional array
         or a list or a set of nested lists representing a 1, 2, or
-        3-dimensional array.
+        3-dimensional array.  If an `~astropy.nddata.NDData`, the ``mask`` of
+        the `~astropy.nddata.NDData` will be used as the ``mask`` argument.
     kernel : `numpy.ndarray` or `~astropy.convolution.Kernel`
         The convolution kernel. The number of dimensions should match those
         for the array, and the dimensions should be odd in all directions.
+        If a masked array, the masked values will be replaced by ``fill_value``.
     boundary : str, optional
         A flag indicating how to handle boundaries:
             * `None`
@@ -50,9 +58,15 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
                 Set values outside the array to the nearest ``array``
                 value.
     fill_value : float, optional
-        The value to use outside the array when using boundary='fill'
+        The value to use outside the array when using ``boundary='fill'``
     normalize_kernel : bool, optional
         Whether to normalize the kernel prior to convolving
+    mask : `None` or `numpy.ndarray`
+        A "mask" array.  Shape must match ``array``, and anything that is masked
+        (i.e., not 0/`False`) will be set to NaN for the convolution.  If
+        `None`, no masking will be performed unless ``array`` is a masked array.
+        If ``mask`` is not `None` *and* ``array`` is a masked array, a pixel is
+        masked of it is masked in either ``mask`` *or* ``array.mask``.
 
     Returns
     -------
@@ -65,7 +79,7 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
 
     Notes
     -----
-    Masked arrays are not supported at this time.  The convolution
+    For masked arrays, masked values are treated as NaNs.  The convolution
     is always done at ``numpy.float`` precision.
     '''
     from .boundary_none import (convolve1d_boundary_none,
@@ -145,6 +159,22 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
         raise Exception('array and kernel have differing number of '
                         'dimensions.')
 
+    # anything that's masked must be turned into NaNs for the interpolation.
+    # This requires copying the array_internal
+    array_internal_copied = False
+    if np.ma.is_masked(array):
+        array_internal = array_internal.filled(np.nan)
+        array_internal_copied = True
+    if mask is not None:
+        if not array_internal_copied:
+            array_internal = array_internal.copy()
+            array_internal_copied = True
+        # mask != 0 yields a bool mask for all ints/floats/bool
+        array_internal[mask != 0] = np.nan
+    if np.ma.is_masked(kernel):
+        # *kernel* doesn't support NaN interpolation, so instead we just fill it
+        kernel_internal = kernel.filled(fill_value)
+
     # Because the Cython routines have to normalize the kernel on the fly, we
     # explicitly normalize the kernel here, and then scale the image at the
     # end if normalization was not requested.
@@ -220,12 +250,13 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
         return result
 
 
+@support_nddata(data='array')
 def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
-                 return_fft=False, fft_pad=True, psf_pad=False,
+                 return_fft=False, fft_pad=None, psf_pad=None,
                  interpolate_nan=False, quiet=False, ignore_edge_zeros=False,
                  min_wt=0.0, normalize_kernel=False, allow_huge=False,
                  fftn=np.fft.fftn, ifftn=np.fft.ifftn,
-                 complex_dtype=np.complex):
+                 complex_dtype=np.complex, mask=None):
     """
     Convolve an ndarray with an nd-kernel.  Returns a convolved image with
     shape = array.shape.  Assumes kernel is centered.
@@ -247,11 +278,14 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
 
     Parameters
     ----------
-    array : `numpy.ndarray`
-          Array to be convolved with ``kernel``
+    array : `numpy.ndarray` or `~astropy.nddata.NDData`
+          Array to be convolved with ``kernel``. If `~astropy.nddata.NDData`,
+          the ``mask`` of the `~astropy.nddata.NDData` will be used as the
+          ``mask`` argument.
     kernel : `numpy.ndarray`
           Will be normalized if ``normalize_kernel`` is set.  Assumed to be
-          centered (i.e., shifts may result if your kernel is asymmetric)
+          centered (i.e., shifts may result if your kernel is asymmetric).
+          If a masked array, the masked values will be replaced by ``fill_value``.
     boundary : {'fill', 'wrap'}, optional
         A flag indicating how to handle boundaries:
 
@@ -280,14 +314,23 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
         e.g., ``normalize_kernel=np.sum`` means that kernel will be modified to be:
         ``kernel = kernel / np.sum(kernel)``.  If True, defaults to
         ``normalize_kernel = np.sum``.
+    mask : `None` or `numpy.ndarray`
+        A "mask" array.  Shape must match ``array``, and anything that is masked
+        (i.e., not 0/`False`) will be set to NaN for the convolution.  If
+        `None`, no masking will be performed unless ``array`` is a masked array.
+        If ``mask`` is not `None` *and* ``array`` is a masked array, a pixel is
+        masked of it is masked in either ``mask`` *or* ``array.mask``.
 
     Other Parameters
     ----------------
     fft_pad : bool, optional
-        Default on.  Zero-pad image to the nearest 2^n
+        Default on.  Zero-pad image to the nearest 2^n.  With
+        ``boundary='wrap'``, this will be disabled.
     psf_pad : bool, optional
-        Default off.  Zero-pad image to be at least the sum of the image sizes
-        (in order to avoid edge-wrapping when smoothing)
+        Zero-pad image to be at least the sum of the image sizes to avoid
+        edge-wrapping when smoothing.  This is enabled by default with
+        ``boundary='fill'``, but it can be overridden with a boolean option.
+        ``boundary='wrap'`` and ``psf_pad=True`` are not compatible.
     crop : bool, optional
         Default on.  Return an image of the size of the largest input image.
         If the images are asymmetric in opposite directions, will return the
@@ -329,6 +372,12 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
         If crop is not set, returns the image, but with the fft-padded size
         instead of the input size
 
+    Notes
+    -----
+        With psf_pad=True and a large PSF, the resulting data can become very
+        large and consume a lot of memory.  See Issue
+        https://github.com/astropy/astropy/pull/4366 for further detail.
+
     Examples
     --------
     >>> convolve_fft([1, 0, 3], [1, 1, 1])
@@ -389,29 +438,35 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
     kernshape = kernel.shape
 
     array_size_B = (np.product(arrayshape, dtype=np.int64) *
-                    np.dtype(complex_dtype).itemsize)
-    if array_size_B > 1024**3 and not allow_huge:
-        raise ValueError("Size Error: Arrays will be %s.  Use "
+                    np.dtype(complex_dtype).itemsize)*u.byte
+    if array_size_B > 1*u.GB and not allow_huge:
+        raise ValueError("Size Error: Arrays will be {}.  Use "
                          "allow_huge=True to override this exception."
-                         % human_file_size(array_size_B))
+                         .format(human_file_size(array_size_B.to(u.byte).value)))
 
-    # mask catching - masks must be turned into NaNs for use later
+    # mask catching - masks must be turned into NaNs for use later in the image
     if np.ma.is_masked(array):
-        mask = array.mask
+        mamask = array.mask
         array = np.array(array)
-        array[mask] = np.nan
+        array[mamask] = np.nan
+    elif mask is not None:
+        # copying here because we have to mask it below.  But no need to copy
+        # if mask is None because we won't modify it.
+        array = np.array(array)
+    if mask is not None:
+        # mask != 0 yields a bool mask for all ints/floats/bool
+        array[mask != 0] = np.nan
+    # the *kenrel* doesn't support NaN interpolation, so instead we just fill it
     if np.ma.is_masked(kernel):
-        mask = kernel.mask
-        kernel = np.array(kernel)
-        kernel[mask] = np.nan
+        kernel = kernel.filled(fill_value)
 
     # NaN and inf catching
-    nanmaskarray = np.isnan(array) + np.isinf(array)
+    nanmaskarray = np.isnan(array) | np.isinf(array)
     array[nanmaskarray] = 0
-    nanmaskkernel = np.isnan(kernel) + np.isinf(kernel)
+    nanmaskkernel = np.isnan(kernel) | np.isinf(kernel)
     kernel[nanmaskkernel] = 0
-    if ((nanmaskarray.sum() > 0 or nanmaskkernel.sum() > 0) and
-            not interpolate_nan and not quiet):
+    if (not interpolate_nan and not quiet and (np.any(nanmaskarray) or
+                                               np.any(nanmaskkernel))):
         warnings.warn("NOT ignoring NaN values even though they are present "
                       " (they are treated as 0)", AstropyUserWarning)
 
@@ -442,12 +497,27 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
                       "equivalent to the convolve boundary='fill'.  There is "
                       "no FFT equivalent to convolve's "
                       "zero-if-kernel-leaves-boundary", AstropyUserWarning)
-        psf_pad = True
+        if psf_pad is None:
+            psf_pad = True
+        if fft_pad is None:
+            fft_pad = True
     elif boundary == 'fill':
         # create a boundary region at least as large as the kernel
-        psf_pad = True
+        if psf_pad is False:
+            warnings.warn("psf_pad was set to {0}, which overrides the "
+                          "boundary='fill' setting.".format(psf_pad),
+                          AstropyUserWarning)
+        else:
+            psf_pad = True
+        if fft_pad is None:
+            # default is 'True' according to the docstring
+            fft_pad = True
     elif boundary == 'wrap':
+        if psf_pad:
+            raise ValueError("With boundary='wrap', psf_pad cannot be enabled.")
         psf_pad = False
+        if fft_pad:
+            raise ValueError("With boundary='wrap', fft_pad cannot be enabled.")
         fft_pad = False
         fill_value = 0  # force zero; it should not be used
     elif boundary == 'extend':
@@ -474,6 +544,14 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0, crop=True,
             newshape = np.array([np.max([imsh, kernsh])
                                  for imsh, kernsh in zip(arrayshape, kernshape)])
 
+    # perform a second check after padding
+    array_size_C = (np.product(newshape, dtype=np.int64) *
+                    np.dtype(complex_dtype).itemsize)*u.byte
+    if array_size_C > 1*u.GB and not allow_huge:
+        raise ValueError("Size Error: Arrays will be {}.  Use "
+                         "allow_huge=True to override this exception."
+                         .format(human_file_size(array_size_C.to(u.byte).value)))
+
     # For future reference, this can be used to predict "almost exactly"
     # how much *additional* memory will be used.
     # size * (array + kernel + kernelfft + arrayfft +
