@@ -201,7 +201,7 @@ class TestKernels(object):
         """
         gauss = Gaussian1DKernel(3)
         gauss_new = number * gauss
-        assert type(gauss_new) == Gaussian1DKernel
+        assert type(gauss_new) is Gaussian1DKernel
 
     @pytest.mark.parametrize(('number'), NUMS)
     def test_rmultiply_scalar_type(self, number):
@@ -210,23 +210,23 @@ class TestKernels(object):
         """
         gauss = Gaussian1DKernel(3)
         gauss_new = gauss * number
-        assert type(gauss_new) == Gaussian1DKernel
+        assert type(gauss_new) is Gaussian1DKernel
 
     def test_multiply_kernel1d(self):
-        """Test that multipling two 1D kernels raises an exception."""
+        """Test that multiplying two 1D kernels raises an exception."""
         gauss = Gaussian1DKernel(3)
         with pytest.raises(Exception):
             gauss * gauss
 
     def test_multiply_kernel2d(self):
-        """Test that multipling two 2D kernels raises an exception."""
+        """Test that multiplying two 2D kernels raises an exception."""
         gauss = Gaussian2DKernel(3)
         with pytest.raises(Exception):
             gauss * gauss
 
     def test_multiply_kernel1d_kernel2d(self):
         """
-        Test that multipling a 1D kernel with a 2D kernel raises an
+        Test that multiplying a 1D kernel with a 2D kernel raises an
         exception.
         """
         with pytest.raises(Exception):
@@ -292,7 +292,7 @@ class TestKernels(object):
         Check if CustomKernel works with lists.
         """
         custom = CustomKernel([1, 1, 1, 1, 1])
-        assert custom.is_bool == True
+        assert custom.is_bool is True
 
     def test_custom_2D_kernel_list(self):
         """
@@ -301,7 +301,7 @@ class TestKernels(object):
         custom = CustomKernel([[1, 1, 1],
                                [1, 1, 1],
                                [1, 1, 1]])
-        assert custom.is_bool == True
+        assert custom.is_bool is True
 
     def test_custom_1D_kernel_zerosum(self):
         """
@@ -412,7 +412,7 @@ class TestKernels(object):
         box.normalize()
         assert_almost_equal(box._kernel_sum, 1., decimal=12)
 
-        # Check seperability
+        # Check separability
         assert box.separable
 
     @pytest.mark.parametrize(('kernel_type', 'mode'), list(itertools.product(KERNEL_TYPES, MODES)))
@@ -473,16 +473,6 @@ class TestKernels(object):
             kernel = CustomKernel(np.ones(3))
             kernel.normalize(mode='invalid')
 
-    def test_kernel_normalization_large(self, recwarn):
-        """
-        Test that a warning is issued when the inverse normalization factor
-        is large.
-        """
-        kernel = CustomKernel(np.ones(3) * 1.e-3)
-        kernel.normalize()
-        w = recwarn.pop(AstropyUserWarning)
-        assert issubclass(w.category, AstropyWarning)
-
     def test_kernel1d_int_size(self):
         """
         Test that an error is raised if ``Kernel1D`` ``x_size`` is not
