@@ -5,6 +5,7 @@ from __future__ import (absolute_import, division, print_function,
 import numpy as np
 
 from ..modeling.core import FittableModel, custom_model
+from ..extern.six.moves import range
 
 __all__ = ['discretize_model']
 
@@ -84,10 +85,13 @@ def discretize_model(model, x_range, y_range=None, mode='center', factor=10):
         instances of `~astropy.modeling.FittableModel` are passed to
         `~astropy.modeling.custom_model` and then evaluated.
     x_range : tuple
-        x range in which the model is evaluated.
+        x range in which the model is evaluated. The difference between the
+        upper an lower limit must be a whole number, so that the output array
+        size is well defined.
     y_range : tuple, optional
-        y range in which the model is evaluated.
-        Necessary only for 2D models.
+        y range in which the model is evaluated. The difference between the
+        upper an lower limit must be a whole number, so that the output array
+        size is well defined. Necessary only for 2D models.
     mode : str, optional
         One of the following modes:
             * ``'center'`` (default)
@@ -146,6 +150,15 @@ def discretize_model(model, x_range, y_range=None, mode='center', factor=10):
     if ndim > 2:
         raise ValueError('discretize_model only supports 1-d and 2-d models.')
 
+    if not float(np.diff(x_range)).is_integer():
+        raise ValueError("The difference between the upper an lower limit of"
+                         " 'x_range' must be a whole number.")
+
+    if y_range:
+        if not float(np.diff(y_range)).is_integer():
+            raise ValueError("The difference between the upper an lower limit of"
+                             " 'y_range' must be a whole number.")
+
     if ndim == 2 and y_range is None:
         raise ValueError("y range not specified, but model is 2-d")
     if ndim == 1 and y_range is not None:
