@@ -17,8 +17,6 @@ from ..extern import six
 from . import angle_utilities as util
 from .. import units as u
 from ..utils import isiterable
-from ..utils.compat import NUMPY_LT_1_7
-
 
 __all__ = ['Angle', 'Latitude', 'Longitude']
 
@@ -31,7 +29,7 @@ dms_tuple = namedtuple('dms_tuple', ('d', 'm', 's'))
 signed_dms_tuple = namedtuple('signed_dms_tuple', ('sign', 'd', 'm', 's'))
 
 
-class Angle(u.Quantity):
+class Angle(u.SpecificTypeQuantity):
     """
     One or more angular value(s) with units equivalent to radians or degrees.
 
@@ -51,6 +49,7 @@ class Angle(u.Quantity):
       Angle('-1h2m3s')
       Angle('-1h2.5m')
       Angle('-1:2.5', unit=u.deg)
+      Angle((10, 11, 12), unit='hourangle')  # (h, m, s)
       Angle((-1, 2, 3), unit=u.deg)  # (d, m, s)
       Angle(10.2345 * u.deg)
       Angle(Angle(10.2345 * u.deg))
@@ -83,93 +82,58 @@ class Angle(u.Quantity):
     `~astropy.units.UnitsError`
         If a unit is not provided or it is not an angular unit.
     """
+    _equivalent_unit = u.radian
     _include_easy_conversion_members = True
 
     def __new__(cls, angle, unit=None, dtype=None, copy=True):
-        unit = cls._convert_unit_to_angle_unit(unit)
-        if (unit is not None and not unit.is_equivalent(u.radian)):
-            raise u.UnitsError("Requested unit {0} is not convertible to an "
-                               "angle".format(unit))
 
-        if isinstance(angle, u.Quantity):
-            # This includes Angle subclasses as well
+        if not isinstance(angle, u.Quantity):
             if unit is not None:
-                angle = angle.to(unit).value
-            else:
-                unit = angle.unit
-                unit = cls._convert_unit_to_angle_unit(unit)
-                if not unit.is_equivalent(u.radian):
-                    raise u.UnitsError(
-                        "Given quantity {0} is not convertible to an "
-                        "angle".format(angle))
+                unit = cls._convert_unit_to_angle_unit(u.Unit(unit))
 
-                angle = angle.value
-        else:
-            # this does nothing if it's not a tuple
-            angle = cls._tuple_to_float(angle, unit)
-
-            if isinstance(angle, six.string_types):
-                angle, new_unit = util.parse_angle(angle, unit)
-                if new_unit is not None and unit is None:
-                    unit = new_unit
+            if isinstance(angle, tuple):
                 angle = cls._tuple_to_float(angle, unit)
-                if new_unit is not None and unit is not None and new_unit != unit:
-                    angle = new_unit.to(unit, angle)
+
+            elif isinstance(angle, six.string_types):
+                angle, angle_unit = util.parse_angle(angle, unit)
+                if angle_unit is None:
+                    angle_unit = unit
+
+                if isinstance(angle, tuple):
+                    angle = cls._tuple_to_float(angle, angle_unit)
+
+                if angle_unit is not unit:
+                    # Possible conversion to `unit` will be done below.
+                    angle = u.Quantity(angle, angle_unit, copy=False)
+
             elif (isiterable(angle) and
                   not (isinstance(angle, np.ndarray) and
                        angle.dtype.kind not in 'SUVO')):
-                angle = [Angle(x, unit) for x in angle]
-                if unit is None:
-                    unit = angle[0].unit
-                angle = [x.to(unit) for x in angle]
+                angle = [Angle(x, unit, copy=False) for x in angle]
 
-        self = super(Angle, cls).__new__(cls, angle, unit, dtype=dtype,
+        return super(Angle, cls).__new__(cls, angle, unit, dtype=dtype,
                                          copy=copy)
 
-        if self.unit is u.dimensionless_unscaled:
-            raise u.UnitsError("No unit was given - must be some kind of angle")
-        elif not self.unit.is_equivalent(u.radian):
-            raise u.UnitsError("Unit {0} is not an angle".format(self.unit))
-
-        if self.dtype.kind not in 'iuf':
-                raise TypeError("Unsupported dtype for "
-                                "Angle:'{0}'".format(angle.dtype))
-
-        return self
-
     @staticmethod
     def _tuple_to_float(angle, unit):
         """
         Converts an angle represented as a 3-tuple or 2-tuple into a floating
         point number in the given unit.
         """
-        if isinstance(angle, tuple):
-            # TODO: Numpy array of tuples?
-            if unit is u.hourangle:
-                angle = util.hms_to_hours(*angle)
-            elif unit is u.degree:
-                angle = util.dms_to_degrees(*angle)
-            else:
-                raise u.UnitsError(
-                    "Can not parse '{0}' as unit '{1}'".format(
-                        angle, unit))
-        return angle
-
+        # TODO: Numpy array of tuples?
+        if unit == u.hourangle:
+            return util.hms_to_hours(*angle)
+        elif unit == u.degree:
+            return util.dms_to_degrees(*angle)
+        else:
+            raise u.UnitsError("Can not parse '{0}' as unit '{1}'"
+                               .format(angle, unit))
     @staticmethod
     def _convert_unit_to_angle_unit(unit):
-        if unit is not None:
-            unit = u.Unit(unit)
+        return u.hourangle if unit is u.hour else unit
 
-            if unit is u.hour:
-                unit = u.hourangle
-        return unit
-
-    def __quantity_subclass__(self, unit):
-        unit = self._convert_unit_to_angle_unit(unit)
-        if unit.is_equivalent(u.radian):
-            return Angle, True
-        else:
-            return super(Angle, self).__quantity_subclass__(unit)[0], False
+    def _set_unit(self, unit):
+        super(Angle, self)._set_unit(self._convert_unit_to_angle_unit(unit))
 
     @property
     def hour(self):
@@ -277,7 +241,8 @@ class Angle(u.Quantity):
         """
         if unit is None:
             unit = self.unit
-        unit = self._convert_unit_to_angle_unit(unit)
+        else:
+            unit = self._convert_unit_to_angle_unit(u.Unit(unit))
 
         separators = {
             None: {
@@ -370,18 +335,8 @@ class Angle(u.Quantity):
                 s = '${0}$'.format(s)
             return s
 
-        if NUMPY_LT_1_7 and not np.isscalar(values):  # pragma: no cover
-            format_ufunc = np.vectorize(do_format, otypes=[np.object])
-            # In Numpy 1.6, unicode output is broken.  vectorize always seems to
-            # yield U2 even if you tell it something else.  So we convert in
-            # a second step with 60 chars, on the theory that you'll never want
-            # better than what double-precision decimals give, which end up
-            # around that many characters.
-            result = format_ufunc(values).astype('U60')
-        else:
-            #for newer Numpy versions, this just works as you would expect
-            format_ufunc = np.vectorize(do_format, otypes=['U'])
-            result = format_ufunc(values)
+        format_ufunc = np.vectorize(do_format, otypes=['U'])
+        result = format_ufunc(values)
 
         if result.ndim == 0:
             result = result[()]
@@ -485,13 +440,8 @@ class Angle(u.Quantity):
         else:
             # Need to do a magic incantation to convert to str.  Regular str
             # or array2string causes all backslashes to get doubled.
-            if NUMPY_LT_1_7:
-                # Except that numpy 1.6 doesn't do formatter... so instead we
-                # just replace all double-backslashes with one.
-                return str(self.to_string(format='latex')).replace('\\\\', '\\')
-            else:
-                return np.array2string(self.to_string(format='latex'),
-                                       formatter={'str_kind': lambda x: x})
+            return np.array2string(self.to_string(format='latex'),
+                                   formatter={'str_kind': lambda x: x})
 
 
 class Latitude(Angle):
@@ -505,7 +455,7 @@ class Latitude(Angle):
       -90.0 * u.deg <= angle(s) <= +90.0 * u.deg
 
     Any attempt to set a value outside that range will result in a
-    `~.exceptions.ValueError`.
+    `ValueError`.
 
     The input angle(s) can be specified either as an array, list,
     scalar, tuple (see below), string,
@@ -638,14 +588,17 @@ class Longitude(Angle):
     """
 
     _wrap_angle = None
+    _default_wrap_angle = Angle(360 * u.deg)
 
     def __new__(cls, angle, unit=None, wrap_angle=None, **kwargs):
         # Forbid creating a Long from a Lat.
         if isinstance(angle, Latitude):
-            raise TypeError("A Longitude angle cannot be created from a Latitude angle")
+            raise TypeError("A Longitude angle cannot be created from "
+                            "a Latitude angle.")
         self = super(Longitude, cls).__new__(cls, angle, unit=unit, **kwargs)
-        self.wrap_angle = (wrap_angle if wrap_angle is not None
-                           else getattr(angle, 'wrap_angle', 360 * u.deg))
+        if wrap_angle is None:
+            wrap_angle = getattr(angle, 'wrap_angle', self._default_wrap_angle)
+        self.wrap_angle = wrap_angle
         return self
 
     def __setitem__(self, item, value):
@@ -685,7 +638,8 @@ class Longitude(Angle):
 
     def __array_finalize__(self, obj):
         super(Longitude, self).__array_finalize__(obj)
-        self._wrap_angle = getattr(obj, '_wrap_angle', 360 * u.deg)
+        self._wrap_angle = getattr(obj, '_wrap_angle',
+                                   self._default_wrap_angle)
 
     # Any calculation should drop to Angle
     def __array_wrap__(self, obj, context=None):
@@ -697,8 +651,21 @@ class Longitude(Angle):
         return obj
 
 #<----------------------------------Rotations--------------------------------->
+# The main routines have been moved to matrix_utilities.  The definitions here
+# are for backward compatibility.
+from . import matrix_utilities
+from ..utils.decorators import deprecated
 
-
+_DEPRECATION_MESSAGE="""
+Numpy matrix instances are no longer used to represent rotation matrices, since
+they do not allow one to represent stacks of matrices. Instead, plain arrays
+are used. Instead of %(func)s, use %(alternative)s.
+For matrix multiplication and tranposes, it is suggested to use
+:func:`~astropy.coordinates.matrix_utilities.matrix_product` and
+:func:`~astropy.coordinates.matrix_utilities.matrix_transpose`, respectively.
+"""
+@deprecated(since='1.3', message=_DEPRECATION_MESSAGE, alternative=':func:'
+            '`~astropy.coordinates.matrix_utilities.rotation_matrix`')
 def rotation_matrix(angle, axis='z', unit=None):
     """
     Generate a 3x3 cartesian rotation matrix in for rotation about
@@ -721,43 +688,14 @@ def rotation_matrix(angle, axis='z', unit=None):
 
     Returns
     -------
-    rmat: `numpy.matrix`
+    rmat : `numpy.matrix`
         A unitary rotation matrix.
     """
-    if unit is None:
-        unit = u.degree
-
-    angle = Angle(angle, unit=unit)
-
-    s = np.sin(angle)
-    c = np.cos(angle)
-
-    # use optimized implementations for x/y/z
-    if axis == 'z':
-        return np.matrix(((c, s, 0),
-                          (-s, c, 0),
-                          (0, 0, 1)))
-    elif axis == 'y':
-        return np.matrix(((c, 0, -s),
-                          (0, 1, 0),
-                          (s, 0, c)))
-    elif axis == 'x':
-        return np.matrix(((1, 0, 0),
-                          (0, c, s),
-                          (0, -s, c)))
-    else:
-        axis = np.asarray(axis)
-        axis = axis / np.sqrt((axis * axis).sum())
-
-        R = np.diag((c, c, c))
-        R += np.outer(axis, axis) * (1. - c)
-        axis *= s
-        R += np.array([[0., axis[2], -axis[1]],
-                       [-axis[2], 0., axis[0]],
-                       [axis[1], -axis[0], 0.]])
-        return R.view(np.matrix)
+    return matrix_utilities.rotation_matrix(angle, axis, unit).view(np.matrix)
 
 
+@deprecated(since='1.3', message=_DEPRECATION_MESSAGE, alternative=':func:'
+            '`~astropy.coordinates.matrix_utilities.angle_axis`')
 def angle_axis(matrix):
     """
     Computes the angle of rotation and the rotation axis for a given rotation
@@ -777,11 +715,4 @@ def angle_axis(matrix):
         The (normalized) axis of rotation for this matrix.
     """
     m = np.asmatrix(matrix)
-    if m.shape != (3, 3):
-        raise ValueError('matrix is not 3x3')
-
-    axis = np.array((m[2, 1] - m[1, 2], m[0, 2] - m[2, 0], m[1, 0] - m[0, 1]))
-    r = np.sqrt((axis * axis).sum())
-    angle = np.arctan2(r, np.trace(m) - 1)
-
-    return Angle(angle, u.radian), -axis / r
+    return matrix_utilities.angle_axis(m.view(np.ndarray))
