@@ -9,18 +9,23 @@ from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
 # Standard library
+import abc
+import copy
 import inspect
-from copy import deepcopy
-from collections import namedtuple, OrderedDict
+from collections import namedtuple, OrderedDict, defaultdict
 
 # Dependencies
 import numpy as np
 
 # Project
 from ..utils.compat.misc import override__dir__
+from ..utils.compat.numpy import broadcast_to as np_broadcast_to
+from ..utils.decorators import lazyproperty
 from ..extern import six
+from ..extern.six.moves import zip
 from .. import units as u
-from ..utils import OrderedDescriptor, OrderedDescriptorContainer
+from ..utils import (OrderedDescriptor, OrderedDescriptorContainer,
+                     ShapedLikeNDArray, check_broadcast)
 from .transformations import TransformGraph
 from .representation import (BaseRepresentation, CartesianRepresentation,
                              SphericalRepresentation,
@@ -31,7 +36,7 @@ from .representation import (BaseRepresentation, CartesianRepresentation,
 __all__ = ['BaseCoordinateFrame', 'frame_transform_graph', 'GenericFrame',
            'FrameAttribute', 'TimeFrameAttribute', 'QuantityFrameAttribute',
            'EarthLocationAttribute', 'RepresentationMapping',
-           'CoordinateAttribute']
+           'CartesianRepresentationFrameAttribute', 'CoordinateAttribute']
 
 
 # the graph used for all transformations between frames
@@ -56,7 +61,11 @@ def _get_repr_cls(value):
     return value
 
 
-class FrameMeta(OrderedDescriptorContainer):
+# Need to subclass ABCMeta as well, so that this meta class can be combined
+# with ShapedLikeNDArray below (which is an ABC); without it, one gets
+# "TypeError: metaclass conflict: the metaclass of a derived class must be a
+#  (non-strict) subclass of the metaclasses of all its bases"
+class FrameMeta(OrderedDescriptorContainer, abc.ABCMeta):
     def __new__(mcls, name, bases, members):
         if 'default_representation' in members:
             default_repr = members.pop('default_representation')
@@ -102,7 +111,7 @@ class FrameMeta(OrderedDescriptorContainer):
                                    default_repr)
         mcls.readonly_prop_factory(members,
                                    'frame_specific_representation_info',
-                                   deepcopy(repr_info))
+                                   copy.deepcopy(repr_info))
 
         # now set the frame name as lower-case class name, if it isn't explicit
         if 'name' not in members:
@@ -199,19 +208,36 @@ class FrameAttribute(OrderedDescriptor):
         return value, False
 
     def __get__(self, instance, frame_cls=None):
-        out = None
-
-        if instance is not None:
-            out = getattr(instance, '_' + self.name, None)
-            if out is None and self.default is None:
-                out = getattr(instance, self.secondary_attribute, None)
-
-        if out is None:
+        if instance is None:
             out = self.default
+        else:
+            out = getattr(instance, '_' + self.name, self.default)
+            if out is None:
+                out = getattr(instance, self.secondary_attribute, self.default)
 
         out, converted = self.convert_input(out)
-        if instance is not None and converted:
-            setattr(instance, '_' + self.name, out)
+        if instance is not None:
+            instance_shape = getattr(instance, 'shape', None)
+            if instance_shape is not None and (getattr(out, 'size', 1) > 1 and
+                                               out.shape != instance_shape):
+                # If the shapes do not match, try broadcasting.
+                try:
+                    if isinstance(out, ShapedLikeNDArray):
+                        out = out._apply(np_broadcast_to, shape=instance_shape,
+                                         subok=True)
+                    else:
+                        out = np_broadcast_to(out, instance_shape, subok=True)
+                except ValueError:
+                    # raise more informative exception.
+                    raise ValueError(
+                        "attribute {0} should be scalar or have shape {1}, "
+                        "but is has shape {2} and could not be broadcast."
+                        .format(self.name, instance_shape, out.shape))
+
+                converted = True
+
+            if converted:
+                setattr(instance, '_' + self.name, out)
 
         return out
 
@@ -276,6 +302,68 @@ class TimeFrameAttribute(FrameAttribute):
         return out, converted
 
 
+class CartesianRepresentationFrameAttribute(FrameAttribute):
+    """
+    A frame attribute that is a CartesianRepresentation with specified units.
+
+    Parameters
+    ----------
+    default : object
+        Default value for the attribute if not provided
+    secondary_attribute : str
+        Name of a secondary instance attribute which supplies the value if
+        ``default is None`` and no value was supplied during initialization.
+    unit : unit object or None
+        Name of a unit that the input will be converted into. If None, no
+        unit-checking or conversion is performed
+    """
+    def __init__(self, default=None, secondary_attribute='', unit=None):
+        super(CartesianRepresentationFrameAttribute, self).__init__(default, secondary_attribute)
+        self.unit = unit
+
+    def convert_input(self, value):
+        """
+        Checks that the input is a CartesianRepresentation with the correct
+        unit, or the special value ``[0, 0, 0]``.
+
+        Parameters
+        ----------
+        value : object
+            Input value to be converted.
+
+        Returns
+        -------
+        out, converted : correctly-typed object, boolean
+            Tuple consisting of the correctly-typed object and a boolean which
+            indicates if conversion was actually performed.
+
+        Raises
+        ------
+        ValueError
+            If the input is not valid for this attribute.
+        """
+        if (isinstance(value, list) and len(value) == 3 and
+                all(v == 0 for v in value) and self.unit is not None):
+            return CartesianRepresentation(np.zeros(3) * self.unit), True
+        else:
+            # is it a CartesianRepresentation with correct unit?
+            if hasattr(value, 'xyz') and value.xyz.unit == self.unit:
+                return value, False
+
+            converted = True
+            # if it's a CartesianRepresentation, get the xyz Quantity
+            value = getattr(value, 'xyz', value)
+            if not hasattr(value, 'unit'):
+                raise TypeError('tried to set a CartesianRepresentationFrameAttribute with '
+                                'something that does not have a unit.')
+
+            value = value.to(self.unit)
+
+            # now try and make a CartesianRepresentation.
+            cartrep = CartesianRepresentation(value, copy=False)
+            return cartrep, converted
+
+
 class QuantityFrameAttribute(FrameAttribute):
     """
     A frame attribute that is a quantity with specified units and shape
@@ -320,22 +408,19 @@ class QuantityFrameAttribute(FrameAttribute):
         ValueError
             If the input is not valid for this attribute.
         """
-        if np.all(value == 0) and self.unit is not None and self.unit is not None:
+        if np.all(value == 0) and self.unit is not None:
             return u.Quantity(np.zeros(self.shape), self.unit), True
         else:
-            converted = True
-            if not (hasattr(value, 'unit') ):
+            if not hasattr(value, 'unit'):
                 raise TypeError('Tried to set a QuantityFrameAttribute with '
                                 'something that does not have a unit.')
             oldvalue = value
-            value = u.Quantity(oldvalue, copy=False).to(self.unit)
+            value = u.Quantity(oldvalue, self.unit, copy=False)
             if self.shape is not None and value.shape != self.shape:
                 raise ValueError('The provided value has shape "{0}", but '
                                  'should have shape "{1}"'.format(value.shape,
                                                                   self.shape))
-            if (oldvalue.unit == value.unit and hasattr(oldvalue, 'value') and
-                np.all(oldvalue.value == value.value)):
-                converted = False
+            converted = oldvalue is not value
             return value, converted
 
 
@@ -470,9 +555,8 @@ class RepresentationMapping(_RepresentationMappingBase):
                                                          framename,
                                                          defaultunit)
 
-
 @six.add_metaclass(FrameMeta)
-class BaseCoordinateFrame(object):
+class BaseCoordinateFrame(ShapedLikeNDArray):
     """
     The base class for coordinate frames.
 
@@ -526,21 +610,6 @@ class BaseCoordinateFrame(object):
         # if not set below, this is a frame with no data
         representation_data = None
 
-        for fnm, fdefault in self.get_frame_attr_names().items():
-            # Read-only frame attributes are defined as FrameAttribue
-            # descriptors which are not settable, so set 'real' attributes as
-            # the name prefaced with an underscore.
-
-            if fnm in kwargs:
-                value = kwargs.pop(fnm)
-                setattr(self, '_' + fnm, value)
-            else:
-                setattr(self, '_' + fnm, fdefault)
-                self._attr_names_with_defaults.append(fnm)
-
-            # Validate input by getting the attribute here.
-            getattr(self, fnm)
-
         args = list(args)  # need to be able to pop them
         if (len(args) > 0) and (isinstance(args[0], BaseRepresentation) or
                                 args[0] is None):
@@ -576,18 +645,82 @@ class BaseCoordinateFrame(object):
             raise TypeError(
                 '{0}.__init__ had {1} remaining unhandled arguments'.format(
                     self.__class__.__name__, len(args)))
+
+        self._data = representation_data
+
+        values = {}
+        for fnm, fdefault in self.get_frame_attr_names().items():
+            # Read-only frame attributes are defined as FrameAttribue
+            # descriptors which are not settable, so set 'real' attributes as
+            # the name prefaced with an underscore.
+
+            if fnm in kwargs:
+                value = kwargs.pop(fnm)
+                setattr(self, '_' + fnm, value)
+                # Validate attribute by getting it.  If the instance has data,
+                # this also checks its shape is OK.  If not, we do it below.
+                values[fnm] = getattr(self, fnm)
+            else:
+                setattr(self, '_' + fnm, fdefault)
+                self._attr_names_with_defaults.append(fnm)
+
         if kwargs:
             raise TypeError(
                 'Coordinate frame got unexpected keywords: {0}'.format(
                     list(kwargs)))
 
-        self._data = representation_data
-
-        # We do ``is not None`` because self._data might evaluate to false for
+        # We do ``is None`` because self._data might evaluate to false for
         # empty arrays or data == 0
-        if self._data is not None:
-            self._rep_cache = dict()
-            self._rep_cache[self._data.__class__.__name__, False] = self._data
+        if self._data is None:
+            # No data: we still need to check that any non-scalar attributes
+            # have consistent shapes. Collect them for all attributes with
+            # size > 1 (which should be array-like and thus have a shape).
+            shapes = {fnm: value.shape for fnm, value in values.items()
+                      if getattr(value, 'size', 1) > 1}
+            if shapes:
+                if len(shapes) > 1:
+                    try:
+                        self._no_data_shape = check_broadcast(*shapes.values())
+                    except ValueError:
+                        raise ValueError(
+                            "non-scalar attributes with inconsistent "
+                            "shapes: {0}".format(shapes))
+
+                    # Above, we checked that it is possible to broadcast all
+                    # shapes.  By getting and thus validating the attributes,
+                    # we verify that the attributes can in fact be broadcast.
+                    for fnm in shapes:
+                        getattr(self, fnm)
+                else:
+                    self._no_data_shape = shapes.popitem()[1]
+
+            else:
+                self._no_data_shape = ()
+        else:
+            # Set up representation cache.
+            self.cache['representation'][self._data.__class__.__name__, False] = self._data
+
+    @lazyproperty
+    def cache(self):
+        """
+        Cache for this frame, a dict.  It stores anything that should be
+        computed from the coordinate data (*not* from the frame attributes).
+        This can be used in functions to store anything that might be
+        expensive to compute but might be re-used by some other function.
+        E.g.::
+
+            if 'user_data' in myframe.cache:
+                data = myframe.cache['user_data']
+            else:
+                myframe.cache['user_data'] = data = expensive_func(myframe.lat)
+
+        If in-place modifications are made to the frame data, the cache should
+        be cleared::
+
+            myframe.cache.clear()
+
+        """
+        return defaultdict(dict)
 
     @property
     def data(self):
@@ -608,22 +741,28 @@ class BaseCoordinateFrame(object):
         """
         return self._data is not None
 
+    @property
+    def shape(self):
+        return self.data.shape if self.has_data else self._no_data_shape
+
+    # We have to override the ShapedLikeNDArray definitions, since our shape
+    # does not have to be that of the data.
     def __len__(self):
         return len(self.data)
 
     def __nonzero__(self):  # Py 2.x
-        return self.isscalar or len(self) != 0
+        return self.has_data and self.size > 0
 
     def __bool__(self):  # Py 3.x
-        return self.isscalar or len(self) != 0
+        return self.has_data and self.size > 0
 
     @property
-    def shape(self):
-        return self.data.shape
+    def size(self):
+        return self.data.size
 
     @property
     def isscalar(self):
-        return self.data.isscalar
+        return self.has_data and self.data.isscalar
 
     @classmethod
     def get_frame_attr_names(cls):
@@ -775,17 +914,17 @@ class BaseCoordinateFrame(object):
         >>> coord = SkyCoord(0*u.deg, 0*u.deg)
         >>> coord.represent_as(CartesianRepresentation)
         <CartesianRepresentation (x, y, z) [dimensionless]
-                (1.0, 0.0, 0.0)>
+                ( 1.,  0.,  0.)>
 
         >>> coord.representation = CartesianRepresentation
         >>> coord
         <SkyCoord (ICRS): (x, y, z) [dimensionless]
-            (1.0, 0.0, 0.0)>
+            ( 1.,  0.,  0.)>
         """
         new_representation = _get_repr_cls(new_representation)
 
-        cached_repr = self._rep_cache.get((new_representation.__name__,
-                                           in_frame_units))
+        cached_repr = self.cache['representation'].get((new_representation.__name__,
+                                                        in_frame_units))
         if not cached_repr:
             data = self.data.represent_as(new_representation)
 
@@ -800,9 +939,9 @@ class BaseCoordinateFrame(object):
                         datakwargs[comp] = datakwargs[comp].to(new_attr_unit)
                 data = data.__class__(copy=False, **datakwargs)
 
-            self._rep_cache[new_representation.__name__, in_frame_units] = data
+            self.cache['representation'][new_representation.__name__, in_frame_units] = data
 
-        return self._rep_cache[new_representation.__name__, in_frame_units]
+        return self.cache['representation'][new_representation.__name__, in_frame_units]
 
     def transform_to(self, new_frame):
         """
@@ -936,7 +1075,8 @@ class BaseCoordinateFrame(object):
         """
         if self.__class__ == other.__class__:
             for frame_attr_name in self.get_frame_attr_names():
-                if getattr(self, frame_attr_name) != getattr(other, frame_attr_name):
+                if np.any(getattr(self, frame_attr_name) !=
+                          getattr(other, frame_attr_name)):
                     return False
             return True
         elif not isinstance(other, BaseCoordinateFrame):
@@ -998,13 +1138,60 @@ class BaseCoordinateFrame(object):
         return ', '.join([attrnm + '=' + str(getattr(self, attrnm))
                           for attrnm in self.get_frame_attr_names()])
 
-    def __getitem__(self, view):
-        if self.has_data:
-            out = self.realize_frame(self.data[view])
-            out.representation = self.representation
-            return out
-        else:
-            raise ValueError('Cannot index a frame with no data')
+    def _apply(self, method, *args, **kwargs):
+        """Create a new instance, applying a method to the underlying data.
+
+        In typical usage, the method is any of the shape-changing methods for
+        `~numpy.ndarray` (``reshape``, ``swapaxes``, etc.), as well as those
+        picking particular elements (``__getitem__``, ``take``, etc.), which
+        are all defined in `~astropy.utils.misc.ShapedLikeNDArray`. It will be
+        applied to the underlying arrays in the representation (e.g., ``x``,
+        ``y``, and ``z`` for `~astropy.coordinates.CartesianRepresentation`),
+        as well as to any frame attributes that have a shape, with the results
+        used to create a new instance.
+
+        Internally, it is also used to apply functions to the above parts
+        (in particular, `~numpy.broadcast_to`).
+
+        Parameters
+        ----------
+        method : str or callable
+            If str, it is the name of a method that is applied to the internal
+            ``components``. If callable, the function is applied.
+        args : tuple
+            Any positional arguments for ``method``.
+        kwargs : dict
+            Any keyword arguments for ``method``.
+        """
+        def apply_method(value):
+            if isinstance(value, ShapedLikeNDArray):
+                return value._apply(method, *args, **kwargs)
+            else:
+                if callable(method):
+                    return method(value, *args, **kwargs)
+                else:
+                    return getattr(value, method)(*args, **kwargs)
+
+
+        data = apply_method(self.data) if self.has_data else None
+
+        frattrs = {}
+        for attr in self.get_frame_attr_names():
+            if attr not in self._attr_names_with_defaults:
+                value = getattr(self, attr)
+                if getattr(value, 'size', 1) > 1:
+                    value = apply_method(value)
+                elif method == 'copy' or method == 'flatten':
+                    # flatten should copy also for a single element array, but
+                    # we cannot use it directly for array scalars, since it
+                    # always returns a one-dimensional array. So, just copy.
+                    value = copy.copy(value)
+
+                frattrs[attr] = value
+
+        out = self.__class__(data, **frattrs)
+        out.representation = self.representation
+        return out
 
     @override__dir__
     def __dir__(self):
@@ -1130,12 +1317,8 @@ class BaseCoordinateFrame(object):
             raise ValueError('The other object does not have a distance; '
                              'cannot compute 3d separation.')
 
-        dx = self.cartesian.x - other_in_self_system.cartesian.x
-        dy = self.cartesian.y - other_in_self_system.cartesian.y
-        dz = self.cartesian.z - other_in_self_system.cartesian.z
-
-        distval = (dx.value ** 2 + dy.value ** 2 + dz.value ** 2) ** 0.5
-        return Distance(distval, dx.unit)
+        return Distance((self.cartesian -
+                         other_in_self_system.cartesian).norm())
 
     @property
     def cartesian(self):
