@@ -34,9 +34,10 @@ from .altaz import AltAz
 from .gcrs import GCRS, PrecessedGeocentric
 from .cirs import CIRS
 from .itrs import ITRS
+from .hcrs import HCRS
 from .ecliptic import GeocentricTrueEcliptic, BarycentricTrueEcliptic, HeliocentricTrueEcliptic
-
-#need to import transformations so that they get registered in the graph
+from .skyoffset import SkyOffsetFrame
+# need to import transformations so that they get registered in the graph
 from . import icrs_fk5_transforms
 from . import fk4_fk5_transforms
 from . import galactic_transforms
@@ -48,9 +49,11 @@ from . import ecliptic_transforms
 
 # we define an __all__ because otherwise the transformation modules get included
 __all__ = ['ICRS', 'FK5', 'FK4', 'FK4NoETerms', 'Galactic', 'Galactocentric',
-           'Supergalactic', 'AltAz', 'GCRS', 'CIRS', 'ITRS',
+           'Supergalactic', 'AltAz', 'GCRS', 'CIRS', 'ITRS', 'HCRS',
            'PrecessedGeocentric', 'GeocentricTrueEcliptic',
-           'BarycentricTrueEcliptic', 'HeliocentricTrueEcliptic']
+           'BarycentricTrueEcliptic', 'HeliocentricTrueEcliptic',
+           'SkyOffsetFrame']
+
 
 def _make_transform_graph_docs():
     """
@@ -63,7 +66,7 @@ def _make_transform_graph_docs():
     from ..baseframe import BaseCoordinateFrame, frame_transform_graph
 
     isclass = inspect.isclass
-    coosys = [item for item in list(six.itervalues(globals()))
+    coosys = [item for item in six.itervalues(globals())
               if isclass(item) and issubclass(item, BaseCoordinateFrame)]
     graphstr = frame_transform_graph.to_dot_graph(addnodes=coosys)
 
