@@ -20,14 +20,17 @@ from .icrs import ICRS
 from .gcrs import GCRS
 from .cirs import CIRS
 from .hcrs import HCRS
-from .utils import get_jd12, aticq, atciqz
+from .utils import get_jd12, aticq, atciqz, get_cip, prepare_earth_position_vel
 
 
 # First the ICRS/CIRS related transforms
 @frame_transform_graph.transform(FunctionTransform, ICRS, CIRS)
 def icrs_to_cirs(icrs_coo, cirs_frame):
     # first set up the astrometry context for ICRS<->CIRS
-    astrom, eo = erfa.apci13(*get_jd12(cirs_frame.obstime, 'tdb'))
+    jd1, jd2 = get_jd12(cirs_frame.obstime, 'tdb')
+    x, y, s = get_cip(jd1, jd2)
+    earth_pv, earth_heliocentric = prepare_earth_position_vel(cirs_frame.obstime)
+    astrom = erfa.apci(jd1, jd2, earth_pv, earth_heliocentric, x, y, s)
 
     if icrs_coo.data.get_name() == 'unitspherical' or icrs_coo.data.to_cartesian().x.unit == u.one:
         # if no distance, just do the infinite-distance/no parallax calculation
@@ -44,11 +47,9 @@ def icrs_to_cirs(icrs_coo, cirs_frame):
         # astrometric coordinate direction and *then* run the ERFA transform for
         # no parallax/PM. This ensures reversibility and is more sensible for
         # inside solar system objects
-        newxyz = icrs_coo.cartesian.xyz
-        newxyz = np.rollaxis(newxyz, 0, newxyz.ndim) - astrom['eb'] * u.au
-        # roll xyz back to the first axis
-        newxyz = np.rollaxis(newxyz, -1, 0)
-        newcart = CartesianRepresentation(newxyz)
+        astrom_eb = CartesianRepresentation(astrom['eb'], unit=u.au,
+                                            xyz_axis=-1, copy=False)
+        newcart = icrs_coo.cartesian - astrom_eb
 
         srepr = newcart.represent_as(SphericalRepresentation)
         i_ra = srepr.lon.to(u.radian).value
@@ -70,7 +71,10 @@ def cirs_to_icrs(cirs_coo, icrs_frame):
 
     # set up the astrometry context for ICRS<->cirs and then convert to
     # astrometric coordinate direction
-    astrom, eo = erfa.apci13(*get_jd12(cirs_coo.obstime, 'tdb'))
+    jd1, jd2 = get_jd12(cirs_coo.obstime, 'tdb')
+    x, y, s = get_cip(jd1, jd2)
+    earth_pv, earth_heliocentric = prepare_earth_position_vel(cirs_coo.obstime)
+    astrom = erfa.apci(jd1, jd2, earth_pv, earth_heliocentric, x, y, s)
     i_ra, i_dec = aticq(cirs_ra, cirs_dec, astrom)
 
     if cirs_coo.data.get_name() == 'unitspherical' or cirs_coo.data.to_cartesian().x.unit == u.one:
@@ -90,12 +94,9 @@ def cirs_to_icrs(cirs_coo, icrs_frame):
                                               distance=cirs_coo.distance,
                                               copy=False)
 
-        newxyz = intermedrep.to_cartesian().xyz
-        # roll xyz to last axis and add the barycentre position
-        newxyz = np.rollaxis(newxyz, 0, newxyz.ndim) + astrom['eb'] * u.au
-        # roll xyz back to the first axis
-        newxyz = np.rollaxis(newxyz, -1, 0)
-        newrep = CartesianRepresentation(newxyz).represent_as(SphericalRepresentation)
+        astrom_eb = CartesianRepresentation(astrom['eb'], unit=u.au,
+                                            xyz_axis=-1, copy=False)
+        newrep = intermedrep + astrom_eb
 
     return icrs_frame.realize_frame(newrep)
 
@@ -118,12 +119,21 @@ def cirs_to_cirs(from_coo, to_frame):
 
 @frame_transform_graph.transform(FunctionTransform, ICRS, GCRS)
 def icrs_to_gcrs(icrs_coo, gcrs_frame):
-    # first set up the astrometry context for ICRS<->GCRS
-    pv = np.array([gcrs_frame.obsgeoloc.value,
-                   gcrs_frame.obsgeovel.value])
+    # first set up the astrometry context for ICRS<->GCRS. There are a few steps...
+    # get the position and velocity arrays for the observatory.  Need to
+    # have xyz in last dimension, and pos/vel in one-but-last.
+    # (Note could use np.stack once our minimum numpy version is >=1.10.)
+    pv = np.concatenate(
+        (gcrs_frame.obsgeoloc.get_xyz(xyz_axis=-1).value[..., np.newaxis, :],
+         gcrs_frame.obsgeovel.get_xyz(xyz_axis=-1).value[..., np.newaxis, :]),
+        axis=-2)
 
+    # find the position and velocity of earth
     jd1, jd2 = get_jd12(gcrs_frame.obstime, 'tdb')
-    astrom = erfa.apcs13(jd1, jd2, pv)
+    earth_pv, earth_heliocentric = prepare_earth_position_vel(gcrs_frame.obstime)
+
+    # get astrometry context object, astrom.
+    astrom = erfa.apcs(jd1, jd2, pv, earth_pv, earth_heliocentric)
 
     if icrs_coo.data.get_name() == 'unitspherical' or icrs_coo.data.to_cartesian().x.unit == u.one:
         # if no distance, just do the infinite-distance/no parallax calculation
@@ -140,10 +150,9 @@ def icrs_to_gcrs(icrs_coo, gcrs_frame):
         # BCRS coordinate direction and *then* run the ERFA transform for no
         # parallax/PM. This ensures reversibility and is more sensible for
         # inside solar system objects
-        newxyz = icrs_coo.cartesian.xyz
-        newxyz = np.rollaxis(newxyz, 0, newxyz.ndim) - astrom['eb'] * u.au
-        newxyz = np.rollaxis(newxyz, -1, 0)
-        newcart = CartesianRepresentation(newxyz)
+        astrom_eb = CartesianRepresentation(astrom['eb'], unit=u.au,
+                                            xyz_axis=-1, copy=False)
+        newcart = icrs_coo.cartesian - astrom_eb
 
         srepr = newcart.represent_as(SphericalRepresentation)
         i_ra = srepr.lon.to(u.radian).value
@@ -165,10 +174,15 @@ def gcrs_to_icrs(gcrs_coo, icrs_frame):
 
     # set up the astrometry context for ICRS<->GCRS and then convert to BCRS
     # coordinate direction
-    pv = np.array([gcrs_coo.obsgeoloc.value,
-                   gcrs_coo.obsgeovel.value])
+    pv = np.concatenate(
+        (gcrs_coo.obsgeoloc.get_xyz(xyz_axis=-1).value[..., np.newaxis, :],
+         gcrs_coo.obsgeovel.get_xyz(xyz_axis=-1).value[..., np.newaxis, :]),
+        axis=-2)
+
     jd1, jd2 = get_jd12(gcrs_coo.obstime, 'tdb')
-    astrom = erfa.apcs13(jd1, jd2, pv)
+
+    earth_pv, earth_heliocentric = prepare_earth_position_vel(gcrs_coo.obstime)
+    astrom = erfa.apcs(jd1, jd2, pv, earth_pv, earth_heliocentric)
 
     i_ra, i_dec = aticq(gcrs_ra, gcrs_dec, astrom)
 
@@ -189,12 +203,10 @@ def gcrs_to_icrs(gcrs_coo, icrs_frame):
                                               distance=gcrs_coo.distance,
                                               copy=False)
 
-        newxyz = intermedrep.to_cartesian().xyz
-        # roll xyz to last axis and add the heliocentre position
-        newxyz = np.rollaxis(newxyz, 0, newxyz.ndim) + astrom['eb'] * u.au
-        # roll xyz back to the first axis
-        newxyz = np.rollaxis(newxyz, -1, 0)
-        newrep = CartesianRepresentation(newxyz).represent_as(SphericalRepresentation)
+        astrom_eb = CartesianRepresentation(astrom['eb'], unit=u.au,
+                                            xyz_axis=-1, copy=False)
+        newrep = intermedrep + astrom_eb
+
     return icrs_frame.realize_frame(newrep)
 
 
@@ -224,11 +236,14 @@ def gcrs_to_hcrs(gcrs_coo, hcrs_frame):
 
     # set up the astrometry context for ICRS<->GCRS and then convert to ICRS
     # coordinate direction
-    pv = np.array([gcrs_coo.obsgeoloc.value,
-                   gcrs_coo.obsgeovel.value])
+    pv = np.concatenate(
+        (gcrs_coo.obsgeoloc.get_xyz(xyz_axis=-1).value[..., np.newaxis, :],
+         gcrs_coo.obsgeovel.get_xyz(xyz_axis=-1).value[..., np.newaxis, :]),
+        axis=-2)
 
     jd1, jd2 = get_jd12(hcrs_frame.obstime, 'tdb')
-    astrom = erfa.apcs13(jd1, jd2, pv)
+    earth_pv, earth_heliocentric = prepare_earth_position_vel(gcrs_coo.obstime)
+    astrom = erfa.apcs(jd1, jd2, pv, earth_pv, earth_heliocentric)
 
     i_ra, i_dec = aticq(gcrs_ra, gcrs_dec, astrom)
 
@@ -250,40 +265,48 @@ def gcrs_to_hcrs(gcrs_coo, hcrs_frame):
                                               distance=gcrs_coo.distance,
                                               copy=False)
 
-        newxyz = intermedrep.to_cartesian().xyz
-        # roll astrom['eh'] to the last axis and scale
-        eh = np.rollaxis(astrom['eh'], 0, astrom['eh'].ndim) * astrom['em'] * u.au
-        # roll astrom['eh'] back to the first axis
-        eh = np.rollaxis(eh, -1, 0)
+        # astrom['eh'] and astrom['em'] contain Sun to observer unit vector,
+        # and distance, respectively. Shapes are (X) and (X,3), where (X) is the
+        # shape resulting from broadcasting the shape of the times object
+        # against the shape of the pv array.
+        # broadcast em to eh and scale eh
+        eh = astrom['eh'] * astrom['em'][..., np.newaxis]
+        eh = CartesianRepresentation(eh, unit=u.au, xyz_axis=-1, copy=False)
 
-        # roll xyz to last axis and add the heliocentre position
-        newxyz = np.rollaxis(newxyz, 0, newxyz.ndim) + eh
-        # roll xyz back to the first axis
-        newxyz = np.rollaxis(newxyz, -1, 0)
-        newrep = CartesianRepresentation(newxyz).represent_as(SphericalRepresentation)
+        newrep = intermedrep.to_cartesian() + eh
 
     return hcrs_frame.realize_frame(newrep)
 
 
+_NEED_ORIGIN_HINT = ("The input {0} coordinates do not have length units. This "
+                     "probably means you created coordinates with lat/lon but "
+                     "no distance.  Heliocentric<->ICRS transforms cannot "
+                     "function in this case because there is an origin shift.")
+
+
 @frame_transform_graph.transform(FunctionTransform, HCRS, ICRS)
 def hcrs_to_icrs(hcrs_coo, icrs_frame):
-    jd1, jd2 = get_jd12(hcrs_coo.obstime, 'tdb')
-    earth_pv_helio, earth_pv_bary = erfa.epv00(jd1, jd2)
-    bary_sun_pos = earth_pv_bary[..., 0, :] - earth_pv_helio[..., 0, :]
-    sun_vector = u.Quantity(bary_sun_pos, u.au)
-    new_vector = np.rollaxis(hcrs_coo.cartesian.xyz, -1, 0) + sun_vector
-    newrep = CartesianRepresentation(np.rollaxis(new_vector, 0, new_vector.ndim))
+    # this is just an origin translation so without a distance it cannot go ahead
+    if hcrs_coo.data.__class__ == UnitSphericalRepresentation:
+        raise u.UnitsError(_NEED_ORIGIN_HINT.format(hcrs_coo.__class__.__name__))
+
+    # this goes here to avoid circular import errors
+    from ..solar_system import get_body_barycentric
+    bary_sun_pos = get_body_barycentric('sun', hcrs_coo.obstime)
+    newrep = hcrs_coo.cartesian + bary_sun_pos
     return icrs_frame.realize_frame(newrep)
 
 
 @frame_transform_graph.transform(FunctionTransform, ICRS, HCRS)
 def icrs_to_hcrs(icrs_coo, hcrs_frame):
-    jd1, jd2 = get_jd12(hcrs_frame.obstime, 'tdb')
-    earth_pv_helio, earth_pv_bary = erfa.epv00(jd1, jd2)
-    bary_sun_pos = earth_pv_bary[..., 0, :] - earth_pv_helio[..., 0, :]
-    sun_vector = u.Quantity(bary_sun_pos, u.au)
-    new_vector = np.rollaxis(icrs_coo.cartesian.xyz, -1, 0) - sun_vector
-    newrep = CartesianRepresentation(np.rollaxis(new_vector, 0, new_vector.ndim))
+    # this is just an origin translation so without a distance it cannot go ahead
+    if icrs_coo.data.__class__ == UnitSphericalRepresentation:
+        raise u.UnitsError(_NEED_ORIGIN_HINT.format(icrs_coo.__class__.__name__))
+
+    # this goes here to avoid circular import errors
+    from ..solar_system import get_body_barycentric
+    bary_sun_pos = get_body_barycentric('sun', hcrs_frame.obstime)
+    newrep = icrs_coo.cartesian - bary_sun_pos
     return hcrs_frame.realize_frame(newrep)
 
 
