@@ -3,7 +3,8 @@
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-from ..angles import rotation_matrix
+from ..matrix_utilities import (rotation_matrix,
+                                matrix_product, matrix_transpose)
 from ..baseframe import frame_transform_graph
 from ..transformations import DynamicMatrixTransform
 
@@ -26,7 +27,7 @@ def _icrs_to_fk5_matrix():
     m2 = rotation_matrix(xi0, 'y')
     m3 = rotation_matrix(da0, 'z')
 
-    return m1 * m2 * m3
+    return matrix_product(m1, m2, m3)
 # define this here because it only needs to be computed once
 _ICRS_TO_FK5_J2000_MAT = _icrs_to_fk5_matrix()
 
@@ -35,7 +36,7 @@ _ICRS_TO_FK5_J2000_MAT = _icrs_to_fk5_matrix()
 def icrs_to_fk5(icrscoord, fk5frame):
     # ICRS is by design very close to J2000 equinox
     pmat = fk5frame._precession_matrix(EQUINOX_J2000, fk5frame.equinox)
-    return pmat * _ICRS_TO_FK5_J2000_MAT
+    return matrix_product(pmat, _ICRS_TO_FK5_J2000_MAT)
 
 
 # can't be static because the equinox is needed
@@ -43,4 +44,4 @@ def icrs_to_fk5(icrscoord, fk5frame):
 def fk5_to_icrs(fk5coord, icrsframe):
     # ICRS is by design very close to J2000 equinox
     pmat = fk5coord._precession_matrix(fk5coord.equinox, EQUINOX_J2000)
-    return _ICRS_TO_FK5_J2000_MAT.T * pmat
+    return matrix_product(matrix_transpose(_ICRS_TO_FK5_J2000_MAT), pmat)
