@@ -2,14 +2,24 @@
 from __future__ import absolute_import, division, print_function
 
 from warnings import warn
+import socket
+import json
 
 import numpy as np
 from .. import units as u
+from ..units.quantity import QuantityInfo
 from ..extern import six
+from ..extern.six.moves import urllib
 from ..utils.exceptions import AstropyUserWarning
-from . import Longitude, Latitude
+from ..utils.compat.numpycompat import NUMPY_LT_1_12
+from ..utils.compat.numpy import broadcast_to
+from .angles import Longitude, Latitude
 from .builtin_frames import ITRS, GCRS
+from .representation import CartesianRepresentation
 from .errors import UnknownSiteException
+from ..utils import data
+
+from .name_resolve import NameResolveError
 
 try:
     # Not guaranteed available at setup time.
@@ -23,13 +33,13 @@ __all__ = ['EarthLocation']
 # Available ellipsoids (defined in erfam.h, with numbers exposed in erfa).
 ELLIPSOIDS = ('WGS84', 'GRS80', 'WGS72')
 
+OMEGA_EARTH = u.Quantity(7.292115855306589e-5, 1./u.s)
 """
 Rotational velocity of Earth. In UT1 seconds, this would be 2 pi / (24 * 3600),
 but we need the value in SI seconds.
 See Explanatory Supplement to the Astronomical Almanac, ed. P. Kenneth Seidelmann (1992),
 University Science Books.
 """
-V_EARTH = u.Quantity([0, 0, 7.292115855306589e-5])*u.rad/u.s
 
 
 def _check_ellipsoid(ellipsoid=None, default='WGS84'):
@@ -40,6 +50,53 @@ def _check_ellipsoid(ellipsoid=None, default='WGS84'):
                          .format(ellipsoid, ELLIPSOIDS))
     return ellipsoid
 
+def _get_json_result(url, err_str):
+    try:
+        # Retrieve JSON response from Google maps API
+        resp = urllib.request.urlopen(url, timeout=data.conf.remote_timeout)
+        resp_data = json.loads(resp.read().decode('utf8'))
+
+    except urllib.error.URLError as e:
+        # This catches a timeout error, see:
+        #   http://stackoverflow.com/questions/2712524/handling-urllib2s-timeout-python
+        if isinstance(e.reason, socket.timeout):
+            raise NameResolveError(err_str.format(msg="connection timed out"))
+        else:
+            raise NameResolveError(err_str.format(msg=e.reason))
+
+    except socket.timeout:
+        # There are some cases where urllib2 does not catch socket.timeout
+        # especially while receiving response data on an already previously
+        # working request
+        raise NameResolveError(err_str.format(msg="connection timed out"))
+
+    results = resp_data.get('results', [])
+
+    if not results:
+        raise NameResolveError(err_str.format(msg="no results returned"))
+
+    if resp_data.get('status', None) != 'OK':
+        raise NameResolveError(err_str.format(msg="unknown failure with Google maps API"))
+
+    return results
+
+
+class EarthLocationInfo(QuantityInfo):
+    """
+    Container for meta information like name, description, format.  This is
+    required when the object is used as a mixin column within a table, but can
+    be used as a general way to store meta information.
+    """
+    _represent_as_dict_attrs = ('x', 'y', 'z', 'ellipsoid')
+
+    def _construct_from_dict(self, map):
+        # Need to pop ellipsoid off and update post-instantiation.  This is
+        # on the to-fix list in #4261.
+        ellipsoid = map.pop('ellipsoid')
+        out = self._parent_cls(**map)
+        out.ellipsoid = ellipsoid
+        return out
+
 
 class EarthLocation(u.Quantity):
     """
@@ -71,6 +128,8 @@ class EarthLocation(u.Quantity):
                                 'formats': [np.float64]*3})
     _array_dtype = np.dtype((np.float64, (3,)))
 
+    info = EarthLocationInfo()
+
     def __new__(cls, *args, **kwargs):
         try:
             self = cls.from_geocentric(*args, **kwargs)
@@ -180,7 +239,7 @@ class EarthLocation(u.Quantity):
         # get geocentric coordinates. Have to give one-dimensional array.
         xyz = erfa.gd2gc(getattr(erfa, ellipsoid), _lon.ravel(),
                                  _lat.ravel(), _height.ravel())
-        self = xyz.view(cls._location_dtype, cls).reshape(lon.shape)
+        self = xyz.view(cls._location_dtype, cls).reshape(_lon.shape)
         self._unit = u.meter
         self._ellipsoid = ellipsoid
         return self.to(height.unit)
@@ -234,6 +293,72 @@ class EarthLocation(u.Quantity):
             newel.info.name = el.info.name
             return newel
 
+    @classmethod
+    def of_address(cls, address, get_height=False):
+        """
+        Return an object of this class for a given address by querying the Google
+        maps geocoding API.
+
+        This is intended as a quick convenience function to get fast access to
+        locations. In the background, this just issues a query to the Google maps
+        geocoding API. It is not meant to be abused! Google uses IP-based query
+        limiting and will ban your IP if you send more than a few thousand queries
+        per hour [1]_.
+
+        .. warning::
+            If the query returns more than one location (e.g., searching on
+            ``address='springfield'``), this function will use the **first** returned
+            location.
+
+        Parameters
+        ----------
+        address : str
+            The address to get the location for. As per the Google maps API, this
+            can be a fully specified street address (e.g., 123 Main St., New York,
+            NY) or a city name (e.g., Danbury, CT), or etc.
+        get_height : bool (optional)
+            Use the retrieved location to perform a second query to the Google maps
+            elevation API to retrieve the height of the input address [2]_.
+
+        Returns
+        -------
+        location : This class (a `~astropy.coordinates.EarthLocation` or subclass)
+            The location of the input address.
+
+        References
+        ----------
+        .. [1] https://developers.google.com/maps/documentation/geocoding/intro
+        .. [2] https://developers.google.com/maps/documentation/elevation/intro
+
+        """
+
+        pars = urllib.parse.urlencode({'address': address})
+        geo_url = "https://maps.googleapis.com/maps/api/geocode/json?{0}".format(pars)
+
+        # get longitude and latitude location
+        err_str = ("Unable to retrieve coordinates for address '{address}'; {{msg}}"
+                   .format(address=address))
+        geo_result = _get_json_result(geo_url, err_str=err_str)
+        loc = geo_result[0]['geometry']['location']
+
+        if get_height:
+            pars = {'locations': '{lat:.8f},{lng:.8f}'.format(lat=loc['lat'],
+                                                              lng=loc['lng'])}
+            pars = urllib.parse.urlencode(pars)
+            ele_url = "https://maps.googleapis.com/maps/api/elevation/json?{0}".format(pars)
+
+            err_str = ("Unable to retrieve elevation for address '{address}'; {{msg}}"
+                       .format(address=address))
+            ele_result = _get_json_result(ele_url, err_str=err_str)
+            height = ele_result[0]['elevation']*u.meter
+
+        else:
+            height = 0.
+
+        return cls.from_geodetic(lon=loc['lng']*u.degree,
+                                 lat=loc['lat']*u.degree,
+                                 height=height)
+
     @classmethod
     def get_site_names(cls):
         """
@@ -397,6 +522,11 @@ class EarthLocation(u.Quantity):
         itrs : `~astropy.coordinates.ITRS`
             The new object in the ITRS frame
         """
+        # Broadcast for a single position at multiple times, but don't attempt
+        # to be more general here.
+        if obstime and self.size == 1 and obstime.size > 1:
+            self = broadcast_to(self, obstime.shape, subok=True)
+
         return ITRS(x=self.x, y=self.y, z=self.z, obstime=obstime)
 
     itrs = property(get_itrs, doc="""An `~astropy.coordinates.ITRS` object  with
@@ -415,19 +545,19 @@ class EarthLocation(u.Quantity):
 
         Returns
         --------
-        obsgeoloc : `~astropy.units.Quantity`
+        obsgeoloc : `~astropy.coordinates.CartesianRepresentation`
             The GCRS position of the object
-        obsgeovel : `~astropy.units.Quantity`
+        obsgeovel : `~astropy.coordinates.CartesianRepresentation`
             The GCRS velocity of the object
         """
         itrs = self.get_itrs(obstime)
         geocentric_frame = GCRS(obstime=obstime)
         # GCRS position
-        obsgeoloc = itrs.transform_to(geocentric_frame).cartesian.xyz.to(u.m)
-
-        vel_arr = np.cross(V_EARTH, np.rollaxis(obsgeoloc, 0, obsgeoloc.ndim))
-        vel_arr = np.rollaxis(vel_arr, -1, 0)
-        obsgeovel = u.Quantity(vel_arr, u.m/u.s, copy=False)
+        obsgeoloc = itrs.transform_to(geocentric_frame).cartesian
+        vel_x = -OMEGA_EARTH * obsgeoloc.y
+        vel_y = OMEGA_EARTH * obsgeoloc.x
+        vel_z = 0. * vel_x.unit
+        obsgeovel = CartesianRepresentation(vel_x, vel_y, vel_z)
         return obsgeoloc, obsgeovel
 
     @property
@@ -470,6 +600,14 @@ class EarthLocation(u.Quantity):
                               unit)
     to.__doc__ = u.Quantity.to.__doc__
 
+    if NUMPY_LT_1_12:
+        def __repr__(self):
+            # Use the numpy >=1.12 way to format structured arrays.
+            from  .representation import _array2string
+            prefixstr = '<' + self.__class__.__name__ + ' '
+            arrstr = _array2string(self.view(np.ndarray), prefix=prefixstr)
+            return '{0}{1}{2:s}>'.format(prefixstr, arrstr, self._unitstr)
+
 
 # need to do this here at the bottom to avoid circular dependencies
 from .sites import get_builtin_sites, get_downloaded_sites
