@@ -14,6 +14,7 @@ from __future__ import (absolute_import, division, print_function,
 import numpy as np
 
 from .. import units as u
+from ..constants import c
 from .. import _erfa as erfa
 from ..io import ascii
 from ..utils import isiterable, data
@@ -153,24 +154,22 @@ def get_sun(time):
     # We have to manually do aberration because we're outputting directly into
     # GCRS
     earth_p = earth_pv_helio[..., 0, :]
-    earth_v = earth_pv_helio[..., 1, :]
+    earth_v = earth_pv_bary[..., 1, :]
+
+    # convert barycentric velocity to units of c, but keep as array for passing in to erfa
+    earth_v /= c.to(u.au/u.d).value
 
     dsun = np.sqrt(np.sum(earth_p**2, axis=-1))
-    invlorentz = (1-np.sum(earth_v**2, axis=-1))**-0.5
-    properdir = erfa.ab(earth_p/dsun.reshape(-1, 1), earth_v, dsun, invlorentz)
+    invlorentz = (1-np.sum(earth_v**2, axis=-1))**0.5
+    properdir = erfa.ab(earth_p/dsun.reshape(dsun.shape + (1,)),
+                        -earth_v, dsun, invlorentz)
 
-    x = -dsun*properdir[..., 0] * u.AU
-    y = -dsun*properdir[..., 1] * u.AU
-    z = -dsun*properdir[..., 2] * u.AU
-
-    if time.isscalar:
-        cartrep = CartesianRepresentation(x=x[0], y=y[0], z=z[0])
-    else:
-        cartrep = CartesianRepresentation(x=x, y=y, z=z)
+    cartrep = CartesianRepresentation(x=-dsun*properdir[..., 0] * u.AU,
+                                      y=-dsun*properdir[..., 1] * u.AU,
+                                      z=-dsun*properdir[..., 2] * u.AU)
     return SkyCoord(cartrep, frame=GCRS(obstime=time))
 
 
-
 def concatenate(coords):
     """
     Combine multiple coordinate objects into a single
