@@ -8,16 +8,18 @@ import numpy as np
 
 from ..utils.compat.misc import override__dir__
 from ..extern import six
-from ..extern.six.moves import zip
+from ..extern.six.moves import zip, range
 from ..units import Unit, IrreducibleUnit
 from .. import units as u
 from ..wcs.utils import skycoord_to_pixel, pixel_to_skycoord
 from ..utils.exceptions import AstropyDeprecationWarning
-from ..utils.data_info import MixinInfo
+from ..utils.data_info import MixinInfo, _get_obj_attrs_map
+from ..utils import ShapedLikeNDArray
 
 from .distances import Distance
+from .angles import Angle
 from .baseframe import BaseCoordinateFrame, frame_transform_graph, GenericFrame, _get_repr_cls
-from .builtin_frames import ICRS
+from .builtin_frames import ICRS, SkyOffsetFrame
 from .representation import (BaseRepresentation, SphericalRepresentation,
                              UnitSphericalRepresentation)
 
@@ -78,8 +80,23 @@ class SkyCoordInfo(MixinInfo):
             repr_data = sc.represent_as(sc.representation, in_frame_units=True)
         return repr_data
 
+    def _represent_as_dict(self):
+        obj = self._parent
+        attrs = list(obj.representation_component_names)
+        attrs += list(FRAME_ATTR_NAMES_SET())
+        out = _get_obj_attrs_map(obj, attrs)
 
-class SkyCoord(object):
+        # Don't output distance if it is all unitless 1.0
+        if 'distance' in out and np.all(out['distance'] == 1.0):
+            del out['distance']
+
+        out['representation'] = obj.representation.get_name()
+        out['frame'] = obj.frame.name
+
+        return out
+
+
+class SkyCoord(ShapedLikeNDArray):
     """High-level object providing a flexible interface for celestial coordinate
     representation, manipulation, and transformation between systems.
 
@@ -159,6 +176,9 @@ class SkyCoord(object):
         Specifies the representation, e.g. 'spherical', 'cartesian', or
         'cylindrical'.  This affects the positional args and other keyword args
         which must correspond to the given representation.
+    copy : bool, optional
+        If `True` (default), a copy of any coordinate data is made.  This
+        argument can only be passed in as a keyword argument.
     **keyword_args
         Other keyword arguments as applicable for user-defined coordinate frames.
         Common options include:
@@ -188,6 +208,7 @@ class SkyCoord(object):
         # kwargs dict for initializing attributes for this object and for
         # creating the internal self._sky_coord_frame object
         args = list(args)  # Make it mutable
+        copy = kwargs.pop('copy', True)
         kwargs = self._parse_inputs(args, kwargs)
 
         # Set internal versions of object state attributes
@@ -204,7 +225,7 @@ class SkyCoord(object):
                 coord_kwargs[attr] = value
 
         # Finally make the internal coordinate object.
-        self._sky_coord_frame = frame.__class__(**coord_kwargs)
+        self._sky_coord_frame = frame.__class__(copy=copy, **coord_kwargs)
 
         if not self._sky_coord_frame.has_data:
             raise ValueError('Cannot create a SkyCoord without data')
@@ -221,22 +242,42 @@ class SkyCoord(object):
     def representation(self, value):
         self.frame.representation = value
 
-    def __len__(self):
-        return len(self.frame)
+    @property
+    def shape(self):
+        return self.frame.shape
 
-    def __nonzero__(self):  # Py 2.x
-        return self.frame.__nonzero__()
+    def _apply(self, method, *args, **kwargs):
+        """Create a new instance, applying a method to the underlying data.
 
-    def __bool__(self):  # Py 3.x
-        return self.frame.__bool__()
+        In typical usage, the method is any of the shape-changing methods for
+        `~numpy.ndarray` (``reshape``, ``swapaxes``, etc.), as well as those
+        picking particular elements (``__getitem__``, ``take``, etc.), which
+        are all defined in `~astropy.utils.misc.ShapedLikeNDArray`. It will be
+        applied to the underlying arrays in the representation (e.g., ``x``,
+        ``y``, and ``z`` for `~astropy.coordinates.CartesianRepresentation`),
+        as well as to any frame attributes that have a shape, with the results
+        used to create a new instance.
+
+        Internally, it is also used to apply functions to the above parts
+        (in particular, `~numpy.broadcast_to`).
+
+        Parameters
+        ----------
+        method : str or callable
+            If str, it is the name of a method that is applied to the internal
+            ``components``. If callable, the function is applied.
+        args : tuple
+            Any positional arguments for ``method``.
+        kwargs : dict
+            Any keyword arguments for ``method``.
+        """
 
-    def __getitem__(self, item):
         self_frame = self._sky_coord_frame
         try:
             # First turn `self` into a mockup of the thing we want - we can copy
             # this to get all the right attributes
-            self._sky_coord_frame = self_frame[item]
-            out = SkyCoord(self, representation=self.representation)
+            self._sky_coord_frame = self_frame._apply(method, *args, **kwargs)
+            out = SkyCoord(self, representation=self.representation, copy=False)
 
             # Copy other 'info' attr only if it has actually been defined.
             # See PR #3898 for further explanation and justification, along
@@ -355,7 +396,7 @@ class SkyCoord(object):
         # Frame name (string) or frame class?  Coerce into an instance.
         try:
             frame = _get_frame_class(frame)()
-        except:
+        except Exception:
             pass
 
         if isinstance(frame, SkyCoord):
@@ -419,7 +460,17 @@ class SkyCoord(object):
                 if attr in self.frame.get_frame_attr_names():
                     return getattr(self.frame, attr)
                 else:
-                    return getattr(self, '_' + attr)
+                    try:
+                        return getattr(self, '_' + attr)
+                    except AttributeError:
+                        # this can happen because FRAME_ATTR_NAMES_SET is
+                        # dynamic.  So if a frame is added to the transform
+                        # graph after this SkyCoord was created, the "real"
+                        # underlying attribute - e.g. `_equinox` does not exist
+                        # on the SkyCoord.  So we add this case to just use
+                        # None, as it wouldn't have been possible for the user
+                        # to have set the value until the frame existed anyway
+                        return None
 
             # Some attributes might not fall in the above category but still
             # are available through self._sky_coord_frame.
@@ -681,12 +732,55 @@ class SkyCoord(object):
             raise ValueError('The other object does not have a distance; '
                              'cannot compute 3d separation.')
 
-        dx = self_in_other_system.cartesian.x - other.cartesian.x
-        dy = self_in_other_system.cartesian.y - other.cartesian.y
-        dz = self_in_other_system.cartesian.z - other.cartesian.z
+        return Distance((self_in_other_system.cartesian -
+                         other.cartesian).norm())
 
-        distval = (dx.value ** 2 + dy.value ** 2 + dz.value ** 2) ** 0.5
-        return Distance(distval, dx.unit)
+    def spherical_offsets_to(self, tocoord):
+        r"""
+        Computes angular offsets to go *from* this coordinate *to* another.
+
+        Parameters
+        ----------
+        tocoord : `~astropy.coordinates.BaseCoordinateFrame`
+            The coordinate to offset to.
+
+        Returns
+        -------
+        lon_offset : `~astropy.coordinates.Angle`
+            The angular offset in the longitude direction (i.e., RA for
+            equatorial coordinates).
+        lat_offset : `~astropy.coordinates.Angle`
+            The angular offset in the latitude direction (i.e., Dec for
+            equatorial coordinates).
+
+        Raises
+        ------
+        ValueError
+            If the ``tocoord`` is not in the same frame as this one. This is
+            different from the behavior of the `separation`/`separation_3d`
+            methods because the offset components depend critically on the
+            specific choice of frame.
+
+        Notes
+        -----
+        This uses the sky offset frame machinery, and hence will produce a new
+        sky offset frame if one does not already exist for this object's frame
+        class.
+
+        See Also
+        --------
+        separation : for the *total* angular offset (not broken out into components)
+
+        """
+        if not self.is_equivalent_frame(tocoord):
+            raise ValueError('Tried to use spherical_offsets_to with two non-matching frames!')
+
+        aframe = self.skyoffset_frame()
+        acoord = tocoord.transform_to(aframe)
+
+        dlon = acoord.spherical.lon.view(Angle)
+        dlat = acoord.spherical.lat.view(Angle)
+        return dlon, dlat
 
     def match_to_catalog_sky(self, catalogcoord, nthneighbor=1):
         """
@@ -975,6 +1069,24 @@ class SkyCoord(object):
 
         return angle_utilities.position_angle(slon, slat, olon, olat)
 
+    def skyoffset_frame(self, rotation=None):
+        """
+        Returns the sky offset frame with this `SkyCoord` at the origin.
+
+        Returns
+        -------
+        astrframe : `~astropy.coordinates.SkyOffsetFrame`
+            A sky offset frame of the same type as this `SkyCoord` (e.g., if
+            this object has an ICRS coordinate, the resulting frame is
+            SkyOffsetICRS, with the origin set to this object)
+        rotation : `~astropy.coordinates.Angle` or `~astropy.units.Quantity` with angle units
+            The final rotation of the frame about the ``origin``. The sign of
+            the rotation is the left-hand rule. That is, an object at a
+            particular position angle in the un-rotated system will be sent to
+            the positive latitude (z) direction in the final frame.
+        """
+        return SkyOffsetFrame(origin=self, rotation=rotation)
+
     def get_constellation(self, short_name=False, constellation_list='iau'):
         """
         Determines the constellation(s) of the coordinates this `SkyCoord`
@@ -1072,7 +1184,7 @@ class SkyCoord(object):
     # Table interactions
     @classmethod
     def guess_from_table(cls, table, **coord_kwargs):
-        """
+        r"""
         A convenience method to create and return a new `SkyCoord` from the data
         in an astropy Table.
 
@@ -1319,7 +1431,7 @@ def _get_units(args, kwargs):
             units.extend(None for x in range(3 - len(units)))
             if len(units) > 3:
                 raise ValueError()
-        except:
+        except Exception:
             raise ValueError('Unit keyword must have one to three unit values as '
                              'tuple or comma-separated string')
 
@@ -1401,15 +1513,14 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
             # SkyCoords from the list elements and then combine them.
             scs = [SkyCoord(coord, **init_kwargs) for coord in coords]
 
-            # now check that they're all self-consistent in their frames and
-            # check if they are all UnitSphericalRepresentation internally
-            allunitsphrepr = True
+            # Check that all frames are equivalent
             for sc in scs[1:]:
                 if not sc.is_equivalent_frame(scs[0]):
-                        raise ValueError("List of inputs don't have equivalent "
-                                         "frames: {0} != {1}".format(sc, scs[0]))
-                if allunitsphrepr and not isinstance(sc.data, UnitSphericalRepresentation):
-                    allunitsphrepr = False
+                    raise ValueError("List of inputs don't have equivalent "
+                                     "frames: {0} != {1}".format(sc, scs[0]))
+
+            # Now use the first to determine if they are all UnitSpherical
+            allunitsphrepr = isinstance(scs[0].data, UnitSphericalRepresentation)
 
             # get the frame attributes from the first one, because from above we
             # know it matches all the others
@@ -1449,7 +1560,7 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
             # lengths the same
             try:
                 n_coords = sorted(set(len(x) for x in vals))
-            except:
+            except Exception:
                 raise ValueError('One or more elements of input sequence does not have a length')
 
             if len(n_coords) > 1:
@@ -1480,7 +1591,8 @@ def _parse_coordinate_arg(coords, frame, units, init_kwargs):
     try:
         for frame_attr_name, repr_attr_class, value, unit in zip(
                 frame_attr_names, repr_attr_classes, values, units):
-            valid_kwargs[frame_attr_name] = repr_attr_class(value, unit=unit)
+            valid_kwargs[frame_attr_name] = repr_attr_class(value, unit=unit,
+                                                            copy=False)
     except Exception as err:
         raise ValueError('Cannot parse first argument data "{0}" for attribute '
                          '{1}'.format(value, frame_attr_name), err)
