@@ -11,10 +11,11 @@ from __future__ import (absolute_import, division, print_function,
 import numpy as np
 from numpy.testing.utils import assert_allclose, assert_array_equal
 
-from ..angles import Longitude, Latitude, Angle
-from ...tests.helper import pytest
+from ..angles import Longitude, Latitude, Angle, rotation_matrix, angle_axis
+from ...tests.helper import pytest, catch_warnings
 from ... import units as u
 from ..errors import IllegalSecondError, IllegalMinuteError, IllegalHourError
+from ...utils.exceptions import AstropyDeprecationWarning
 
 
 def test_create_angles():
@@ -47,6 +48,8 @@ def test_create_angles():
     a10 = Angle(3.60827466667, unit=u.hour)
     a11 = Angle("3:36:29.7888000120", unit=u.hour)
     a12 = Angle((3, 36, 29.7888000120), unit=u.hour)  # *must* be a tuple
+    # Regression test for #5001
+    a13 = Angle((3, 36, 29.7888000120), unit='hour')
 
     Angle(0.944644098745, unit=u.radian)
 
@@ -54,12 +57,16 @@ def test_create_angles():
         Angle(54.12412)
         #raises an exception because this is ambiguous
 
+    with pytest.raises(u.UnitsError):
+        Angle(54.12412, unit=u.m)
+
     with pytest.raises(ValueError):
-        a13 = Angle(12.34, unit="not a unit")
+        Angle(12.34, unit="not a unit")
 
     a14 = Angle("03h36m29.7888000120") # no trailing 's', but unambiguous
 
     a15 = Angle("5h4m3s") # single digits, no decimal
+    assert a15.unit == u.hourangle
 
     a16 = Angle("1 d")
     a17 = Angle("1 degree")
@@ -85,7 +92,7 @@ def test_create_angles():
     assert_allclose(a6.radian, a7.radian)
 
     assert_allclose(a10.degree, a11.degree)
-    assert a11 == a12 == a14
+    assert a11 == a12 == a13 == a14
     assert a21 == a22
     assert a23 == -a24
 
@@ -113,6 +120,19 @@ def test_create_angles():
 
     assert a1 is not None
 
+
+def test_angle_from_view():
+    q = np.arange(3.) * u.deg
+    a = q.view(Angle)
+    assert type(a) is Angle
+    assert a.unit is q.unit
+    assert np.all(a == q)
+
+    q2 = np.arange(4) * u.m
+    with pytest.raises(u.UnitTypeError):
+        q2.view(Angle)
+
+
 def test_angle_ops():
     """
     Tests operations on Angle objects
@@ -149,13 +169,24 @@ def test_angle_ops():
     assert a1 < a5
     assert a1 <= a5
 
+    # check operations with non-angular result give Quantity.
     a6 = Angle(45., u.degree)
     a7 = a6 * a5
     assert type(a7) is u.Quantity
 
+    # but those with angular result yield Angle.
+    # (a9 is regression test for #5327)
+    a8 = a1 + 1.*u.deg
+    assert type(a8) is Angle
+    a9 = 1.*u.deg + a1
+    assert type(a9) is Angle
+
     with pytest.raises(TypeError):
         a6 *= a5
 
+    with pytest.raises(TypeError):
+        a6 *= u.m
+
     with pytest.raises(TypeError):
         np.sin(a6, out=a6)
 
@@ -793,49 +824,6 @@ def test_mixed_string_and_quantity():
     assert_array_equal(a2.value, [1., 180., 3.])
     assert a2.unit == u.deg
 
-def test_rotation_matrix():
-    from ..angles import rotation_matrix
-
-    assert_array_equal(rotation_matrix(0*u.deg, 'x'), np.eye(3))
-
-    assert_allclose(rotation_matrix(90*u.deg, 'y'), [[ 0, 0,-1],
-                                                     [ 0, 1, 0],
-                                                     [ 1, 0, 0]], atol=1e-12)
-
-    assert_allclose(rotation_matrix(-90*u.deg, 'z'), [[ 0,-1, 0],
-                                                      [ 1, 0, 0],
-                                                      [ 0, 0, 1]], atol=1e-12)
-
-    assert_allclose(rotation_matrix(45*u.deg, 'x'),
-                    rotation_matrix(45*u.deg, [1, 0, 0]))
-    assert_allclose(rotation_matrix(125*u.deg, 'y'),
-                    rotation_matrix(125*u.deg, [0, 1, 0]))
-    assert_allclose(rotation_matrix(-30*u.deg, 'z'),
-                    rotation_matrix(-30*u.deg, [0, 0, 1]))
-
-    assert_allclose(np.dot(rotation_matrix(180*u.deg, [1, 1, 0]).A, [1, 0, 0]),
-                    [0, 1, 0], atol=1e-12)
-
-    #make sure it also works for very small angles
-    assert_allclose(rotation_matrix(0.000001*u.deg, 'x'),
-                    rotation_matrix(0.000001*u.deg, [1, 0, 0]))
-
-def test_angle_axis():
-    from ..angles import rotation_matrix, angle_axis
-
-    m1 = rotation_matrix(35*u.deg, 'x')
-    an1, ax1 = angle_axis(m1)
-
-    assert an1 - 35*u.deg < 1e-10*u.deg
-    assert_allclose(ax1, [1, 0, 0])
-
-
-    m2 = rotation_matrix(-89*u.deg, [1, 1, 0])
-    an2, ax2 = angle_axis(m2)
-
-    assert an2 - 89*u.deg < 1e-10*u.deg
-    assert_allclose(ax2, [-2**-0.5, -2**-0.5, 0])
-
 def test_array_angle_tostring():
     aobj = Angle([1, 2], u.deg)
     assert aobj.to_string().dtype.kind == 'U'
@@ -868,9 +856,42 @@ def test_repr_latex():
     arrangle = Angle([1, 2.1], u.deg)
     rlarrangle = arrangle._repr_latex_()
 
-    assert rlscangle == '$2^\circ06{}^\prime00{}^{\prime\prime}$'
+    assert rlscangle == r'$2^\circ06{}^\prime00{}^{\prime\prime}$'
     assert rlscangle.split('$')[1] in rlarrangle
 
     # make sure the ... appears for large arrays
     bigarrangle = Angle(np.ones(50000)/50000., u.deg)
     assert '...' in bigarrangle._repr_latex_()
+
+
+def test_rotation_matrix_deprecation():
+    with catch_warnings(AstropyDeprecationWarning):
+        m = rotation_matrix(0.*u.deg, 'x')
+    assert isinstance(m, np.matrix)
+    assert_array_equal(m, np.eye(3))
+
+
+def test_angle_axis_deprecation():
+    m = np.matrix([[0., 1., 0,],
+                   [-1, 0., 0.],
+                   [0., 0., 1.]])
+    with catch_warnings(AstropyDeprecationWarning):
+        an1, ax1 = angle_axis(m)
+    assert_allclose(an1.to(u.deg).value, 90.)
+    assert_allclose(ax1, [0., 0., 1.])
+
+
+def test_angle_with_cds_units_enabled():
+    """Regression test for #5350
+
+    Especially the example in
+    https://github.com/astropy/astropy/issues/5350#issuecomment-248770151
+    """
+    from ...units import cds
+    # the problem is with the parser, so remove it temporarily
+    from ..angle_utilities import _AngleParser
+    del _AngleParser._parser
+    with cds.enable():
+        Angle('5d')
+    del _AngleParser._parser
+    Angle('5d')
