@@ -10,24 +10,27 @@ core.py:
 
 from __future__ import absolute_import, division, print_function
 
+import copy
+import csv
+import functools
+import itertools
+import operator
 import os
 import re
-import csv
-import itertools
-import functools
-import numpy
 import warnings
-import copy
+
+from collections import OrderedDict
+
+import numpy
 
 from ...extern import six
-from ...extern.six.moves import zip
+from ...extern.six.moves import zip, range
 from ...extern.six.moves import cStringIO as StringIO
 from ...utils.exceptions import AstropyWarning
 
 from ...table import Table
-from ...utils.compat import ignored
+from ...utils.compat import suppress
 from ...utils.data import get_readable_fileobj
-from ...utils import OrderedDict
 from . import connect
 
 # Global dictionary mapping format arg to the corresponding Reader class
@@ -36,6 +39,104 @@ FORMAT_CLASSES = {}
 # Similar dictionary for fast readers
 FAST_CLASSES = {}
 
+
+class CsvWriter(object):
+    """
+    Internal class to replace the csv writer ``writerow`` and ``writerows``
+    functions so that in the case of ``delimiter=' '`` and
+    ``quoting=csv.QUOTE_MINIMAL``, the output field value is quoted for empty
+    fields (when value == '').
+
+    This changes the API slightly in that the writerow() and writerows()
+    methods return the output written string instead of the length of
+    that string.
+
+    Examples
+    --------
+
+    >>> from astropy.io.ascii.core import CsvWriter
+    >>> writer = CsvWriter(delimiter=' ')
+    >>> print(writer.writerow(['hello', '', 'world']))
+    hello "" world
+    """
+    # Random 16-character string that gets injected instead of any
+    # empty fields and is then replaced post-write with doubled-quotechar.
+    # Created with:
+    # ''.join(random.choice(string.printable[:90]) for _ in range(16))
+    replace_sentinel = '2b=48Av%0-V3p>bX'
+
+    def __init__(self, csvfile=None, **kwargs):
+        self.csvfile = csvfile
+
+        # Temporary StringIO for catching the real csv.writer() object output
+        self.temp_out = StringIO()
+        self.writer = csv.writer(self.temp_out, **kwargs)
+
+        dialect = self.writer.dialect
+        self.quotechar2 = dialect.quotechar * 2
+        self.quote_empty = (dialect.quoting == csv.QUOTE_MINIMAL) and (dialect.delimiter == ' ')
+
+    def writerow(self, values):
+        """
+        Similar to csv.writer.writerow but with the custom quoting behavior.
+        Returns the written string instead of the length of that string.
+        """
+        has_empty = False
+
+        # If QUOTE_MINIMAL and space-delimited then replace empty fields with
+        # the sentinel value.
+        if self.quote_empty:
+            for i, value in enumerate(values):
+                if value == '':
+                    has_empty = True
+                    values[i] = self.replace_sentinel
+
+        return self._writerow(self.writer.writerow, values, has_empty)
+
+    def writerows(self, values_list):
+        """
+        Similar to csv.writer.writerows but with the custom quoting behavior.
+        Returns the written string instead of the length of that string.
+        """
+        has_empty = False
+
+        # If QUOTE_MINIMAL and space-delimited then replace empty fields with
+        # the sentinel value.
+        if self.quote_empty:
+            for values in values_list:
+                for i, value in enumerate(values):
+                    if value == '':
+                        has_empty = True
+                        values[i] = self.replace_sentinel
+
+        return self._writerow(self.writer.writerows, values_list, has_empty)
+
+    def _writerow(self, writerow_func, values, has_empty):
+        """
+        Call ``writerow_func`` (either writerow or writerows) with ``values``.
+        If it has empty fields that have been replaced then change those
+        sentinel strings back to quoted empty strings, e.g. ``""``.
+        """
+        # Clear the temporary StringIO buffer that self.writer writes into and
+        # then call the real csv.writer().writerow or writerows with values.
+        self.temp_out.seek(0)
+        self.temp_out.truncate()
+        writerow_func(values)
+
+        row_string = self.temp_out.getvalue()
+
+        if self.quote_empty and has_empty:
+            row_string = re.sub(self.replace_sentinel, self.quotechar2, row_string)
+
+        # self.csvfile is defined then write the output.  In practice the pure
+        # Python writer calls with csvfile=None, while the fast writer calls with
+        # a file-like object.
+        if self.csvfile:
+            self.csvfile.write(row_string)
+
+        return row_string
+
+
 class MaskedConstant(numpy.ma.core.MaskedConstant):
     """A trivial extension of numpy.ma.masked
 
@@ -291,7 +392,7 @@ class DefaultSplitter(BaseSplitter):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
         whitespace-delimited files to prevent spurious columns at the beginning or end.
         If splitting on whitespace then replace unquoted tabs with space first"""
-        if self.delimiter == '\s':
+        if self.delimiter == r'\s':
             line = _replace_tab_with_space(line, self.escapechar, self.quotechar)
         return line.strip()
 
@@ -319,7 +420,7 @@ class DefaultSplitter(BaseSplitter):
         quotechar = None if self.quotechar is None else str(self.quotechar)
         delimiter = None if self.delimiter is None else str(self.delimiter)
 
-        if delimiter == '\s':
+        if delimiter == r'\s':
             delimiter = ' '
 
         csv_reader = csv.reader(lines,
@@ -344,21 +445,17 @@ class DefaultSplitter(BaseSplitter):
         delimiter = ' ' if self.delimiter is None else str(self.delimiter)
 
         if self.csv_writer is None:
-            self.csv_writer = csv.writer(self.csv_writer_out,
-                                         delimiter=delimiter,
-                                         doublequote=self.doublequote,
-                                         escapechar=escapechar,
-                                         quotechar=quotechar,
-                                         quoting=self.quoting,
-                                         lineterminator='',
-                                         )
-        self.csv_writer_out.seek(0)
-        self.csv_writer_out.truncate()
+            self.csv_writer = CsvWriter(delimiter=delimiter,
+                                        doublequote=self.doublequote,
+                                        escapechar=escapechar,
+                                        quotechar=quotechar,
+                                        quoting=self.quoting,
+                                        lineterminator='')
         if self.process_val:
             vals = [self.process_val(x) for x in vals]
-        self.csv_writer.writerow(vals)
+        out = self.csv_writer.writerow(vals)
 
-        return self.csv_writer_out.getvalue()
+        return out
 
 
 def _replace_tab_with_space(line, escapechar, quotechar):
@@ -370,7 +467,7 @@ def _replace_tab_with_space(line, escapechar, quotechar):
         String containing tabs to be replaced with spaces.
     escapechar : str
         Character in ``line`` used to escape special characters.
-    quotechar: str
+    quotechar : str
         Character in ``line`` indicating the start/end of a substring.
 
     Returns
@@ -412,7 +509,7 @@ class BaseHeader(object):
     """
     Base table header reader
     """
-    auto_format = 'col%d'
+    auto_format = 'col{}'
     """ format string for auto-generating column names """
     start_line = None
     """ None, int, or a function of ``lines`` that returns None or int """
@@ -470,7 +567,8 @@ class BaseHeader(object):
                 raise InconsistentTableError('No data lines found so cannot autogenerate '
                                              'column names')
             n_data_cols = len(first_data_vals)
-            self.names = [self.auto_format % i for i in range(1, n_data_cols + 1)]
+            self.names = [self.auto_format.format(i)
+                          for i in range(1, n_data_cols + 1)]
 
         else:
             for i, line in enumerate(self.process_lines(lines)):
@@ -518,7 +616,7 @@ class BaseHeader(object):
             type_map_key = self.get_type_map_key(col)
             return self.col_type_map[type_map_key.lower()]
         except KeyError:
-            raise ValueError('Unknown data type ""%s"" for column "%s"' % (
+            raise ValueError('Unknown data type ""{}"" for column "{}"'.format(
                 col.raw_type, col.name))
 
     def check_column_names(self, names, strict_names, guessing):
@@ -573,8 +671,7 @@ class BaseData(object):
     write_spacer_lines = ['ASCII_TABLE_WRITE_SPACER_LINE']
     fill_include_names = None
     fill_exclude_names = None
-    # Currently, the default matches the numpy default for masked values.
-    fill_values = [(masked, '--')]
+    fill_values = [(masked, '')]
     formats = {}
 
     def __init__(self):
@@ -648,11 +745,11 @@ class BaseData(object):
             # when we write tables the columns may be astropy.table.Columns
             # which don't carry a fill_values by default
             for col in cols:
-                if ~hasattr(col, 'fill_values'):
+                if not hasattr(col, 'fill_values'):
                     col.fill_values = {}
 
             # if input is only one <fill_spec>, then make it a list
-            with ignored(TypeError):
+            with suppress(TypeError):
                 self.fill_values[0] + ''
                 self.fill_values = [self.fill_values]
 
@@ -777,6 +874,9 @@ def convert_numpy(numpy_type):
         Convert values "False" and "True" to bools.  Raise an exception
         for any other string values.
         """
+        if len(vals) == 0:
+            return numpy.array([], dtype=bool)
+
         # Try a smaller subset first for a long array
         if len(vals) > 10000:
             svals = numpy.asarray(vals[:1000])
@@ -819,8 +919,8 @@ class BaseOutputter(object):
                     converters_out.append((converter_func, converter_type))
 
         except (ValueError, TypeError):
-            raise ValueError('Error: invalid format for converters, see documentation\n%s' %
-                             converters)
+            raise ValueError('Error: invalid format for converters, see '
+                             'documentation\n{}'.format(converters))
         return converters_out
 
     def _convert_vals(self, cols):
@@ -835,23 +935,31 @@ class BaseOutputter(object):
 
             col.converters = self._validate_and_copy(col, converters)
 
+            # Catch the last error in order to provide additional information
+            # in case all attempts at column conversion fail.  The initial
+            # value of of last_error will apply if no converters are defined
+            # and the first col.converters[0] access raises IndexError.
+            last_err = 'no converters defined'
+
             while not hasattr(col, 'data'):
                 try:
                     converter_func, converter_type = col.converters[0]
                     if not issubclass(converter_type, col.type):
-                        raise TypeError()
+                        raise TypeError('converter type does not match column type')
                     col.data = converter_func(col.str_vals)
                     col.type = converter_type
-                except (TypeError, ValueError):
+                except (TypeError, ValueError) as err:
                     col.converters.pop(0)
-                except OverflowError:
+                    last_err = err
+                except OverflowError as err:
                     # Overflow during conversion (most likely an int that doesn't fit in native C long).
                     # Put string at the top of the converters list for the next while iteration.
                     warnings.warn("OverflowError converting to {0} for column {1}, using string instead."
                                   .format(converter_type.__name__, col.name), AstropyWarning)
                     col.converters.insert(0, convert_numpy(numpy.str))
+                    last_err = err
                 except IndexError:
-                    raise ValueError('Column %s failed to convert' % col.name)
+                    raise ValueError('Column {} failed to convert: {}'.format(col.name, last_err))
 
 
 class TableOutputter(BaseOutputter):
@@ -914,7 +1022,7 @@ class MetaBaseReader(type):
 
 
 def _is_number(x):
-    with ignored(ValueError):
+    with suppress(ValueError):
         x = float(x)
         return True
     return False
@@ -931,7 +1039,7 @@ def _apply_include_exclude_names(table, names, include_names, exclude_names):
         List of names to override those in table (set to None to use existing names)
     include_names : list
         List of names to include in output
-    exclude_names: list
+    exclude_names : list
         List of names to exclude from output (applied after ``include_names``)
 
     """
@@ -1027,7 +1135,7 @@ class BaseReader(object):
         # If ``table`` is a file then store the name in the ``data``
         # attribute. The ``table`` is a "file" if it is a string
         # without the new line specific to the OS.
-        with ignored(TypeError):
+        with suppress(TypeError):
             # Strings only
             if os.linesep not in table + '':
                 self.data.table_name = os.path.basename(table)
@@ -1061,11 +1169,13 @@ class BaseReader(object):
 
                 # otherwise, we raise an error only if it is still inconsistent
                 if len(str_vals) != n_cols:
-                    errmsg = ('Number of header columns (%d) inconsistent with '
-                              'data columns (%d) at data line %d\n'
-                              'Header values: %s\n'
-                              'Data values: %s' % (n_cols, len(str_vals), i,
-                                                   [x.name for x in cols], str_vals))
+                    errmsg = ('Number of header columns ({}) inconsistent with'
+                              ' data columns ({}) at data line {}\n'
+                              'Header values: {}\n'
+                              'Data values: {}'.format(
+                            n_cols, len(str_vals), i,
+                            [x.name for x in cols], str_vals))
+
                     raise InconsistentTableError(errmsg)
 
             for j, col in enumerate(cols):
@@ -1319,6 +1429,13 @@ def _get_writer(Writer, fast_writer, **kwargs):
 
     from .fastbasic import FastBasic
 
+    # A value of None for fill_values imply getting the default string
+    # representation of masked values (depending on the writer class), but the
+    # machinery expects a list.  The easiest here is to just pop the value off,
+    # i.e. fill_values=None is the same as not providing it at all.
+    if 'fill_values' in kwargs and kwargs['fill_values'] is None:
+        del kwargs['fill_values']
+
     if issubclass(Writer, FastBasic): # Fast writers handle args separately
         return Writer(**kwargs)
     elif fast_writer and 'fast_{0}'.format(Writer._format_name) in FAST_CLASSES:
@@ -1345,7 +1462,7 @@ def _get_writer(Writer, fast_writer, **kwargs):
             # Restore the default SplitterClass process_val method which strips
             # whitespace.  This may have been changed in the Writer
             # initialization (e.g. Rdb and Tab)
-            writer.data.splitter.process_val = lambda x: x.strip()
+            writer.data.splitter.process_val = operator.methodcaller('strip')
         else:
             writer.data.splitter.process_val = None
     if 'names' in kwargs:
@@ -1355,7 +1472,13 @@ def _get_writer(Writer, fast_writer, **kwargs):
     if 'exclude_names' in kwargs:
         writer.exclude_names = kwargs['exclude_names']
     if 'fill_values' in kwargs:
-        writer.data.fill_values = kwargs['fill_values']
+        # Prepend user-specified values to the class default.
+        with suppress(TypeError, IndexError):
+            # Test if it looks like (match, replace_string, optional_colname),
+            # in which case make it a list
+            kwargs['fill_values'][1] + ''
+            kwargs['fill_values'] = [kwargs['fill_values']]
+        writer.data.fill_values = kwargs['fill_values'] + writer.data.fill_values
     if 'fill_include_names' in kwargs:
         writer.data.fill_include_names = kwargs['fill_include_names']
     if 'fill_exclude_names' in kwargs:
