@@ -13,6 +13,7 @@ from __future__ import absolute_import, division, print_function
 import re
 
 from ...extern import six
+from ...extern.six.moves import zip
 from . import core
 
 latexdicts = {'AA':  {'tabletype': 'table',
@@ -31,6 +32,8 @@ latexdicts = {'AA':  {'tabletype': 'table',
               }
 
 
+RE_COMMENT = re.compile(r'(?<!\\)%')  # % character but not \%
+
 def add_dictval_to_list(adict, key, alist):
     '''
     Add a value from a dictionary to a list
@@ -39,7 +42,7 @@ def add_dictval_to_list(adict, key, alist):
     ----------
     adict : dictionary
     key : hashable
-    alist: list
+    alist : list
         List where value should be added
     '''
     if key in adict:
@@ -79,13 +82,20 @@ class LatexSplitter(core.BaseSplitter):
     '''
     delimiter = '&'
 
+    def __call__(self, lines):
+        last_line = RE_COMMENT.split(lines[-1])[0].strip()
+        if not last_line.endswith(r'\\'):
+            lines[-1] = last_line + r'\\'
+
+        return super(LatexSplitter, self).__call__(lines)
+
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line. Also remove
         \\ at end of line"""
-        line = line.split('%')[0]
+        line = RE_COMMENT.split(line)[0]
         line = line.strip()
-        if line[-2:] == r'\\':
-            line = line.strip(r'\\')
+        if line.endswith(r'\\'):
+            line = line.rstrip(r'\\')
         else:
             raise core.InconsistentTableError(r'Lines in LaTeX table have to end with \\')
         return line
@@ -110,7 +120,7 @@ class LatexHeader(core.BaseHeader):
 
     def start_line(self, lines):
         line = find_latex_line(lines, self.header_start)
-        if line:
+        if line is not None:
             return line + 1
         else:
             return None
@@ -127,7 +137,7 @@ class LatexHeader(core.BaseHeader):
         return units
 
     def write(self, lines):
-        if not 'col_align' in self.latex:
+        if 'col_align' not in self.latex:
             self.latex['col_align'] = len(self.cols) * 'c'
         if 'tablealign' in self.latex:
             align = '[' + self.latex['tablealign'] + ']'
@@ -316,12 +326,15 @@ class Latex(core.BaseReader):
 
 
 class AASTexHeaderSplitter(LatexSplitter):
-    '''Extract column names from a `deluxetable`_.
+    r'''Extract column names from a `deluxetable`_.
 
     This splitter expects the following LaTeX code **in a single line**:
 
         \tablehead{\colhead{col1} & ... & \colhead{coln}}
     '''
+    def __call__(self, lines):
+        return super(LatexSplitter, self).__call__(lines)
+
     def process_line(self, line):
         """extract column names from tablehead
         """
@@ -339,7 +352,7 @@ class AASTexHeaderSplitter(LatexSplitter):
 
 
 class AASTexHeader(LatexHeader):
-    '''In a `deluxetable
+    r'''In a `deluxetable
     <http://fits.gsfc.nasa.gov/standard30/deluxetable.sty>`_ some header
     keywords differ from standard LaTeX.
 
@@ -352,7 +365,7 @@ class AASTexHeader(LatexHeader):
         return find_latex_line(lines, r'\tablehead')
 
     def write(self, lines):
-        if not 'col_align' in self.latex:
+        if 'col_align' not in self.latex:
             self.latex['col_align'] = len(self.cols) * 'c'
         if 'tablealign' in self.latex:
             align = '[' + self.latex['tablealign'] + ']'
@@ -374,7 +387,7 @@ class AASTexHeader(LatexHeader):
 
 
 class AASTexData(LatexData):
-    '''In a `deluxetable`_ the data is enclosed in `\startdata` and `\enddata`
+    r'''In a `deluxetable`_ the data is enclosed in `\startdata` and `\enddata`
     '''
     data_start = r'\startdata'
     data_end = r'\enddata'
@@ -384,7 +397,14 @@ class AASTexData(LatexData):
 
     def write(self, lines):
         lines.append(self.data_start)
+        lines_length_initial = len(lines)
         core.BaseData.write(self, lines)
+        # To remove extra space(s) and // appended which creates an extra new line
+        # in the end.
+        if len(lines) > lines_length_initial:
+            # we compile separately because py2.6 doesn't have a flags keyword in re.sub
+            re_final_line = re.compile(r'\s* \\ \\ \s* $', flags=re.VERBOSE)
+            lines[-1] = re.sub(re_final_line, '', lines[-1])
         lines.append(self.data_end)
         add_dictval_to_list(self.latex, 'tablefoot', lines)
         lines.append(r'\end{' + self.latex['tabletype'] + r'}')
