@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 """
@@ -15,8 +17,9 @@ from ....table import Table
 import numpy as np
 
 from .common import setup_function, teardown_function
+from ... import ascii
 from ....tests.helper import pytest
-from ....extern.six.moves import cStringIO
+from ....extern.six.moves import range, cStringIO as StringIO
 from ....utils.xml.writer import HAS_BLEACH
 
 # Check to see if the BeautifulSoup dependency is present.
@@ -546,7 +549,7 @@ def test_raw_html_write():
     t = Table([['<em>x</em>'], ['<em>y</em>']], names=['a', 'b'])
 
     # One column contains raw HTML (string input)
-    out = cStringIO()
+    out = StringIO()
     t.write(out, format='ascii.html', htmldict={'raw_html_cols': 'a'})
     expected = """\
    <tr>
@@ -556,12 +559,12 @@ def test_raw_html_write():
     assert expected in out.getvalue()
 
     # One column contains raw HTML (list input)
-    out = cStringIO()
+    out = StringIO()
     t.write(out, format='ascii.html', htmldict={'raw_html_cols': ['a']})
     assert expected in out.getvalue()
 
     # Two columns contains raw HTML (list input)
-    out = cStringIO()
+    out = StringIO()
     t.write(out, format='ascii.html', htmldict={'raw_html_cols': ['a', 'b']})
     expected = """\
    <tr>
@@ -581,7 +584,7 @@ def test_raw_html_write_clean():
     t = Table([['<script>x</script>'], ['<p>y</p>'], ['<em>y</em>']], names=['a', 'b', 'c'])
 
     # Confirm that <script> and <p> get escaped but not <em>
-    out = cStringIO()
+    out = StringIO()
     t.write(out, format='ascii.html', htmldict={'raw_html_cols': t.colnames})
     expected = """\
    <tr>
@@ -592,7 +595,7 @@ def test_raw_html_write_clean():
     assert expected in out.getvalue()
 
     # Confirm that we can whitelist <p>
-    out = cStringIO()
+    out = StringIO()
     t.write(out, format='ascii.html',
             htmldict={'raw_html_cols': t.colnames,
                       'raw_html_clean_kwargs': {'tags': bleach.ALLOWED_TAGS + ['p']}})
@@ -603,3 +606,106 @@ def test_raw_html_write_clean():
     <td><em>y</em></td>
    </tr>"""
     assert expected in out.getvalue()
+
+def test_write_table_html_fill_values():
+    """
+    Test that passing fill_values should replace any matching row
+    """
+    buffer_output = StringIO()
+    t = Table([[1], [2]], names=('a', 'b'))
+    ascii.write(t, buffer_output, fill_values=('1', 'Hello world'),
+        format='html')
+
+    t_expected = Table([['Hello world'], [2]], names=('a', 'b'))
+    buffer_expected = StringIO()
+    ascii.write(t_expected, buffer_expected, format='html')
+
+    assert buffer_output.getvalue() == buffer_expected.getvalue()
+
+def test_write_table_html_fill_values_optional_columns():
+    """
+    Test that passing optional column in fill_values should only replace
+    matching columns
+    """
+    buffer_output = StringIO()
+    t = Table([[1], [1]], names=('a', 'b'))
+    ascii.write(t, buffer_output, fill_values=('1', 'Hello world', 'b'),
+        format='html')
+
+    t_expected = Table([[1], ['Hello world']], names=('a', 'b'))
+    buffer_expected = StringIO()
+    ascii.write(t_expected, buffer_expected, format='html')
+
+    assert buffer_output.getvalue() == buffer_expected.getvalue()
+
+def test_write_table_html_fill_values_masked():
+    """
+    Test that passing masked values in fill_values should only replace
+    masked columns or values
+    """
+    buffer_output = StringIO()
+    t = Table([[1], [1]], names=('a', 'b'), masked=True, dtype=('i4', 'i8'))
+    t['a'] = np.ma.masked
+    ascii.write(t, buffer_output, fill_values=(ascii.masked, 'TEST'),
+        format='html')
+
+    t_expected = Table([['TEST'], [1]], names=('a', 'b'))
+    buffer_expected = StringIO()
+    ascii.write(t_expected, buffer_expected, format='html')
+
+    assert buffer_output.getvalue() == buffer_expected.getvalue()
+
+def test_multicolumn_table_html_fill_values():
+    """
+    Test to make sure that the HTML writer writes multidimensional
+    columns with correctly replaced fill_values.
+    """
+    col1 = [1, 2, 3]
+    col2 = [(1.0, 1.0), (2.0, 2.0), (3.0, 3.0)]
+    col3 = [('a', 'a', 'a'), ('b', 'b', 'b'), ('c', 'c', 'c')]
+
+    buffer_output = StringIO()
+    t = Table([col1, col2, col3], names=('C1', 'C2', 'C3'))
+    ascii.write(t, buffer_output, fill_values=('a', 'z'),
+        format='html')
+
+    col1 = [1, 2, 3]
+    col2 = [(1.0, 1.0), (2.0, 2.0), (3.0, 3.0)]
+    col3 = [('z', 'z', 'z'), ('b', 'b', 'b'), ('c', 'c', 'c')]
+
+    buffer_expected = StringIO()
+    t_expected = Table([col1, col2, col3], names=('C1', 'C2', 'C3'))
+    ascii.write(t_expected, buffer_expected, format='html')
+
+    assert buffer_output.getvalue() == buffer_expected.getvalue()
+
+def test_multi_column_write_table_html_fill_values_masked():
+    """
+    Test that passing masked values in fill_values should only replace
+    masked columns or values for multidimensional tables
+    """
+    buffer_output = StringIO()
+    t = Table([[1, 2, 3, 4], ['--', 'a', '--', 'b']], names=('a', 'b'), masked=True)
+    t['a'][0:2] = np.ma.masked
+    t['b'][0:2] = np.ma.masked
+    ascii.write(t, buffer_output, fill_values=[(ascii.masked, 'MASKED')],
+        format='html')
+
+    t_expected = Table([['MASKED', 'MASKED', 3, 4], ['MASKED', 'MASKED', '--', 'b']], names=('a', 'b'))
+    buffer_expected = StringIO()
+    ascii.write(t_expected, buffer_expected, format='html')
+    print(buffer_expected.getvalue())
+
+    assert buffer_output.getvalue() == buffer_expected.getvalue()
+
+@pytest.mark.skipif('not HAS_BEAUTIFUL_SOUP')
+def test_read_html_unicode():
+    """
+    Test reading an HTML table with unicode values
+    """
+    table_in = [u'<table>',
+                u'<tr><td>&#x0394;</td></tr>',
+                u'<tr><td>Δ</td></tr>',
+                u'</table>']
+    dat = Table.read(table_in, format='ascii.html')
+    assert np.all(dat['col1'] == [u'Δ', u'Δ'])
