@@ -15,6 +15,7 @@ import os
 import sys
 import copy
 import time
+import warnings
 
 from . import core
 from . import basic
@@ -32,11 +33,12 @@ from . import fixedwidth
 from ...table import Table
 from ...utils.data import get_readable_fileobj
 from ...extern import six
+from ...utils.exceptions import AstropyWarning, AstropyDeprecationWarning
 
 _read_trace = []
 
 try:
-    import yaml
+    import yaml  # pylint: disable=W0611
     HAS_YAML = True
 except ImportError:
     HAS_YAML = False
@@ -63,7 +65,7 @@ def _probably_html(table, maxchars=100000):
                 if size > maxchars:
                     break
             table = os.linesep.join(table[:i+1])
-        except:
+        except Exception:
             pass
 
     if isinstance(table, six.string_types):
@@ -186,9 +188,9 @@ def read(table, guess=None, **kwargs):
 
     Parameters
     ----------
-    table : str, file-like, list
-        Input table as a file name, file-like object, list of strings, or
-        single newline-separated string.
+    table : str, file-like, list, pathlib.Path object
+        Input table as a file name, file-like object, list of strings,
+        single newline-separated string or pathlib.Path object .
     guess : bool
         Try to guess the table format (default= ``True``)
     format : str, `~astropy.io.ascii.BaseReader`
@@ -230,9 +232,18 @@ def read(table, guess=None, **kwargs):
         List of names to include in fill_values (default= ``None`` selects all names)
     fill_exclude_names : list
         List of names to exclude from fill_values (applied after ``fill_include_names``)
-    fast_reader : bool
+    fast_reader : bool or dict
         Whether to use the C engine, can also be a dict with options which default to ``False``
-        (default= ``True``)
+        (default= ``True``); parameters for options dict:
+
+        use_fast_converter: bool
+            enable faster but slightly imprecise floating point conversion method
+        parallel: bool or int
+            multiprocessing conversion using ``cpu_count()`` or ``'number'`` processes
+        exponent_style: str
+            One-character string defining the exponent or ``'Fortran'`` to auto-detect
+            Fortran-style scientific notation like ``'3.14159D+00'`` (``'E'``, ``'D'``, ``'Q'``),
+            all case-insensitive; default ``'E'``, all other imply ``use_fast_converter``
     Reader : `~astropy.io.ascii.BaseReader`
         Reader class (DEPRECATED) (default= :class:`Basic`).
 
@@ -287,7 +298,7 @@ def read(table, guess=None, **kwargs):
                     table = fileobj.read()
             except ValueError:  # unreadable or invalid binary file
                 raise
-            except:
+            except Exception:
                 pass
             else:
                 # Ensure that `table` has at least one \r or \n in it
@@ -361,8 +372,18 @@ def _guess(table, read_kwargs, format, fast_reader):
     format : str
         Table format
     fast_reader : bool
+    fast_reader : bool or dict
         Whether to use the C engine, can also be a dict with options which
-        default to ``False`` (default= ``True``)
+        default to ``False`` (default= ``True``); parameters for options dict:
+
+        use_fast_converter: bool
+            enable faster but slightly imprecise floating point conversion method
+        parallel: bool or int
+            multiprocessing conversion using ``cpu_count()`` or ``'number'`` processes
+        exponent_style: str
+            Character to use for exponent or ``'Fortran'`` to auto-detect any
+            Fortran-style scientific notation like ``'3.14159D+00'`` (``'E'``, ``'D'``, ``'Q'``),
+            all case-insensitive; default ``'E'``, all other imply ``use_fast_converter``
 
     Returns
     -------
@@ -479,7 +500,7 @@ def _guess(table, read_kwargs, format, fast_reader):
                 reader_repr = repr(kwargs.get('Reader', basic.Basic))
                 keys_vals = ['Reader:' + re.search(r"\.(\w+)'>", reader_repr).group(1)]
                 kwargs_sorted = ((key, kwargs[key]) for key in sorted_keys)
-                keys_vals.extend(['%s: %s' % (key, repr(val)) for key, val in kwargs_sorted])
+                keys_vals.extend(['{}: {}'.format(key, repr(val)) for key, val in kwargs_sorted])
                 lines.append(' '.join(keys_vals))
 
             msg = ['',
@@ -562,7 +583,7 @@ def _get_guess_kwargs_list(read_kwargs):
     for Reader in (fastbasic.FastCommentedHeader, basic.CommentedHeader,
                    fastbasic.FastBasic, basic.Basic,
                    fastbasic.FastNoHeader, basic.NoHeader):
-        for delimiter in ("|", ",", " ", "\s"):
+        for delimiter in ("|", ",", " ", r"\s"):
             for quotechar in ('"', "'"):
                 guess_kwargs_list.append(dict(
                     Reader=Reader, delimiter=delimiter, quotechar=quotechar))
@@ -658,14 +679,44 @@ def write(table, output=None,  format=None, Writer=None, fast_writer=True, **kwa
         List of names to exclude from output (applied after ``include_names``)
     fast_writer : bool
         Whether to use the fast Cython writer (default= ``True``)
+    overwrite : bool
+        If ``overwrite=None`` (default) and the file exists, then a
+        warning will be issued. In a future release this will instead
+        generate an exception. If ``overwrite=False`` and the file
+        exists, then an exception is raised.
+        This parameter is ignored when the ``output`` arg is not a string
+        (e.g., a file object).
     Writer : ``Writer``
         Writer class (DEPRECATED) (default= :class:`Basic`)
+
     """
+    overwrite = kwargs.pop('overwrite', None)
+    if isinstance(output, six.string_types):
+        if os.path.lexists(output):
+            if overwrite is None:
+                warnings.warn(
+                    "{} already exists. "
+                    "Automatically overwriting ASCII files is deprecated. "
+                    "Use the argument 'overwrite=True' in the future.".format(
+                        output), AstropyDeprecationWarning)
+            elif not overwrite:
+                raise IOError("{} already exists".format(output))
+
     if output is None:
         output = sys.stdout
 
     table = Table(table, names=kwargs.get('names'))
 
+    table0 = table[:0].copy()
+    core._apply_include_exclude_names(table0, kwargs.get('names'),
+                    kwargs.get('include_names'), kwargs.get('exclude_names'))
+    diff_format_with_names = set(kwargs.get('formats', [])) - set(table0.colnames)
+
+    if diff_format_with_names:
+        warnings.warn(
+            'The keys {} specified in the formats argument does not match a column name.'
+            .format(diff_format_with_names), AstropyWarning)
+
     if table.has_mixin_columns:
         fast_writer = False
 
