@@ -55,7 +55,9 @@ explanation of all the different formats.
 """
 
 
+import operator
 import os
+import warnings
 
 import numpy as np
 
@@ -66,13 +68,17 @@ from .hdu.image import PrimaryHDU, ImageHDU
 from .hdu.table import BinTableHDU
 from .header import Header
 from .util import fileobj_closed, fileobj_name, fileobj_mode, _is_int
+from .fitsrec import FITS_rec
+from ...units.format.fits import UnitScaleError
+from ...extern import six
 from ...extern.six import string_types
-from ...utils import deprecated
+from ...utils.exceptions import AstropyUserWarning
+from ...utils.decorators import deprecated_renamed_argument
 
 
 __all__ = ['getheader', 'getdata', 'getval', 'setval', 'delval', 'writeto',
-           'append', 'update', 'info', 'tdump', 'tcreate', 'tabledump',
-           'tableload']
+           'append', 'update', 'info', 'tabledump', 'tableload',
+           'table_to_hdu']
 
 
 def getheader(filename, *args, **kwargs):
@@ -100,9 +106,18 @@ def getheader(filename, *args, **kwargs):
 
     mode, closed = _get_file_mode(filename)
     hdulist, extidx = _getext(filename, mode, *args, **kwargs)
-    hdu = hdulist[extidx]
-    header = hdu.header
-    hdulist.close(closed=closed)
+    try:
+        hdu = hdulist[extidx]
+        header = hdu.header
+    finally:
+        # Use _close instead of close to close without loading any
+        # remaining HDUs for pre-lazy-loading backwards compatibility
+        # In other words, when the full HDUList is opened by a user they
+        # previously expected to be able to look at arbitrary HDUs even
+        # after the file was closed, but for getheader and other convenience
+        # functions this is irrelevant
+        hdulist._close(closed=closed)
+
     return header
 
 
@@ -184,26 +199,29 @@ def getdata(filename, *args, **kwargs):
     view = kwargs.pop('view', None)
 
     hdulist, extidx = _getext(filename, mode, *args, **kwargs)
-    hdu = hdulist[extidx]
-    data = hdu.data
-    if data is None and extidx == 0:
-        try:
-            hdu = hdulist[1]
-            data = hdu.data
-        except IndexError:
+    try:
+        hdu = hdulist[extidx]
+        data = hdu.data
+        if data is None and extidx == 0:
+            try:
+                hdu = hdulist[1]
+                data = hdu.data
+            except IndexError:
+                raise IndexError('No data in this HDU.')
+        if data is None:
             raise IndexError('No data in this HDU.')
-    if data is None:
-        raise IndexError('No data in this HDU.')
-    if header:
-        hdr = hdu.header
-    hdulist.close(closed=closed)
+        if header:
+            hdr = hdu.header
+    finally:
+        # _close instead of close; see note in getheader
+        hdulist._close(closed=closed)
 
     # Change case of names if requested
     trans = None
     if lower:
-        trans = lambda s: s.lower()
+        trans = operator.methodcaller('lower')
     elif upper:
-        trans = lambda s: s.upper()
+        trans = operator.methodcaller('upper')
     if trans:
         if data.dtype.names is None:
             # this data does not have fields
@@ -325,11 +343,15 @@ def setval(filename, keyword, *args, **kwargs):
     after = kwargs.pop('after', None)
     savecomment = kwargs.pop('savecomment', False)
 
+    closed = fileobj_closed(filename)
     hdulist, extidx = _getext(filename, 'update', *args, **kwargs)
-    if keyword in hdulist[extidx].header and savecomment:
-        comment = None
-    hdulist[extidx].header.set(keyword, value, comment, before, after)
-    hdulist.close()
+    try:
+        if keyword in hdulist[extidx].header and savecomment:
+            comment = None
+        hdulist[extidx].header.set(keyword, value, comment, before, after)
+    finally:
+        # _close instead of close; see note in getheader
+        hdulist._close(closed=closed)
 
 
 def delval(filename, keyword, *args, **kwargs):
@@ -362,13 +384,18 @@ def delval(filename, keyword, *args, **kwargs):
     if 'do_not_scale_image_data' not in kwargs:
         kwargs['do_not_scale_image_data'] = True
 
+    closed = fileobj_closed(filename)
     hdulist, extidx = _getext(filename, 'update', *args, **kwargs)
-    del hdulist[extidx].header[keyword]
-    hdulist.close()
+    try:
+        del hdulist[extidx].header[keyword]
+    finally:
+        # _close instead of close; see note in getheader
+        hdulist._close(closed=closed)
 
 
+@deprecated_renamed_argument('clobber', 'overwrite', '1.3')
 def writeto(filename, data, header=None, output_verify='exception',
-            clobber=False, checksum=False):
+            overwrite=False, checksum=False):
     """
     Create a new FITS file using the supplied data/header.
 
@@ -393,9 +420,13 @@ def writeto(filename, data, header=None, output_verify='exception',
         ``+warn``, or ``+exception" (e.g. ``"fix+warn"``).  See :ref:`verify`
         for more info.
 
-    clobber : bool, optional
-        If `True`, and if filename already exists, it will overwrite
-        the file.  Default is `False`.
+    overwrite : bool, optional
+        If ``True``, overwrite the output file if it exists. Raises an
+        ``OSError`` (``IOError`` for Python 2) if ``False`` and the
+        output file exists. Default is ``False``.
+
+        .. versionchanged:: 1.3
+           ``overwrite`` replaces the deprecated ``clobber`` argument.
 
     checksum : bool, optional
         If `True`, adds both ``DATASUM`` and ``CHECKSUM`` cards to the
@@ -405,10 +436,107 @@ def writeto(filename, data, header=None, output_verify='exception',
     hdu = _makehdu(data, header)
     if hdu.is_image and not isinstance(hdu, PrimaryHDU):
         hdu = PrimaryHDU(data, header=header)
-    hdu.writeto(filename, clobber=clobber, output_verify=output_verify,
+    hdu.writeto(filename, overwrite=overwrite, output_verify=output_verify,
                 checksum=checksum)
 
 
+def table_to_hdu(table):
+    """
+    Convert an astropy.table.Table object to a FITS BinTableHDU
+
+    Parameters
+    ----------
+    table : astropy.table.Table
+        The table to convert.
+
+    Returns
+    -------
+    table_hdu : astropy.io.fits.BinTableHDU
+        The FITS binary table HDU
+    """
+    # Avoid circular imports
+    from .connect import is_column_keyword, REMOVE_KEYWORDS
+
+    # Tables with mixin columns are not supported
+    if table.has_mixin_columns:
+        mixin_names = [name for name, col in table.columns.items()
+                       if not isinstance(col, table.ColumnClass)]
+        raise ValueError('cannot write table with mixin column(s) {0}'
+                         .format(mixin_names))
+
+    # Create a new HDU object
+    if table.masked:
+        #float column's default mask value needs to be Nan
+        for column in six.itervalues(table.columns):
+            fill_value = column.get_fill_value()
+            if column.dtype.kind == 'f' and np.allclose(fill_value, 1e20):
+                column.set_fill_value(np.nan)
+
+        fits_rec = FITS_rec.from_columns(np.array(table.filled()))
+        table_hdu = BinTableHDU(fits_rec)
+        for col in table_hdu.columns:
+            # Binary FITS tables support TNULL *only* for integer data columns
+            # TODO: Determine a schema for handling non-integer masked columns
+            # in FITS (if at all possible)
+            int_formats = ('B', 'I', 'J', 'K')
+            if not (col.format in int_formats or
+                    col.format.p_format in int_formats):
+                continue
+
+            # The astype is necessary because if the string column is less
+            # than one character, the fill value will be N/A by default which
+            # is too long, and so no values will get masked.
+            fill_value = table[col.name].get_fill_value()
+
+            col.null = fill_value.astype(table[col.name].dtype)
+    else:
+        fits_rec = FITS_rec.from_columns(np.array(table.filled()))
+        table_hdu = BinTableHDU(fits_rec)
+
+    # Set units for output HDU
+    for col in table_hdu.columns:
+        unit = table[col.name].unit
+        if unit is not None:
+            try:
+                col.unit = unit.to_string(format='fits')
+            except UnitScaleError:
+                scale = unit.scale
+                raise UnitScaleError(
+                    "The column '{0}' could not be stored in FITS format "
+                    "because it has a scale '({1})' that "
+                    "is not recognized by the FITS standard. Either scale "
+                    "the data or change the units.".format(col.name, str(scale)))
+            except ValueError:
+                warnings.warn(
+                    "The unit '{0}' could not be saved to FITS format".format(
+                        unit.to_string()), AstropyUserWarning)
+
+    for key, value in table.meta.items():
+        if is_column_keyword(key.upper()) or key.upper() in REMOVE_KEYWORDS:
+            warnings.warn(
+                "Meta-data keyword {0} will be ignored since it conflicts "
+                "with a FITS reserved keyword".format(key), AstropyUserWarning)
+
+        if isinstance(value, list):
+            for item in value:
+                try:
+                    table_hdu.header.append((key, item))
+                except ValueError:
+                    warnings.warn(
+                        "Attribute `{0}` of type {1} cannot be added to "
+                        "FITS Header - skipping".format(key, type(value)),
+                        AstropyUserWarning)
+        else:
+            try:
+                table_hdu.header[key] = value
+            except ValueError:
+                warnings.warn(
+                    "Attribute `{0}` of type {1} cannot be added to FITS "
+                    "Header - skipping".format(key, type(value)),
+                    AstropyUserWarning)
+    return table_hdu
+
+
 def append(filename, data, header=None, checksum=False, verify=True, **kwargs):
     """
     Append the header/data to FITS file if filename exists, create if not.
@@ -461,17 +589,22 @@ def append(filename, data, header=None, checksum=False, verify=True, **kwargs):
 
         if verify or not closed:
             f = fitsopen(filename, mode='append')
-            f.append(hdu)
+            try:
+                f.append(hdu)
 
-            # Set a flag in the HDU so that only this HDU gets a checksum when
-            # writing the file.
-            hdu._output_checksum = checksum
-            f.close(closed=closed)
+                # Set a flag in the HDU so that only this HDU gets a checksum
+                # when writing the file.
+                hdu._output_checksum = checksum
+            finally:
+                # _close instead of close; see note in getheader
+                f._close(closed=closed)
         else:
             f = _File(filename, mode='append')
-            hdu._output_checksum = checksum
-            hdu._writeto(f)
-            f.close()
+            try:
+                hdu._output_checksum = checksum
+                hdu._writeto(f)
+            finally:
+                f._close()
 
 
 def update(filename, data, *args, **kwargs):
@@ -527,9 +660,11 @@ def update(filename, data, *args, **kwargs):
     closed = fileobj_closed(filename)
 
     hdulist, _ext = _getext(filename, 'update', *args, **kwargs)
-    hdulist[_ext] = new_hdu
-
-    hdulist.close(closed=closed)
+    try:
+        hdulist[_ext] = new_hdu
+    finally:
+        # _close instead of close; see note in getheader
+        hdulist._close(closed=closed)
 
 
 def info(filename, output=None, **kwargs):
@@ -558,20 +693,23 @@ def info(filename, output=None, **kwargs):
 
     mode, closed = _get_file_mode(filename, default='readonly')
     # Set the default value for the ignore_missing_end parameter
-    if not 'ignore_missing_end' in kwargs:
+    if 'ignore_missing_end' not in kwargs:
         kwargs['ignore_missing_end'] = True
 
     f = fitsopen(filename, mode=mode, **kwargs)
-    ret = f.info(output=output)
-
-    if closed:
-        f.close()
+    try:
+        ret = f.info(output=output)
+    finally:
+        if closed:
+            # _close instead of close; see note in getheader
+            f._close()
 
     return ret
 
 
+@deprecated_renamed_argument('clobber', 'overwrite', '1.3')
 def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
-              clobber=False):
+              overwrite=False):
     """
     Dump a table HDU to a file in ASCII format.  The table may be
     dumped in three separate files, one containing column definitions,
@@ -599,14 +737,19 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
         The number of the extension containing the table HDU to be
         dumped.
 
-    clobber : bool
-        Overwrite the output files if they exist.
+    overwrite : bool, optional
+        If ``True``, overwrite the output file if it exists. Raises an
+        ``OSError`` (``IOError`` for Python 2) if ``False`` and the
+        output file exists. Default is ``False``.
+
+        .. versionchanged:: 1.3
+           ``overwrite`` replaces the deprecated ``clobber`` argument.
 
     Notes
     -----
     The primary use for the `tabledump` function is to allow editing in a
     standard text editor of the table data and parameters.  The
-    ``tcreate`` function can be used to reassemble the table from the
+    `tableload` function can be used to reassemble the table from the
     three ASCII files.
     """
 
@@ -618,29 +761,24 @@ def tabledump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
     f = fitsopen(filename, mode=mode)
 
     # Create the default data file name if one was not provided
+    try:
+        if not datafile:
+            # TODO: Really need to provide a better way to access the name of
+            # any files underlying an HDU
+            root, tail = os.path.splitext(f._HDUList__file.name)
+            datafile = root + '_' + repr(ext) + '.txt'
 
-    if not datafile:
-        # TODO: Really need to provide a better way to access the name of any
-        # files underlying an HDU
-        root, tail = os.path.splitext(f._HDUList__file.name)
-        datafile = root + '_' + repr(ext) + '.txt'
-
-    # Dump the data from the HDU to the files
-    f[ext].dump(datafile, cdfile, hfile, clobber)
-
-    if closed:
-        f.close()
+        # Dump the data from the HDU to the files
+        f[ext].dump(datafile, cdfile, hfile, overwrite)
+    finally:
+        if closed:
+            # _close instead of close; see note in getheader
+            f._close()
 
 if isinstance(tabledump.__doc__, string_types):
     tabledump.__doc__ += BinTableHDU._tdump_file_format.replace('\n', '\n    ')
 
 
-@deprecated('0.1', alternative=':func:`tabledump`')
-def tdump(filename, datafile=None, cdfile=None, hfile=None, ext=1,
-          clobber=False):
-    tabledump(filename, datafile, cdfile, hfile, ext, clobber)
-
-
 def tableload(datafile, cdfile, hfile=None):
     """
     Create a table from the input ASCII files.  The input is from up
@@ -679,11 +817,6 @@ if isinstance(tableload.__doc__, string_types):
     tableload.__doc__ += BinTableHDU._tdump_file_format.replace('\n', '\n    ')
 
 
-@deprecated('0.1', alternative=':func:`tableload`')
-def tcreate(datafile, cdfile, hfile=None):
-    return tableload(datafile, cdfile, hfile)
-
-
 def _getext(filename, mode, *args, **kwargs):
     """
     Open the input file, return the `HDUList` and the extension.
@@ -696,9 +829,9 @@ def _getext(filename, mode, *args, **kwargs):
     extname = kwargs.pop('extname', None)
     extver = kwargs.pop('extver', None)
 
-    err_msg = ('Redundant/conflicting extension arguments(s): %s' %
-               ({'args': args, 'ext': ext,  'extname': extname,
-                 'extver': extver},))
+    err_msg = ('Redundant/conflicting extension arguments(s): {}'.format(
+            {'args': args, 'ext': ext,  'extname': extname,
+             'extver': extver}))
 
     # This code would be much simpler if just one way of specifying an
     # extension were picked.  But now we need to support all possible ways for
@@ -807,7 +940,7 @@ def _get_file_mode(filename, default='readonly'):
         mode = FILE_MODES.get(fmode)
         if mode is None:
             raise IOError(
-                "File mode of the input file object (%r) cannot be used to "
-                "read/write FITS files." % fmode)
+                "File mode of the input file object ({!r}) cannot be used to "
+                "read/write FITS files.".format(fmode))
 
     return mode, closed
