@@ -18,10 +18,12 @@ from ..util import (_is_int, _is_pseudo_unsigned, _unsigned_zero,
 from ..verify import _Verify, _ErrList
 
 from ....extern.six import string_types, add_metaclass
-from ....utils import lazyproperty, deprecated
-from ....utils.compat import ignored
+from ....extern.six.moves import range
+from ....utils import lazyproperty
+from ....utils.compat import suppress
 from ....utils.compat.funcsigs import signature, Parameter
 from ....utils.exceptions import AstropyUserWarning
+from ....utils.decorators import deprecated_renamed_argument
 
 
 class _Delayed(object):
@@ -230,7 +232,7 @@ class _BaseHDU(object):
             ('XTENSION' in self._header and
              (self._header['XTENSION'] == 'IMAGE' or
               (self._header['XTENSION'] == 'BINTABLE' and
-               'ZIMAGE' in self._header and self._header['ZIMAGE'] == True))))
+               'ZIMAGE' in self._header and self._header['ZIMAGE'] is True))))
 
     @property
     def _data_loaded(self):
@@ -240,102 +242,6 @@ class _BaseHDU(object):
     def _has_data(self):
         return self._data_loaded and self.data is not None
 
-    @property
-    @deprecated('0.3', alternative='the `._header_offset` attribute',
-                pending=True)
-    def _hdrLoc(self):
-        """The byte offset of this HDU's header in the file it came from;
-        available for backwards compatibility--use ._header_offset instead.
-        """
-
-        return self._header_offset
-
-    @_hdrLoc.setter
-    @deprecated('0.3', alternative='the `._header_offset` attribute',
-                pending=True)
-    def _hdrLoc(self, value):
-        self._header_offset = value
-
-    @property
-    @deprecated('0.3', alternative='the `._data_offset` attribute',
-                pending=True)
-    def _datLoc(self):
-        """The byte offset of this HDU's data portion in the file it came from;
-        available for backwards compatibility--use ._data_offset instead.
-        """
-
-        return self._data_offset
-
-    @_datLoc.setter
-    @deprecated('0.3', alternative='the `._data_offset` attribute',
-                pending=True)
-    def _datLoc(self, value):
-        self._data_offset = value
-
-    @property
-    @deprecated('0.3', alternative='the `._data_size` attribute',
-                pending=True)
-    def _datSpan(self):
-        """The byte size of this HDU's data portion in the file it came from;
-        available for backwards compatibility--use ._data_size instead.
-        """
-
-        return self._data_size
-
-    @_datSpan.setter
-    @deprecated('0.3', alternative='the `._data_size` attribute',
-                pending=True)
-    def _datSpan(self, value):
-        self._data_size = value
-
-    @property
-    @deprecated('0.3', alternative='the `._header_offset` attribute',
-                pending=True)
-    def _hdrLoc(self):
-        """The byte offset of this HDU's header in the file it came from;
-        available for backwards compatibility--use ._header_offset instead.
-        """
-
-        return self._header_offset
-
-    @_hdrLoc.setter
-    @deprecated('0.3', alternative='the `._header_offset` attribute',
-                pending=True)
-    def _hdrLoc(self, value):
-        self._header_offset = value
-
-    @property
-    @deprecated('0.3', alternative='the `._data_offset` attribute',
-                pending=True)
-    def _datLoc(self):
-        """The byte offset of this HDU's data portion in the file it came from;
-        available for backwards compatibility--use ._data_offset instead.
-        """
-
-        return self._data_offset
-
-    @_datLoc.setter
-    @deprecated('0.3', alternative='the `._data_offset` attribute',
-                pending=True)
-    def _datLoc(self, value):
-        self._data_offset = value
-
-    @property
-    @deprecated('0.3', alternative='the `._data_size` attribute',
-                pending=True)
-    def _datSpan(self):
-        """The byte size of this HDU's data portion in the file it came from;
-        available for backwards compatibility--use ._data_size instead.
-        """
-
-        return self._data_size
-
-    @_datSpan.setter
-    @deprecated('0.3', alternative='the `._data_size` attribute',
-                pending=True)
-    def _datSpan(self, value):
-        self._data_size = value
-
     @classmethod
     def register_hdu(cls, hducls):
         cls._hdu_registry.add(hducls)
@@ -427,10 +333,11 @@ class _BaseHDU(object):
         fileobj.seek(hdu._data_offset + hdu._data_size, os.SEEK_SET)
         return hdu
 
-    def writeto(self, name, output_verify='exception', clobber=False,
+    @deprecated_renamed_argument('clobber', 'overwrite', '1.3')
+    def writeto(self, name, output_verify='exception', overwrite=False,
                 checksum=False):
         """
-        Write the HDU to a new file.  This is a convenience method to
+        Write the HDU to a new file. This is a convenience method to
         provide a user easier output interface if only one HDU needs
         to be written to a file.
 
@@ -447,8 +354,13 @@ class _BaseHDU(object):
             ``"silentfix"`` with ``"+ignore"``, ``+warn``, or ``+exception"
             (e.g. ``"fix+warn"``).  See :ref:`verify` for more info.
 
-        clobber : bool
-            Overwrite the output file if exists.
+        overwrite : bool, optional
+            If ``True``, overwrite the output file if it exists. Raises an
+            ``OSError`` (``IOError`` for Python 2) if ``False`` and the
+            output file exists. Default is ``False``.
+
+            .. versionchanged:: 1.3
+               ``overwrite`` replaces the deprecated ``clobber`` argument.
 
         checksum : bool
             When `True` adds both ``DATASUM`` and ``CHECKSUM`` cards
@@ -458,7 +370,7 @@ class _BaseHDU(object):
         from .hdulist import HDUList
 
         hdulist = HDUList([self])
-        hdulist.writeto(name, output_verify, clobber=clobber,
+        hdulist.writeto(name, output_verify, overwrite=overwrite,
                         checksum=checksum)
 
     @classmethod
@@ -477,27 +389,25 @@ class _BaseHDU(object):
         header_offset = 0
 
         if isinstance(data, _File):
-            from_file = True
             if header is None:
                 header_offset = data.tell()
                 header = Header.fromfile(data, endcard=not ignore_missing_end)
             hdu_fileobj = data
             data_offset = data.tell()  # *after* reading the header
         else:
-            from_file = False
             try:
                 # Test that the given object supports the buffer interface by
                 # ensuring an ndarray can be created from it
                 np.ndarray((), dtype='ubyte', buffer=data)
             except TypeError:
                 raise TypeError(
-                    'The provided object %r does not contain an underlying '
+                    'The provided object {!r} does not contain an underlying '
                     'memory buffer.  fromstring() requires an object that '
                     'supports the buffer interface such as bytes, str '
                     '(in Python 2.x but not in 3.x), buffer, memoryview, '
                     'ndarray, etc.  This restriction is to ensure that '
                     'efficient access to the array/table data is possible.'
-                    % data)
+                    .format(data))
 
             if header is None:
                 def block_iter(nbytes):
@@ -646,13 +556,13 @@ class _BaseHDU(object):
         if (self._has_data and self._standard and
                 _is_pseudo_unsigned(self.data.dtype)):
             for keyword in ('BSCALE', 'BZERO'):
-                with ignored(KeyError):
+                with suppress(KeyError):
                     del self._header[keyword]
 
     def _writeheader(self, fileobj):
         offset = 0
         if not fileobj.simulateonly:
-            with ignored(AttributeError, IOError):
+            with suppress(AttributeError, IOError):
                 offset = fileobj.tell()
 
             self._header.tofile(fileobj)
@@ -892,7 +802,7 @@ class _NonstandardHDU(_BaseHDU, _Verify):
         # The check that 'GROUPS' is missing is a bit redundant, since the
         # match_header for GroupsHDU will always be called before this one.
         if card.keyword == 'SIMPLE':
-            if 'GROUPS' not in header and card.value == False:
+            if 'GROUPS' not in header and card.value is False:
                 return True
             else:
                 raise InvalidHDUException
@@ -1056,102 +966,10 @@ class _ValidHDU(_BaseHDU, _Verify):
             data = None
         return self.__class__(data=data, header=self._header.copy())
 
-    @deprecated('0.3', alternative='the ``.name`` attribute or `Header.set`',
-                pending=True)
-    def update_ext_name(self, value, comment=None, before=None,
-                        after=None, savecomment=False):
-        """
-        Update the extension name associated with the HDU.
-
-        If the keyword already exists in the Header, it's value and/or comment
-        will be updated.  If it does not exist, a new card will be created
-        and it will be placed before or after the specified location.
-        If no ``before`` or ``after`` is specified, it will be appended at
-        the end.
-
-        Parameters
-        ----------
-        value : str
-            Value to be used for the new extension name
-
-        comment : str, optional
-            To be used for updating, default=None.
-
-        before : str or int, optional
-            Name of the keyword, or index of the `Card` before which the new
-            card will be placed in the Header.  The argument ``before`` takes
-            precedence over ``after`` if both are specified.
-
-        after : str or int, optional
-            Name of the keyword, or index of the `Card` after which
-            the new card will be placed in the Header
-
-        savecomment : bool, optional
-            When `True`, preserve the current comment for an existing
-            keyword.  The argument ``savecomment`` takes precedence over
-            ``comment`` if both specified.  If ``comment`` is not
-            specified then the current comment will automatically be
-            preserved.
-        """
-
-        if 'EXTNAME' in self._header and savecomment:
-            comment = None
-
-        self._header.set('EXTNAME', value, comment, before, after)
-        # This may seem redundant, but the previous header.set call just
-        # handles anyone who might use the before/after keywords to set the
-        # position of the EXTNAME keyword.  Setting self.name = name does some
-        # additional processing on the value such as handling
-        # conf.extension_name_case_sensitive
-        self.name = value
-
-    @deprecated('0.3', alternative='the ``.ver`` attribute or `Header.set`',
-                pending=True)
-    def update_ext_version(self, value, comment=None, before=None,
-                           after=None, savecomment=False):
-        """
-        Update the extension version associated with the HDU.
-
-        If the keyword already exists in the Header, it's value and/or comment
-        will be updated.  If it does not exist, a new card will be created
-        and it will be placed before or after the specified location.
-        If no ``before`` or ``after`` is specified, it will be appended at
-        the end.
-
-        Parameters
-        ----------
-        value : str
-            Value to be used for the new extension version
-
-        comment : str, optional
-            To be used for updating; default=None.
-
-        before : str or int, optional
-            Name of the keyword, or index of the `Card` before which the new
-            card will be placed in the Header.  The argument ``before`` takes
-            precedence over ``after`` if both are specified.
-
-        after : str or int, optional
-            Name of the keyword, or index of the `Card` after which
-            the new card will be placed in the Header.
-
-        savecomment : bool, optional
-            When `True`, preserve the current comment for an existing
-            keyword.  The argument ``savecomment`` takes precedence over
-            ``comment`` if both specified.  If ``comment`` is not
-            specified then the current comment will automatically be
-            preserved.
-        """
-
-        if 'EXTVER' in self._header and savecomment:
-            comment = None
-
-        self._header.set('EXTVER', value, comment, before, after)
-
     def _verify(self, option='warn'):
         errs = _ErrList([], unit='Card')
 
-        is_valid = lambda v: v in [8, 16, 32, 64, -32, -64]
+        is_valid = BITPIX2DTYPE.__contains__
 
         # Verify location and value of mandatory keywords.
         # Do the first card here, instead of in the respective HDU classes, so
@@ -1185,8 +1003,8 @@ class _ValidHDU(_BaseHDU, _Verify):
                         if number <= 0 or number > naxis:
                             raise ValueError
                     except ValueError:
-                        err_text = ("NAXISj keyword out of range ('%s' when "
-                                    "NAXIS == %d)" % (keyword, naxis))
+                        err_text = ("NAXISj keyword out of range ('{}' when "
+                                    "NAXIS == {})".format(keyword, naxis))
 
                         def fix(self=self, keyword=keyword):
                             del self._header[keyword]
@@ -1286,8 +1104,8 @@ class _ValidHDU(_BaseHDU, _Verify):
 
         # if the card does not exist
         if index is None:
-            err_text = "'%s' card does not exist." % keyword
-            fix_text = "Fixed by inserting a new '%s' card." % keyword
+            err_text = "'{}' card does not exist.".format(keyword)
+            fix_text = "Fixed by inserting a new '{}' card.".format(keyword)
             if fixable:
                 # use repr to accommodate both string and non-string types
                 # Boolean is also OK in this constructor
@@ -1302,11 +1120,10 @@ class _ValidHDU(_BaseHDU, _Verify):
             # if the supposed location is specified
             if pos is not None:
                 if not pos(index):
-                    err_text = ("'%s' card at the wrong place (card %d)." %
-                                (keyword, index))
-
+                    err_text = ("'{}' card at the wrong place "
+                                "(card {}).".format(keyword, index))
                     fix_text = ("Fixed by moving it to the right place "
-                                "(card %d)." % insert_pos)
+                                "(card {}).".format(insert_pos))
 
                     def fix(self=self, index=index, insert_pos=insert_pos):
                         card = self._header.cards[index]
@@ -1320,9 +1137,10 @@ class _ValidHDU(_BaseHDU, _Verify):
             if test:
                 val = self._header[keyword]
                 if not test(val):
-                    err_text = ("'%s' card has invalid value '%s'." %
-                                (keyword, val))
-                    fix_text = "Fixed by setting a new value '%s'." % fix_value
+                    err_text = ("'{}' card has invalid value '{}'.".format(
+                            keyword, val))
+                    fix_text = ("Fixed by setting a new value '{}'.".format(
+                            fix_value))
 
                     if fixable:
                         def fix(self=self, keyword=keyword, val=fix_value):
@@ -1369,7 +1187,7 @@ class _ValidHDU(_BaseHDU, _Verify):
         cs = self._calculate_datasum(blocking)
 
         if when is None:
-            when = 'data unit checksum updated %s' % self._get_timestamp()
+            when = 'data unit checksum updated {}'.format(self._get_timestamp())
 
         self._header[datasum_keyword] = (str(cs), when)
         return cs
@@ -1422,7 +1240,7 @@ class _ValidHDU(_BaseHDU, _Verify):
             data_cs = self._calculate_datasum(blocking)
 
         if when is None:
-            when = 'HDU checksum updated %s' % self._get_timestamp()
+            when = 'HDU checksum updated {}'.format(self._get_timestamp())
 
         # Add the CHECKSUM card to the header with a value of all zeros.
         if datasum_keyword in self._header:
@@ -1754,19 +1572,23 @@ class ExtensionHDU(_ValidHDU):
 
         raise NotImplementedError
 
-    def writeto(self, name, output_verify='exception', clobber=False,
+    @deprecated_renamed_argument('clobber', 'overwrite', '1.3')
+    def writeto(self, name, output_verify='exception', overwrite=False,
                 checksum=False):
         """
         Works similarly to the normal writeto(), but prepends a default
         `PrimaryHDU` are required by extension HDUs (which cannot stand on
         their own).
+
+        .. versionchanged:: 1.3
+           ``overwrite`` replaces the deprecated ``clobber`` argument.
         """
 
         from .hdulist import HDUList
         from .image import PrimaryHDU
 
         hdulist = HDUList([PrimaryHDU(), self])
-        hdulist.writeto(name, output_verify, clobber=clobber,
+        hdulist.writeto(name, output_verify, overwrite=overwrite,
                         checksum=checksum)
 
     def _verify(self, option='warn'):
