@@ -11,6 +11,7 @@ from ..fitsrec import FITS_rec, FITS_record
 from ..util import _is_int, _is_pseudo_unsigned, _unsigned_zero
 
 from ....utils import lazyproperty
+from ....extern.six.moves import range, zip
 
 
 class Group(FITS_record):
@@ -80,8 +81,8 @@ class Group(FITS_record):
                     for i in range(len(indx)):
                         self.array[self.row][indx[i]] = value[i]
                 else:
-                    raise ValueError('Parameter value must be a sequence '
-                                     'with %d arrays/numbers.' % len(indx))
+                    raise ValueError('Parameter value must be a sequence with '
+                                     '{} arrays/numbers.'.format(len(indx)))
 
 
 class GroupData(FITS_rec):
@@ -140,7 +141,7 @@ class GroupData(FITS_rec):
                 parbzeros = [None] * npars
 
             if parnames is None:
-                parnames = ['PAR%d' % (idx + 1) for idx in range(npars)]
+                parnames = ['PAR{}'.format(idx + 1) for idx in range(npars)]
 
             if len(parnames) != npars:
                 raise ValueError('The number of parameter data arrays does '
@@ -153,7 +154,7 @@ class GroupData(FITS_rec):
 
             fits_fmt = GroupsHDU._bitpix2tform[bitpix]  # -32 -> 'E'
             format = FITS2NUMPY[fits_fmt]  # 'E' -> 'f4'
-            data_fmt = '%s%s' % (str(input.shape[1:]), format)
+            data_fmt = '{}{}'.format(str(input.shape[1:]), format)
             formats = ','.join(([format] * npars) + [data_fmt])
             gcount = input.shape[0]
 
@@ -279,7 +280,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
     def match_header(cls, header):
         keyword = header.cards[0].keyword
         return (keyword == 'SIMPLE' and 'GROUPS' in header and
-                header['GROUPS'] == True)
+                header['GROUPS'] is True)
 
     @lazyproperty
     def data(self):
@@ -471,7 +472,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
                 # Convert the unsigned array to signed
                 output = np.array(
                     self.data - _unsigned_zero(self.data.dtype),
-                    dtype='>i%d' % self.data.dtype.itemsize)
+                    dtype='>i{}'.format(self.data.dtype.itemsize))
                 should_swap = False
             else:
                 output = self.data
@@ -507,7 +508,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
 
         self.req_cards('GCOUNT', pos, _is_int, 1, option, errs)
         self.req_cards('PCOUNT', pos, _is_int, 0, option, errs)
-        self.req_cards('GROUPS', pos, lambda v: (v == True), True, option,
+        self.req_cards('GROUPS', pos, lambda v: (v is True), True, option,
                        errs)
         return errs
 
@@ -564,7 +565,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
                 format = self.columns[0].dtype.name
 
         # Update the GCOUNT report
-        gcount = '%d Groups  %d Parameters' % (self._gcount, self._pcount)
+        gcount = '{} Groups  {} Parameters'.format(self._gcount, self._pcount)
         return (name, classname, length, shape, format, gcount)
 
 
