@@ -11,6 +11,7 @@ from ..util import _is_pseudo_unsigned, _unsigned_zero, _is_int
 from ..verify import VerifyWarning
 
 from ....extern.six import string_types
+from ....extern.six.moves import range, zip
 from ....utils import isiterable, lazyproperty, classproperty, deprecated
 
 
@@ -99,6 +100,11 @@ class _ImageBaseHDU(_ValidHDU):
         self._uint = uint
         self._scale_back = scale_back
 
+        # Keep track of whether BZERO/BSCALE were set from the header so that
+        # values for self._orig_bzero and self._orig_bscale can be set
+        # properly, if necessary, once the data has been set.
+        bzero_in_header = 'BZERO' in self._header
+        bscale_in_header = 'BSCALE' in self._header
         self._bzero = self._header.get('BZERO', 0)
         self._bscale = self._header.get('BSCALE', 1)
 
@@ -106,16 +112,23 @@ class _ImageBaseHDU(_ValidHDU):
         # the image data
         self._axes = [self._header.get('NAXIS' + str(axis + 1), 0)
                       for axis in range(self._header.get('NAXIS', 0))]
-        self._bitpix = self._header.get('BITPIX', 8)
+
+        # Not supplying a default for BITPIX makes sense because BITPIX
+        # is either in the header or should be determined from the dtype of
+        # the data (which occurs when the data is set).
+        self._bitpix = self._header.get('BITPIX')
         self._gcount = self._header.get('GCOUNT', 1)
         self._pcount = self._header.get('PCOUNT', 0)
         self._blank = None if ignore_blank else self._header.get('BLANK')
         self._verify_blank()
 
         self._orig_bitpix = self._bitpix
+        self._orig_blank = self._header.get('BLANK')
+
+        # These get set again below, but need to be set to sensible defaults
+        # here.
         self._orig_bzero = self._bzero
         self._orig_bscale = self._bscale
-        self._orig_blank = self._header.get('BLANK')
 
         # Set the name attribute if it was provided (if this is an ImageHDU
         # this will result in setting the EXTNAME keyword of the header as
@@ -135,9 +148,31 @@ class _ImageBaseHDU(_ValidHDU):
                 self._data_needs_rescale = True
             return
         else:
+            # Setting data will set set _bitpix, _bzero, and _bscale to the
+            # appropriate BITPIX for the data, and always sets _bzero=0 and
+            # _bscale=1.
             self.data = data
             self.update_header()
 
+            # Check again for BITPIX/BSCALE/BZERO in case they changed when the
+            # data was assigned. This can happen, for example, if the input
+            # data is an unsigned int numpy array.
+            self._bitpix = self._header.get('BITPIX')
+
+            # Do not provide default values for BZERO and BSCALE here because
+            # the keywords will have been deleted in the header if appropriate
+            # after scaling. We do not want to put them back in if they
+            # should not be there.
+            self._bzero = self._header.get('BZERO')
+            self._bscale = self._header.get('BSCALE')
+
+        # Handle case where there was no BZERO/BSCALE in the initial header
+        # but there should be a BSCALE/BZERO now that the data has been set.
+        if not bzero_in_header:
+            self._orig_bzero = self._bzero
+        if bscale_in_header:
+            self._orig_bscale = self._bscale
+
     @classmethod
     def match_header(cls, header):
         """
@@ -212,31 +247,34 @@ class _ImageBaseHDU(_ValidHDU):
 
     @data.setter
     def data(self, data):
-        if 'data' in self.__dict__:
+        if 'data' in self.__dict__ and self.__dict__['data'] is not None:
             if self.__dict__['data'] is data:
                 return
             else:
                 self._data_replaced = True
+            was_unsigned = _is_pseudo_unsigned(self.__dict__['data'].dtype)
         else:
             self._data_replaced = True
+            was_unsigned = False
 
         if data is not None and not isinstance(data, np.ndarray):
             # Try to coerce the data into a numpy array--this will work, on
             # some level, for most objects
             try:
                 data = np.array(data)
-            except:
-                raise TypeError('data object %r could not be coerced into an '
-                                'ndarray' % data)
+            except Exception:
+                raise TypeError('data object {!r} could not be coerced into an '
+                                'ndarray'.format(data))
 
         self.__dict__['data'] = data
         self._modified = True
 
         if isinstance(data, np.ndarray):
+            # Set new values of bitpix, bzero, and bscale now, but wait to
+            # revise original values until header is updated.
             self._bitpix = DTYPE2BITPIX[data.dtype.name]
-            self._orig_bitpix = self._bitpix
-            self._bscale = self._orig_bscale = 1
-            self._bzero = self._orig_bzero = 0
+            self._bscale = 1
+            self._bzero = 0
             self._blank = None
             self._axes = list(data.shape)
             self._axes.reverse()
@@ -245,7 +283,26 @@ class _ImageBaseHDU(_ValidHDU):
         else:
             raise ValueError('not a valid data array')
 
+        # Update the header, including adding BZERO/BSCALE if new data is
+        # unsigned. Does not change the values of self._bitpix,
+        # self._orig_bitpix, etc.
         self.update_header()
+        if (data is not None and was_unsigned):
+            self._update_header_scale_info(data.dtype)
+
+        # Keep _orig_bitpix as it was until header update is done, then
+        # set it, to allow easier handling of the case of unsigned
+        # integer data being converted to something else. Setting these here
+        # is needed only for the case do_not_scale_image_data=True when
+        # setting the data to unsigned int.
+
+        # If necessary during initialization, i.e. if BSCALE and BZERO were
+        # not in the header but the data was unsigned, the attributes below
+        # will be update in __init__.
+        self._orig_bitpix = self._bitpix
+        self._orig_bscale = self._bscale
+        self._orig_bzero = self._bzero
+
 
         # returning the data signals to lazyproperty that we've already handled
         # setting self.__dict__['data']
@@ -313,11 +370,30 @@ class _ImageBaseHDU(_ValidHDU):
         if 'BLANK' in self._header:
             self._blank = self._header['BLANK']
 
+        # Add BSCALE/BZERO to header if data is unsigned int.
         self._update_uint_scale_keywords()
 
         self._modified = False
 
     def _update_header_scale_info(self, dtype=None):
+        """
+        Delete BSCALE/BZERO from header if necessary.
+        """
+
+        # Note that _dtype_for_bitpix determines the dtype based on the
+        # "original" values of bitpix, bscale, and bzero, stored in
+        # self._orig_bitpix, etc. It contains the logic for determining which
+        # special cases of BZERO/BSCALE, if any, are auto-detected as following
+        # the FITS unsigned int convention.
+
+        # Added original_was_unsigned with the intent of facilitating the
+        # special case of do_not_scale_image_data=True and uint=True
+        # eventually.
+        if self._dtype_for_bitpix() is not None:
+            original_was_unsigned = self._dtype_for_bitpix().kind == 'u'
+        else:
+            original_was_unsigned = False
+
         if (self._do_not_scale_image_data or
                 (self._orig_bzero == 0 and self._orig_bscale == 1)):
             return
@@ -439,7 +515,10 @@ class _ImageBaseHDU(_ValidHDU):
         # Do the scaling
         if _zero != 0:
             # 0.9.6.3 to avoid out of range error for BZERO = +32768
-            self.data += -_zero
+            # We have to explcitly cast _zero to prevent numpy from raising an
+            # error when doing self.data -= zero, and we do this instead of
+            # self.data = self.data - zero to avoid doubling memory usage.
+            np.add(self.data, -_zero, out=self.data, casting='unsafe')
             self._header['BZERO'] = _zero
         else:
             try:
@@ -543,7 +622,7 @@ class _ImageBaseHDU(_ValidHDU):
                 # Convert the unsigned array to signed
                 output = np.array(
                     self.data - _unsigned_zero(self.data.dtype),
-                    dtype='>i%d' % self.data.dtype.itemsize)
+                    dtype='>i{}'.format(self.data.dtype.itemsize))
                 should_swap = False
             else:
                 output = self.data
@@ -707,7 +786,8 @@ class _ImageBaseHDU(_ValidHDU):
             if (format and not self._do_not_scale_image_data and
                     (self._orig_bscale != 1 or self._orig_bzero != 0)):
                 new_dtype = self._dtype_for_bitpix()
-                format += ' (rescales to {0})'.format(new_dtype.name)
+                if new_dtype is not None:
+                    format += ' (rescales to {0})'.format(new_dtype.name)
 
         # Display shape in FITS-order
         shape = tuple(reversed(self.shape))
@@ -727,7 +807,7 @@ class _ImageBaseHDU(_ValidHDU):
             # 16, 32 or 64
             if _is_pseudo_unsigned(self.data.dtype):
                 d = np.array(self.data - _unsigned_zero(self.data.dtype),
-                             dtype='i%d' % self.data.dtype.itemsize)
+                             dtype='i{}'.format(self.data.dtype.itemsize))
 
             # Check the byte order of the data.  If it is little endian we
             # must swap it before calculating the datasum.
@@ -928,8 +1008,8 @@ class PrimaryHDU(_ImageBaseHDU):
     def match_header(cls, header):
         card = header.cards[0]
         return (card.keyword == 'SIMPLE' and
-                ('GROUPS' not in header or header['GROUPS'] != True) and
-                card.value == True)
+                ('GROUPS' not in header or not header['GROUPS']) and
+                card.value)
 
     def update_header(self):
         super(PrimaryHDU, self).update_header()
@@ -1041,7 +1121,7 @@ class _IndexInfo(object):
                 self.offset = indx
                 self.contiguous = True
             else:
-                raise IndexError('Index %s out of range.' % indx)
+                raise IndexError('Index {} out of range.'.format(indx))
         elif isinstance(indx, slice):
             start, stop, step = indx.indices(naxis)
             self.npts = (stop - start) // step
@@ -1052,4 +1132,4 @@ class _IndexInfo(object):
             self.offset = 0
             self.contiguous = False
         else:
-            raise IndexError('Illegal index %s' % indx)
+            raise IndexError('Illegal index {}'.format(indx))
