@@ -20,18 +20,12 @@ class FitsTestCase(object):
         # Restore global settings to defaults
         # TODO: Replace this when there's a better way to in the config API to
         # force config values to their defaults
-        fits.ENABLE_RECORD_VALUED_KEYWORD_CARDS.set(True)
-        fits.EXTENSION_NAME_CASE_SENSITIVE.set(False)
-        fits.STRIP_HEADER_WHITESPACE.set(True)
-        fits.USE_MEMMAP.set(True)
-
-        # Ignore deprecation warnings--this only affects Python 2.5 and 2.6,
-        # since deprecation warnings are ignored by defualt on 2.7
-        warnings.simplefilter('ignore')
-        warnings.simplefilter('always', UserWarning)
+        fits.conf.enable_record_valued_keyword_cards = True
+        fits.conf.extension_name_case_sensitive = False
+        fits.conf.strip_header_whitespace = True
+        fits.conf.use_memmap = True
 
     def teardown(self):
-        warnings.resetwarnings()
         if hasattr(self, 'temp_dir') and os.path.exists(self.temp_dir):
             tries = 3
             while tries:
@@ -45,6 +39,11 @@ class FitsTestCase(object):
                     time.sleep(0.5)
                     tries -= 1
 
+        fits.conf.reset('enable_record_valued_keyword_cards')
+        fits.conf.reset('extension_name_case_sensitive')
+        fits.conf.reset('strip_header_whitespace')
+        fits.conf.reset('use_memmap')
+
     def copy_file(self, filename):
         """Copies a backup of a test data file to the temp dir and sets its
         mode to writeable.
