@@ -17,15 +17,17 @@ try:
 except ImportError:
     HAVE_STRINGIO = False
 
-from ....io import fits
-from ....tests.helper import pytest, raises, catch_warnings, ignore_warnings
-from ....utils.exceptions import AstropyDeprecationWarning
-from ....utils.data import get_pkg_data_filename
 from . import FitsTestCase
+
 from ..convenience import _getext
 from ..diff import FITSDiff
 from ..file import _File, GZIP_MAGIC
 
+from ....extern.six.moves import range, zip
+from ....io import fits
+from ....tests.helper import pytest, raises, catch_warnings, ignore_warnings
+from ....utils.data import get_pkg_data_filename
+
 try:
     import pathlib
 except ImportError:
@@ -68,7 +70,7 @@ class TestCore(FitsTestCase):
         l.append(p)
         l.append(t)
 
-        l.writeto(self.temp('test.fits'), clobber=True)
+        l.writeto(self.temp('test.fits'), overwrite=True)
 
         with fits.open(self.temp('test.fits')) as p:
             assert p[1].data[1]['foo'] == 60000.0
@@ -114,7 +116,7 @@ class TestCore(FitsTestCase):
         assert table.data.dtype.names == ('c2', 'c4', 'foo')
         assert table.columns.names == ['c2', 'c4', 'foo']
 
-        hdulist.writeto(self.temp('test.fits'), clobber=True)
+        hdulist.writeto(self.temp('test.fits'), overwrite=True)
         with ignore_warnings():
             # TODO: The warning raised by this test is actually indication of a
             # bug and should *not* be ignored. But as it is a known issue we
@@ -125,7 +127,6 @@ class TestCore(FitsTestCase):
                 assert table.data.dtype.names == ('c2', 'c4', 'foo')
                 assert table.columns.names == ['c2', 'c4', 'foo']
 
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_update_header_card(self):
         """A very basic test for the Header.update method--I'd like to add a
         few more cases to this at some point.
@@ -136,23 +137,12 @@ class TestCore(FitsTestCase):
         header['BITPIX'] = (16, comment)
         assert 'BITPIX' in header
         assert header['BITPIX'] == 16
-        assert header.ascard['BITPIX'].comment == comment
+        assert header.comments['BITPIX'] == comment
 
-        # The new API doesn't support savecomment so leave this line here; at
-        # any rate good to have testing of the new API mixed with the old API
-        header.update('BITPIX', 32, savecomment=True)
-        # Make sure the value has been updated, but the comment was preserved
+        header.update(BITPIX=32)
         assert header['BITPIX'] == 32
-        assert header.ascard['BITPIX'].comment == comment
+        assert header.comments['BITPIX'] == ''
 
-        # The comment should still be preserved--savecomment only takes effect
-        # if a new comment is also specified
-        header['BITPIX'] = 16
-        assert header.ascard['BITPIX'].comment == comment
-        header.update('BITPIX', 16, 'foobarbaz', savecomment=True)
-        assert header.ascard['BITPIX'].comment == comment
-
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_set_card_value(self):
         """Similar to test_update_header_card(), but tests the the
         `header['FOO'] = 'bar'` method of updating card values.
@@ -160,16 +150,16 @@ class TestCore(FitsTestCase):
 
         header = fits.Header()
         comment = 'number of bits per data pixel'
-        card = fits.Card.fromstring('BITPIX  = 32 / %s' % comment)
-        header.ascard.append(card)
+        card = fits.Card.fromstring('BITPIX  = 32 / {}'.format(comment))
+        header.append(card)
 
         header['BITPIX'] = 32
 
         assert 'BITPIX' in header
         assert header['BITPIX'] == 32
-        assert header.ascard['BITPIX'].key == 'BITPIX'
-        assert header.ascard['BITPIX'].value == 32
-        assert header.ascard['BITPIX'].comment == comment
+        assert header.cards[0].keyword == 'BITPIX'
+        assert header.cards[0].value == 32
+        assert header.cards[0].comment == comment
 
     def test_uint(self):
         hdulist_f = fits.open(self.data('o4sp040b0_raw.fits'), uint=False)
@@ -179,14 +169,13 @@ class TestCore(FitsTestCase):
         assert hdulist_i[1].data.dtype == np.uint16
         assert np.all(hdulist_f[1].data == hdulist_i[1].data)
 
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_fix_missing_card_append(self):
         hdu = fits.ImageHDU()
         errs = hdu.req_cards('TESTKW', None, None, 'foo', 'silentfix', [])
         assert len(errs) == 1
         assert 'TESTKW' in hdu.header
         assert hdu.header['TESTKW'] == 'foo'
-        assert hdu.header.ascard[-1].key == 'TESTKW'
+        assert hdu.header.cards[-1].keyword == 'TESTKW'
 
     def test_fix_invalid_keyword_value(self):
         hdu = fits.ImageHDU()
@@ -240,9 +229,9 @@ class TestCore(FitsTestCase):
             del hdu.header['NAXIS']
             try:
                 hdu.verify('ignore')
-            except Exception as e:
+            except Exception as exc:
                 self.fail('An exception occurred when the verification error '
-                          'should have been ignored: %s' % exc)
+                          'should have been ignored: {}'.format(exc))
         # Make sure the error wasn't fixed either, silently or otherwise
         assert 'NAXIS' not in hdu.header
 
@@ -274,7 +263,7 @@ class TestCore(FitsTestCase):
                 hdu.verify('silentfix+ignore')
             except Exception as exc:
                 self.fail('An exception occurred when the verification error '
-                          'should have been ignored: %s' % exc)
+                          'should have been ignored: {}'.format(exc))
 
         # silentfix+warn should be quiet about the fixed HDU and only warn
         # about the unfixable one
@@ -374,9 +363,6 @@ class TestCore(FitsTestCase):
         runtime works.
         """
 
-        if 'PYFITS_EXTENSION_NAME_CASE_SENSITIVE' in os.environ:
-            del os.environ['PYFITS_EXTENSION_NAME_CASE_SENSITIVE']
-
         hdu = fits.ImageHDU()
         hdu.name = 'sCi'
         assert hdu.name == 'SCI'
@@ -525,7 +511,7 @@ class TestCore(FitsTestCase):
             # Add a bunch of header keywords so that the data will be forced to
             # new offsets within the file:
             for idx in range(40):
-                hdul1[1].header['TEST%d' % idx] = 'test'
+                hdul1[1].header['TEST{}'.format(idx)] = 'test'
 
             hdul1.writeto(self.temp('test1.fits'))
             hdul1.writeto(self.temp('test2.fits'))
@@ -536,14 +522,14 @@ class TestCore(FitsTestCase):
             with fits.open(self.data('test0.fits')) as hdul2:
                 with fits.open(self.temp('test2.fits')) as hdul3:
                     for hdul in (hdul1, hdul3):
-                        for idx, hdus in enumerate(zip(hdul1, hdul)):
-                            hdu1, hdu2 = hdus
+                        for idx, hdus in enumerate(zip(hdul2, hdul)):
+                            hdu2, hdu = hdus
                             if idx != 1:
-                                assert hdu1.header == hdu2.header
+                                assert hdu.header == hdu2.header
                             else:
-                                assert (hdu1.header ==
-                                        hdu2.header[:len(hdu1.header)])
-                            assert np.all(hdu1.data == hdu2.data)
+                                assert (hdu2.header ==
+                                        hdu.header[:len(hdu2.header)])
+                            assert np.all(hdu.data == hdu2.data)
 
 
 class TestConvenienceFunctions(FitsTestCase):
@@ -556,12 +542,11 @@ class TestConvenienceFunctions(FitsTestCase):
         data = np.zeros((100, 100))
         header = fits.Header()
         fits.writeto(self.temp('array.fits'), data, header=header,
-                     clobber=True)
+                     overwrite=True)
         hdul = fits.open(self.temp('array.fits'))
         assert len(hdul) == 1
         assert (data == hdul[0].data).all()
 
-    @ignore_warnings(AstropyDeprecationWarning)
     def test_writeto_2(self):
         """
         Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/107
@@ -569,11 +554,11 @@ class TestConvenienceFunctions(FitsTestCase):
         Test of `writeto()` with a trivial header containing a single keyword.
         """
 
-        data = np.zeros((100,100))
+        data = np.zeros((100, 100))
         header = fits.Header()
         header.set('CRPIX1', 1.)
         fits.writeto(self.temp('array.fits'), data, header=header,
-                     clobber=True, output_verify='silentfix')
+                     overwrite=True, output_verify='silentfix')
         hdul = fits.open(self.temp('array.fits'))
         assert len(hdul) == 1
         assert (data == hdul[0].data).all()
@@ -596,8 +581,6 @@ class TestFileFunctions(FitsTestCase):
             fits.open(self.temp('foobar.fits'))
         except IOError as e:
             assert 'No such file or directory' in str(e)
-        except:
-            raise
 
         # But opening in ostream or append mode should be okay, since they
         # allow writing new files
@@ -809,7 +792,7 @@ class TestFileFunctions(FitsTestCase):
         assert old_mode == os.stat(filename).st_mode
 
     def test_fileobj_mode_guessing(self):
-        """Tests whether a file opened without a specified pyfits mode
+        """Tests whether a file opened without a specified io.fits mode
         ('readonly', etc.) is opened in a mode appropriate for the given file
         object.
         """
