@@ -1,17 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 import io
-import pytest
+from ....tests.helper import pytest
 
 import numpy as np
 
 from ..column import Column
 from ..diff import (FITSDiff, HeaderDiff, ImageDataDiff, TableDataDiff,
-                    report_diff_values)
+                    HDUDiff, report_diff_values)
 from ..hdu import HDUList, PrimaryHDU, ImageHDU
 from ..hdu.table import BinTableHDU
 from ..header import Header
 
+from ....tests.helper import catch_warnings
+from ....utils.exceptions import AstropyDeprecationWarning
+from ....extern.six.moves import range
 from ....io import fits
 
 from . import FitsTestCase
@@ -22,6 +25,10 @@ class TestDiff(FitsTestCase):
         ha = Header([('A', 1), ('B', 2), ('C', 3)])
         hb = ha.copy()
         assert HeaderDiff(ha, hb).identical
+        assert HeaderDiff(ha.tostring(), hb.tostring()).identical
+
+        with pytest.raises(TypeError):
+            HeaderDiff(1, 2)
 
     def test_slightly_different_headers(self):
         ha = Header([('A', 1), ('B', 2), ('C', 3)])
@@ -97,6 +104,11 @@ class TestDiff(FitsTestCase):
         assert (diff.diff_duplicate_keywords ==
                 {'A': (3, 1), 'B': (1, 2), 'C': (1, 2)})
 
+        report = diff.report()
+        assert ("Inconsistent duplicates of keyword 'A'     :\n"
+                "  Occurs 3 time(s) in a, 1 times in (b)") in report
+
+
     def test_floating_point_tolerance(self):
         ha = Header([('A', 1), ('B', 2.00001), ('C', 3.000001)])
         hb = ha.copy()
@@ -341,6 +353,10 @@ class TestDiff(FitsTestCase):
         assert diff.diff_ratio == 0
         assert diff.diff_total == 0
 
+        report = diff.report()
+        assert 'Extra column B of format L in a' in report
+        assert 'Extra column C of format L in b' in report
+
     def test_different_table_field_counts(self):
         """
         Test tables with some common columns, but different number of columns
@@ -364,9 +380,13 @@ class TestDiff(FitsTestCase):
         assert diff.diff_ratio == 0
         assert diff.diff_total == 0
 
+        report = diff.report()
+        assert ' Tables have different number of columns:' in report
+        assert '  a: 1\n  b: 3' in report
+
     def test_different_table_rows(self):
         """
-        Test tables taht are otherwise identical but one has more rows than the
+        Test tables that are otherwise identical but one has more rows than the
         other.
         """
 
@@ -455,6 +475,16 @@ class TestDiff(FitsTestCase):
         assert diff.diff_total == 13
         assert diff.diff_ratio == 0.65
 
+        report = diff.report()
+        assert ('Column A data differs in row 0:\n'
+                '    a> True\n'
+                '    b> False') in report
+        assert ('...and at 13 more indices.\n'
+                ' Column D data differs in row 0:') in report
+        assert ('13 different table data element(s) found (65.00% different)'
+                in report)
+        assert report.count('more indices') == 1
+
     def test_identical_files_basic(self):
         """Test identicality of two simple, extensionless files."""
 
@@ -471,6 +501,13 @@ class TestDiff(FitsTestCase):
         assert 'Extension HDU' not in report
         assert 'No differences found.' in report
 
+        a = np.arange(10)
+        ehdu = ImageHDU(data=a)
+        diff = HDUDiff(ehdu, ehdu)
+        assert diff.identical
+        report = diff.report()
+        assert 'No differences found.' in report
+
     def test_partially_identical_files1(self):
         """
         Test files that have some identical HDUs but a different extension
@@ -519,7 +556,7 @@ class TestDiff(FitsTestCase):
         assert hdudiff.diff_extvers == ()
         assert hdudiff.diff_extension_types == ()
         assert hdudiff.diff_headers.identical
-        assert not hdudiff.diff_data is None
+        assert hdudiff.diff_data is not None
 
         datadiff = hdudiff.diff_data
         assert isinstance(datadiff, ImageDataDiff)
@@ -539,9 +576,43 @@ class TestDiff(FitsTestCase):
         assert 'Headers contain differences' not in report
         assert 'Data contains differences' in report
         for y in range(10):
-            assert 'Data differs at [%d, 1]' % (y + 1) in report
+            assert 'Data differs at [{}, 1]'.format(y + 1) in report
         assert '100 different pixels found (100.00% different).' in report
 
+    def test_partially_identical_files3(self):
+        """
+        Test files that have some identical HDUs but a different extension
+        name.
+        """
+
+        phdu = PrimaryHDU()
+        ehdu = ImageHDU(name='FOO')
+        hdula = HDUList([phdu, ehdu])
+        ehdu = BinTableHDU(name='BAR')
+        ehdu.header['EXTVER'] = 2
+        ehdu.header['EXTLEVEL'] = 3
+        hdulb = HDUList([phdu, ehdu])
+        diff = FITSDiff(hdula, hdulb)
+        assert not diff.identical
+
+        assert diff.diff_hdus[0][0] == 1
+
+        hdu_diff = diff.diff_hdus[0][1]
+        assert hdu_diff.diff_extension_types == ('IMAGE', 'BINTABLE')
+        assert hdu_diff.diff_extnames == ('FOO', 'BAR')
+        assert hdu_diff.diff_extvers == (1, 2)
+        assert hdu_diff.diff_extlevels == (1, 3)
+
+        report = diff.report()
+        assert 'Extension types differ' in report
+        assert 'a: IMAGE\n    b: BINTABLE' in report
+        assert 'Extension names differ' in report
+        assert 'a: FOO\n    b: BAR' in report
+        assert 'Extension versions differ' in report
+        assert 'a: 1\n    b: 2' in report
+        assert 'Extension levels differ' in report
+        assert 'a: 1\n    b: 2' in report
+
     def test_diff_nans(self):
         """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/204"""
 
@@ -622,7 +693,7 @@ class TestDiff(FitsTestCase):
         report_as_string = diffobj.report()
         assert open(outpath).read() == report_as_string
 
-    def test_file_output_clobber_safety(self):
+    def test_file_output_overwrite_safety(self):
         outpath = self.temp('diff_output.txt')
         ha = Header([('A', 1), ('B', 2), ('C', 3)])
         hb = ha.copy()
@@ -633,7 +704,7 @@ class TestDiff(FitsTestCase):
         with pytest.raises(IOError):
             diffobj.report(fileobj=outpath)
 
-    def test_file_output_clobber_success(self):
+    def test_file_output_overwrite_success(self):
         outpath = self.temp('diff_output.txt')
         ha = Header([('A', 1), ('B', 2), ('C', 3)])
         hb = ha.copy()
@@ -641,6 +712,23 @@ class TestDiff(FitsTestCase):
         diffobj = HeaderDiff(ha, hb)
         diffobj.report(fileobj=outpath)
         report_as_string = diffobj.report()
-        diffobj.report(fileobj=outpath, clobber=True)
-        assert open(outpath).read() == report_as_string, ("clobbered output "
+        diffobj.report(fileobj=outpath, overwrite=True)
+        assert open(outpath).read() == report_as_string, ("overwritten output "
             "file is not identical to report string")
+
+    def test_file_output_overwrite_vs_clobber(self):
+        """Verify uses of clobber and overwrite."""
+
+        outpath = self.temp('diff_output.txt')
+        ha = Header([('A', 1), ('B', 2), ('C', 3)])
+        hb = ha.copy()
+        hb['C'] = 4
+        diffobj = HeaderDiff(ha, hb)
+        diffobj.report(fileobj=outpath)
+        report_as_string = diffobj.report()
+        with catch_warnings(AstropyDeprecationWarning) as warning_lines:
+            diffobj.report(fileobj=outpath, clobber=True)
+            assert warning_lines[0].category == AstropyDeprecationWarning
+            assert (str(warning_lines[0].message) == '"clobber" was '
+                    'deprecated in version 1.3 and will be removed in a '
+                    'future version. Use argument "overwrite" instead.')
