@@ -2,7 +2,6 @@
 
 from __future__ import print_function
 
-import os
 
 import numpy as np
 
@@ -16,6 +15,12 @@ except ImportError:
 else:
     HAS_H5PY = True
 
+try:
+    import yaml
+except ImportError:
+    HAS_YAML = False
+else:
+    HAS_YAML = True
 
 ALL_DTYPES = [np.uint8, np.uint16, np.uint32, np.uint64, np.int8,
               np.int16, np.int32, np.int64, np.float32, np.float64,
@@ -371,7 +376,7 @@ def test_preserve_meta(tmpdir):
         assert np.all(t1.meta[key] == t2.meta[key])
 
 
-@pytest.mark.skipif('not HAS_H5PY')
+@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
 def test_preserve_serialized(tmpdir):
     test_file = str(tmpdir.join('test.hdf5'))
 
@@ -395,7 +400,7 @@ def test_preserve_serialized(tmpdir):
     assert t1.meta == t2.meta
 
 
-@pytest.mark.skipif('not HAS_H5PY')
+@pytest.mark.skipif('not HAS_H5PY or not HAS_YAML')
 def test_metadata_too_large(tmpdir):
     test_file = str(tmpdir.join('test.hdf5'))
 
