@@ -4,33 +4,70 @@
 
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
-
-import inspect
-
+import warnings
 import numpy as np
-
 from .core import (Fittable1DModel, Fittable2DModel, Model,
-                   ModelDefinitionError, custom_model)
+                   ModelDefinitionError)
 from .parameters import Parameter, InputParameterError
 from .utils import ellipse_extent
-from ..utils import deprecated
-from ..utils.compat.funcsigs import signature
-from ..extern import six
-
-from .utils import get_inputs_and_params
+from ..extern.six.moves import map
+from ..utils.exceptions import AstropyDeprecationWarning
 
 
-__all__ = sorted([
-    'AiryDisk2D', 'Moffat1D', 'Moffat2D', 'Box1D',
-    'Box2D', 'Const1D', 'Const2D', 'Ellipse2D', 'Disk2D',
-    'Gaussian1D', 'GaussianAbsorption1D', 'Gaussian2D', 'Linear1D',
-    'Lorentz1D', 'MexicanHat1D', 'MexicanHat2D', 'Redshift', 'Scale',
-    'Sersic1D', 'Sersic2D', 'Shift', 'Sine1D', 'Trapezoid1D', 'TrapezoidDisk2D',
-    'Ring2D', 'custom_model_1d', 'Voigt1D'
-])
+__all__ = ['AiryDisk2D', 'Moffat1D', 'Moffat2D', 'Box1D', 'Box2D', 'Const1D',
+           'Const2D', 'Ellipse2D', 'Disk2D', 'BaseGaussian1D', 'Gaussian1D',
+           'GaussianAbsorption1D', 'Gaussian2D', 'Linear1D', 'Lorentz1D',
+           'MexicanHat1D', 'MexicanHat2D', 'RedshiftScaleFactor', 'Redshift',
+           'Scale', 'Sersic1D', 'Sersic2D', 'Shift', 'Sine1D', 'Trapezoid1D',
+           'TrapezoidDisk2D', 'Ring2D', 'Voigt1D']
 
 
-class Gaussian1D(Fittable1DModel):
+class BaseGaussian1D(Fittable1DModel):
+    """Base class for 1D Gaussian models. Do not use this directly.
+
+    See Also
+    --------
+    Gaussian1D, GaussianAbsorption1D
+    """
+
+    amplitude = Parameter(default=1)
+    mean = Parameter(default=0)
+    stddev = Parameter(default=1)
+
+    def bounding_box(self, factor=5.5):
+        """
+        Tuple defining the default ``bounding_box`` limits,
+        ``(x_low, x_high)``
+
+        Parameters
+        ----------
+        factor : float
+            The multiple of `stddev` used to define the limits.
+            The default is 5.5, corresponding to a relative error < 1e-7.
+
+        Examples
+        --------
+        >>> from astropy.modeling.models import Gaussian1D
+        >>> model = Gaussian1D(mean=0, stddev=2)
+        >>> model.bounding_box
+        (-11.0, 11.0)
+
+        This range can be set directly (see: `Model.bounding_box
+        <astropy.modeling.Model.bounding_box>`) or by using a different factor,
+        like:
+
+        >>> model.bounding_box = model.bounding_box(factor=2)
+        >>> model.bounding_box
+        (-4.0, 4.0)
+        """
+
+        x0 = self.mean.value
+        dx = factor * self.stddev
+
+        return (x0 - dx, x0 + dx)
+
+
+class Gaussian1D(BaseGaussian1D):
     """
     One dimensional Gaussian model.
 
@@ -88,47 +125,30 @@ class Gaussian1D(Fittable1DModel):
     >>> g1.stddev.fixed
     True
 
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Gaussian1D
+
+        plt.figure()
+        s1 = Gaussian1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -1, 4])
+        plt.show()
+
     See Also
     --------
     Gaussian2D, Box1D, Moffat1D, Lorentz1D
     """
 
-    amplitude = Parameter(default=1)
-    mean = Parameter(default=0)
-    stddev = Parameter(default=1)
-
-    def bounding_box(self, factor=5.5):
-        """
-        Tuple defining the default ``bounding_box`` limits,
-        ``(x_low, x_high)``
-
-        Parameters
-        ----------
-        factor : float
-            The multiple of `stddev` used to define the limits.
-            The default is 5.5, corresponding to a relative error < 1e-7.
-
-        Examples
-        --------
-        >>> from astropy.modeling.models import Gaussian1D
-        >>> model = Gaussian1D(mean=0, stddev=2)
-        >>> model.bounding_box
-        (-11.0, 11.0)
-
-        This range can be set directly (see: `Model.bounding_box
-        <astropy.modeling.Model.bounding_box>`) or by using a different factor,
-        like:
-
-        >>> model.bounding_box = model.bounding_box(factor=2)
-        >>> model.bounding_box
-        (-4.0, 4.0)
-        """
-
-        x0 = self.mean.value
-        dx = factor * self.stddev
-
-        return (x0 - dx, x0 + dx)
-
     @staticmethod
     def evaluate(x, amplitude, mean, stddev):
         """
@@ -148,7 +168,7 @@ class Gaussian1D(Fittable1DModel):
         return [d_amplitude, d_mean, d_stddev]
 
 
-class GaussianAbsorption1D(Fittable1DModel):
+class GaussianAbsorption1D(BaseGaussian1D):
     """
     One dimensional Gaussian absorption line model.
 
@@ -168,46 +188,30 @@ class GaussianAbsorption1D(Fittable1DModel):
 
         .. math:: f(x) = 1 - A e^{- \\frac{\\left(x - x_{0}\\right)^{2}}{2 \\sigma^{2}}}
 
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+        from astropy.modeling.models import GaussianAbsorption1D
+
+        plt.figure()
+        s1 = GaussianAbsorption1D()
+        r = np.arange(-5, 5, .01)
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -3, 2])
+        plt.show()
+
     See Also
     --------
     Gaussian1D
     """
 
-    amplitude = Parameter(default=1)
-    mean = Parameter(default=0)
-    stddev = Parameter(default=1)
-
-    def bounding_box(self, factor=5.5):
-        """
-        Tuple defining the default ``bounding_box`` limits,
-        ``(x_low, x_high)``
-
-        Parameters
-        ----------
-        factor : float
-            The multiple of `stddev` used to define the limits.
-            The default is 5.5, corresponding to a relative error < 1e-7.
-
-        Examples
-        --------
-        >>> from astropy.modeling.models import Gaussian1D
-        >>> model = Gaussian1D(mean=0, stddev=2)
-        >>> model.bounding_box
-        (-11.0, 11.0)
-
-        This range can be set directly (see: ``help(model.bounding_box)``) or by
-        using a different factor, like:
-
-        >>> model.bounding_box = model.bounding_box(factor=2)
-        >>> model.bounding_box
-        (-4.0, 4.0)
-        """
-
-        x0 = self.mean.value
-        dx = factor * self.stddev
-
-        return (x0 - dx, x0 + dx)
-
     @staticmethod
     def evaluate(x, amplitude, mean, stddev):
         """
@@ -221,12 +225,12 @@ class GaussianAbsorption1D(Fittable1DModel):
         GaussianAbsorption1D model function derivatives.
         """
         import operator
-        return list(six.moves.map(
+        return list(map(
             operator.neg, Gaussian1D.fit_deriv(x, amplitude, mean, stddev)))
 
 
 class Gaussian2D(Fittable2DModel):
-    """
+    r"""
     Two dimensional Gaussian model.
 
     Parameters
@@ -237,20 +241,22 @@ class Gaussian2D(Fittable2DModel):
         Mean of the Gaussian in x.
     y_mean : float
         Mean of the Gaussian in y.
-    x_stddev : float
-        Standard deviation of the Gaussian in x before rotating by theta.
-        ``x_stddev`` and ``y_stddev`` must be specified unless a covariance
-        matrix (``cov_matrix``) is input.
-    y_stddev : float
-        Standard deviation of the Gaussian in y before rotating by theta.
-        ``x_stddev`` and ``y_stddev`` must be specified unless a covariance
-        matrix (``cov_matrix``) is input.
+    x_stddev : float or None
+        Standard deviation of the Gaussian in x before rotating by theta. Must
+        be None if a covariance matrix (``cov_matrix``) is provided. If no
+        ``cov_matrix`` is given, ``None`` means the default value (1).
+    y_stddev : float or None
+        Standard deviation of the Gaussian in y before rotating by theta. Must
+        be None if a covariance matrix (``cov_matrix``) is provided. If no
+        ``cov_matrix`` is given, ``None`` means the default value (1).
     theta : float, optional
         Rotation angle in radians. The rotation angle increases
-        counterclockwise, from the positive x-axis.
+        counterclockwise.  Must be None if a covariance matrix (``cov_matrix``)
+        is provided. If no ``cov_matrix`` is given, ``None`` means the default
+        value (0).
     cov_matrix : ndarray, optional
         A 2x2 covariance matrix. If specified, overrides the ``x_stddev``,
-        ``y_stddev``, and ``theta`` specification.
+        ``y_stddev``, and ``theta`` defaults.
 
     Notes
     -----
@@ -258,35 +264,35 @@ class Gaussian2D(Fittable2DModel):
 
         .. math::
 
-            f(x, y) = A e^{-a\\left(x - x_{0}\\right)^{2}  -b\\left(x - x_{0}\\right)
-            \\left(y - y_{0}\\right)  -c\\left(y - y_{0}\\right)^{2}}
+            f(x, y) = A e^{-a\left(x - x_{0}\right)^{2}  -b\left(x - x_{0}\right)
+            \left(y - y_{0}\right)  -c\left(y - y_{0}\right)^{2}}
 
     Using the following definitions:
 
         .. math::
-            a = \\left(\\frac{\\cos^{2}{\\left (\\theta \\right )}}{2 \\sigma_{x}^{2}} +
-            \\frac{\\sin^{2}{\\left (\\theta \\right )}}{2 \\sigma_{y}^{2}}\\right)
+            a = \left(\frac{\cos^{2}{\left (\theta \right )}}{2 \sigma_{x}^{2}} +
+            \frac{\sin^{2}{\left (\theta \right )}}{2 \sigma_{y}^{2}}\right)
 
-            b = \\left(\\frac{\\sin{\\left (2 \\theta \\right )}}{2 \\sigma_{x}^{2}} -
-            \\frac{\\sin{\\left (2 \\theta \\right )}}{2 \\sigma_{y}^{2}}\\right)
+            b = \left(\frac{\sin{\left (2 \theta \right )}}{2 \sigma_{x}^{2}} -
+            \frac{\sin{\left (2 \theta \right )}}{2 \sigma_{y}^{2}}\right)
 
-            c = \\left(\\frac{\\sin^{2}{\\left (\\theta \\right )}}{2 \\sigma_{x}^{2}} +
-            \\frac{\\cos^{2}{\\left (\\theta \\right )}}{2 \\sigma_{y}^{2}}\\right)
+            c = \left(\frac{\sin^{2}{\left (\theta \right )}}{2 \sigma_{x}^{2}} +
+            \frac{\cos^{2}{\left (\theta \right )}}{2 \sigma_{y}^{2}}\right)
 
     If using a ``cov_matrix``, the model is of the form:
         .. math::
-            f(x, y) = A e^{-0.5 \\left(\\vec{x} - \\vec{x}_{0}\\right)^{T} \\Sigma^{-1} \\left(\\vec{x} - \\vec{x}_{0}\\right)}
+            f(x, y) = A e^{-0.5 \left(\vec{x} - \vec{x}_{0}\right)^{T} \Sigma^{-1} \left(\vec{x} - \vec{x}_{0}\right)}
 
-    where :math:`\\vec{x} = [x, y]`, :math:`\\vec{x}_{0} = [x_{0}, y_{0}]`,
-    and :math:`\\Sigma` is the covariance matrix:
+    where :math:`\vec{x} = [x, y]`, :math:`\vec{x}_{0} = [x_{0}, y_{0}]`,
+    and :math:`\Sigma` is the covariance matrix:
 
         .. math::
-            \\Sigma = \\left(\\begin{array}{ccc}
-            \\sigma_x^2               & \\rho \\sigma_x \\sigma_y \\\\
-            \\rho \\sigma_x \\sigma_y & \\sigma_y^2
-            \end{array}\\right)
+            \Sigma = \left(\begin{array}{ccc}
+            \sigma_x^2               & \rho \sigma_x \sigma_y \\
+            \rho \sigma_x \sigma_y   & \sigma_y^2
+            \end{array}\right)
 
-    :math:`\\rho` is the correlation between ``x`` and ``y``, which should
+    :math:`\rho` is the correlation between ``x`` and ``y``, which should
     be between -1 and +1.  Positive correlation corresponds to a
     ``theta`` in the range 0 to 90 degrees.  Negative correlation
     corresponds to a ``theta`` in the range of 0 to -90 degrees.
@@ -307,43 +313,37 @@ class Gaussian2D(Fittable2DModel):
     y_mean = Parameter(default=0)
     x_stddev = Parameter(default=1)
     y_stddev = Parameter(default=1)
-    theta = Parameter(default=0)
+    theta = Parameter(default=0.0)
 
     def __init__(self, amplitude=amplitude.default, x_mean=x_mean.default,
                  y_mean=y_mean.default, x_stddev=None, y_stddev=None,
-                 theta=0.0, cov_matrix=None, **kwargs):
-        if y_stddev is None and cov_matrix is None:
-            raise InputParameterError(
-                "Either x/y_stddev must be specified, or a "
-                "covariance matrix.")
-        elif x_stddev is None and cov_matrix is None:
-            raise InputParameterError(
-                "Either x/y_stddev must be specified, or a "
-                "covariance matrix.")
-        elif cov_matrix is not None and (x_stddev is not None or
-                                         y_stddev is not None):
-            raise InputParameterError(
-                "Cannot specify both cov_matrix and x/y_stddev")
+                 theta=None, cov_matrix=None, **kwargs):
+        if cov_matrix is None:
+            if x_stddev is None:
+                x_stddev = self.__class__.x_stddev.default
+            if y_stddev is None:
+                y_stddev = self.__class__.y_stddev.default
+            if theta is None:
+                theta = self.__class__.theta.default
+        else:
+            if x_stddev is not None or y_stddev is not None or theta is not None:
+                raise InputParameterError("Cannot specify both cov_matrix and "
+                                          "x/y_stddev/theta")
+            else:
+                # Compute principle coordinate system transformation
+                cov_matrix = np.array(cov_matrix)
 
-        # Compute principle coordinate system transformation
-        elif cov_matrix is not None:
-            if (x_stddev is not None or y_stddev is not None):
-                raise InputParameterError(
-                    "Cannot specify both cov_matrix and x/y_stddev")
-            # Compute principle coordinate system transformation
-            cov_matrix = np.array(cov_matrix)
+                if cov_matrix.shape != (2, 2):
+                    # TODO: Maybe it should be possible for the covariance matrix
+                    # to be some (x, y, ..., z, 2, 2) array to be broadcast with
+                    # other parameters of shape (x, y, ..., z)
+                    # But that's maybe a special case to work out if/when needed
+                    raise ValueError("Covariance matrix must be 2x2")
 
-            if cov_matrix.shape != (2, 2):
-                # TODO: Maybe it should be possible for the covariance matrix
-                # to be some (x, y, ..., z, 2, 2) array to be broadcast with
-                # other parameters of shape (x, y, ..., z)
-                # But that's maybe a special case to work out if/when needed
-                raise ValueError("Covariance matrix must be 2x2")
-
-            eig_vals, eig_vecs = np.linalg.eig(cov_matrix)
-            x_stddev, y_stddev = np.sqrt(eig_vals)
-            y_vec = eig_vecs[:, 0]
-            theta = np.arctan2(y_vec[1], y_vec[0])
+                eig_vals, eig_vecs = np.linalg.eig(cov_matrix)
+                x_stddev, y_stddev = np.sqrt(eig_vals)
+                y_vec = eig_vecs[:, 0]
+                theta = np.arctan2(y_vec[1], y_vec[0])
 
         super(Gaussian2D, self).__init__(
             amplitude=amplitude, x_mean=x_mean, y_mean=y_mean,
@@ -466,6 +466,7 @@ class Shift(Model):
     outputs = ('x',)
 
     offset = Parameter(default=0)
+    fittable = True
 
     @property
     def inverse(self):
@@ -493,6 +494,7 @@ class Scale(Model):
 
     factor = Parameter(default=1)
     linear = True
+    fittable = True
 
     @property
     def inverse(self):
@@ -505,46 +507,55 @@ class Scale(Model):
         return factor * x
 
 
-class Redshift(Fittable1DModel):
+class RedshiftScaleFactor(Fittable1DModel):
     """
-    One dimensional redshift model.
+    One dimensional redshift scale factor model.
 
     Parameters
     ----------
-    z : float or a list of floats
-        Redshift value(s).
+    z : float
+        Redshift value.
 
     Notes
     -----
     Model formula:
 
-        .. math:: \\lambda_{obs} = (1 + z) \\lambda_{rest}
-
+        .. math:: f(x) = x (1 + z)
     """
 
     z = Parameter(description='redshift', default=0)
 
     @staticmethod
     def evaluate(x, z):
-        """One dimensional Redshift model function"""
+        """One dimensional RedshiftScaleFactor model function"""
 
         return (1 + z) * x
 
     @staticmethod
     def fit_deriv(x, z):
-        """One dimensional Redshift model derivative"""
+        """One dimensional RedshiftScaleFactor model derivative"""
+
         d_z = x
         return [d_z]
 
     @property
     def inverse(self):
-        """Inverse Redshift model"""
+        """Inverse RedshiftScaleFactor model"""
 
         inv = self.copy()
         inv.z = 1.0 / (1.0 + self.z) - 1.0
         return inv
 
 
+class Redshift(RedshiftScaleFactor):
+    """This is **deprecated**, use :class:`RedshiftScaleFactor`."""
+    def __init__(self, *args):
+        warnings.warn('The "Redshift" class is now deprecated -- use the '
+                      '"RedshiftScaleFactor" class instead.',
+                      AstropyDeprecationWarning)
+        super(Redshift, self).__init__(*args)
+
+
 class Sersic1D(Fittable1DModel):
     r"""
     One dimensional Sersic surface brightness profile.
@@ -568,7 +579,7 @@ class Sersic1D(Fittable1DModel):
 
     .. math::
 
-        I(r)=I_e exp\left[-b_n\left(\frac{r}{r_{e}}\right)^{(1/n)}-1\right]
+        I(r)=I_e\exp\left\{-b_n\left[\left(\frac{r}{r_{e}}\right)^{(1/n)}-1\right]\right\}
 
     The constant :math:`b_n` is defined such that :math:`r_e` contains half the total
     luminosity, and can be solved for numerically.
@@ -614,21 +625,19 @@ class Sersic1D(Fittable1DModel):
     n = Parameter(default=4)
     _gammaincinv = None
 
-    def __init__(self, amplitude=amplitude.default, r_eff=r_eff.default,
-                 n=n.default, **kwargs):
-        try:
-            from scipy.special import gammaincinv
-            self.__class__._gammaincinv = gammaincinv
-        except ValueError:
-            raise ImportError("Sersic1D model requires scipy > 0.11.")
-
-        super(Sersic1D, self).__init__(
-            amplitude=amplitude, r_eff=r_eff, n=n, **kwargs)
-
     @classmethod
     def evaluate(cls, r, amplitude, r_eff, n):
         """One dimensional Sersic profile function."""
-        return amplitude * np.exp(-cls._gammaincinv(2 * n, 0.5) * ((r / r_eff) ** (1 / n) - 1))
+
+        if cls._gammaincinv is None:
+            try:
+                from scipy.special import gammaincinv
+                cls._gammaincinv = gammaincinv
+            except ValueError:
+                raise ImportError('Sersic1D model requires scipy > 0.11.')
+
+        return (amplitude * np.exp(
+            -cls._gammaincinv(2 * n, 0.5) * ((r / r_eff) ** (1 / n) - 1)))
 
 
 class Sine1D(Fittable1DModel):
@@ -654,6 +663,27 @@ class Sine1D(Fittable1DModel):
     Model formula:
 
         .. math:: f(x) = A \\sin(2 \\pi f x + 2 \\pi p)
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Sine1D
+
+        plt.figure()
+        s1 = Sine1D(amplitude=1, frequency=.25)
+        r=np.arange(0, 10, .01)
+
+        for amplitude in range(1,4):
+             s1.amplitude = amplitude
+             plt.plot(r, s1(r), color=str(0.25 * amplitude), lw=2)
+
+        plt.axis([0, 10, -5, 5])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -726,6 +756,53 @@ class Linear1D(Fittable1DModel):
         return self.__class__(slope=new_slope, intercept=new_intercept)
 
 
+class Planar2D(Fittable2DModel):
+    """
+    Two dimensional Plane model.
+
+    Parameters
+    ----------
+    slope_x : float
+        Slope of the straight line in X
+
+    slope_y : float
+        Slope of the straight line in Y
+
+    intercept : float
+        Z-intercept of the straight line
+
+    See Also
+    --------
+    Linear1D, Polynomial2D
+
+    Notes
+    -----
+    Model formula:
+
+        .. math:: f(x, y) = a x + b y + c
+    """
+
+    slope_x = Parameter(default=1)
+    slope_y = Parameter(default=1)
+    intercept = Parameter(default=0)
+    linear = True
+
+    @staticmethod
+    def evaluate(x, y, slope_x, slope_y, intercept):
+        """Two dimensional Plane model function"""
+
+        return slope_x * x + slope_y * y + intercept
+
+    @staticmethod
+    def fit_deriv(x, y, slope_x, slope_y, intercept):
+        """Two dimensional Plane model derivative with respect to parameters"""
+
+        d_slope_x = x
+        d_slope_y = y
+        d_intercept = np.ones_like(x)
+        return [d_slope_x, d_slope_y, d_intercept]
+
+
 class Lorentz1D(Fittable1DModel):
     """
     One dimensional Lorentzian model.
@@ -750,6 +827,27 @@ class Lorentz1D(Fittable1DModel):
     .. math::
 
         f(x) = \\frac{A \\gamma^{2}}{\\gamma^{2} + \\left(x - x_{0}\\right)^{2}}
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Lorentz1D
+
+        plt.figure()
+        s1 = Lorentz1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -1, 4])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -773,6 +871,24 @@ class Lorentz1D(Fittable1DModel):
         d_fwhm = 2 * amplitude * d_amplitude / fwhm * (1 - d_amplitude)
         return [d_amplitude, d_x_0, d_fwhm]
 
+    def bounding_box(self, factor=25):
+        """Tuple defining the default ``bounding_box`` limits,
+        ``(x_low, x_high)``.
+
+        Parameters
+        ----------
+        factor : float
+            The multiple of FWHM used to define the limits.
+            Default is chosen to include most (99%) of the
+            area under the curve, while still showing the
+            central feature of interest.
+
+        """
+        x0 = self.x_0.value
+        dx = factor * self.fwhm
+
+        return (x0 - dx, x0 + dx)
+
 
 class Voigt1D(Fittable1DModel):
     """
@@ -884,6 +1000,27 @@ class Const1D(Fittable1DModel):
     Model formula:
 
         .. math:: f(x) = A
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Const1D
+
+        plt.figure()
+        s1 = Const1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -1, 4])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -1147,7 +1284,7 @@ class Ring2D(Fittable2DModel):
             f(r) = \\left \\{
                      \\begin{array}{ll}
                        A & : r_{in} \\leq r \\leq r_{out} \\\\
-                       0 & : \\textnormal{else}
+                       0 & : \\text{else}
                      \\end{array}
                    \\right.
 
@@ -1236,10 +1373,32 @@ class Box1D(Fittable1DModel):
 
             f(x) = \\left \\{
                      \\begin{array}{ll}
-                       A & : x_0 - w/2 \\geq x \\geq x_0 + w/2 \\\\
-                       0 & : \\textnormal{else}
+                       A & : x_0 - w/2 \\leq x \\leq x_0 + w/2 \\\\
+                       0 & : \\text{else}
                      \\end{array}
                    \\right.
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Box1D
+
+        plt.figure()
+        s1 = Box1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            s1.width = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -1, 4])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -1305,9 +1464,9 @@ class Box2D(Fittable2DModel):
 
             f(x, y) = \\left \\{
                      \\begin{array}{ll}
-                       A & : x_0 - w_x/2 \\geq x \\geq x_0 + w_x/2 \\\\
-                       A & : y_0 - w_y/2 \\geq y \\geq y_0 + w_y/2 \\\\
-                       0 & : \\textnormal{else}
+            A : & x_0 - w_x/2 \\leq x \\leq x_0 + w_x/2 \\text{ and} \\\\
+                & y_0 - w_y/2 \\leq y \\leq y_0 + w_y/2 \\\\
+            0 : & \\text{else}
                      \\end{array}
                    \\right.
 
@@ -1362,6 +1521,28 @@ class Trapezoid1D(Fittable1DModel):
     See Also
     --------
     Box1D, Gaussian1D, Moffat1D
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Trapezoid1D
+
+        plt.figure()
+        s1 = Trapezoid1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            s1.width = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -1, 4])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -1481,6 +1662,27 @@ class MexicanHat1D(Fittable1DModel):
         f(x) = {A \\left(1 - \\frac{\\left(x - x_{0}\\right)^{2}}{\\sigma^{2}}\\right)
         e^{- \\frac{\\left(x - x_{0}\\right)^{2}}{2 \\sigma^{2}}}}
 
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import MexicanHat1D
+
+        plt.figure()
+        s1 = MexicanHat1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            s1.width = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -2, 4])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -1494,6 +1696,21 @@ class MexicanHat1D(Fittable1DModel):
         xx_ww = (x - x_0) ** 2 / (2 * sigma ** 2)
         return amplitude * (1 - 2 * xx_ww) * np.exp(-xx_ww)
 
+    def bounding_box(self, factor=10.0):
+        """Tuple defining the default ``bounding_box`` limits,
+        ``(x_low, x_high)``.
+
+        Parameters
+        ----------
+        factor : float
+            The multiple of sigma used to define the limits.
+
+        """
+        x0 = self.x_0.value
+        dx = factor * self.sigma
+
+        return (x0 - dx, x0 + dx)
+
 
 class MexicanHat2D(Fittable2DModel):
     """
@@ -1586,39 +1803,21 @@ class AiryDisk2D(Fittable2DModel):
     x_0 = Parameter(default=0)
     y_0 = Parameter(default=0)
     radius = Parameter(default=1)
+    _rz = None
     _j1 = None
 
-    def __init__(self, amplitude=amplitude.default, x_0=x_0.default,
-                 y_0=y_0.default, radius=radius.default, **kwargs):
-        if self._j1 is None:
-            try:
-                from scipy.special import j1, jn_zeros
-                self.__class__._j1 = j1
-                self.__class__._rz = jn_zeros(1, 1)[0] / np.pi
-            # add a ValueError here for python3 + scipy < 0.12
-            except ValueError:
-                raise ImportError("AiryDisk2D model requires scipy > 0.11.")
-
-        super(AiryDisk2D, self).__init__(
-            amplitude=amplitude, x_0=x_0, y_0=y_0, radius=radius, **kwargs)
-
-    # TODO: Why does this particular model have its own special __deepcopy__
-    # and __copy__?  If it has anything to do with the use of the j_1 function
-    # that should be reworked.
-    def __deepcopy__(self, memo):
-        new_model = self.__class__(self.amplitude.value, self.x_0.value,
-                                   self.y_0.value, self.radius.value)
-        return new_model
-
-    def __copy__(self):
-        new_model = self.__class__(self.amplitude.value, self.x_0.value,
-                                   self.y_0.value, self.radius.value)
-        return new_model
-
     @classmethod
     def evaluate(cls, x, y, amplitude, x_0, y_0, radius):
         """Two dimensional Airy model function"""
 
+        if cls._rz is None:
+            try:
+                from scipy.special import j1, jn_zeros
+                cls._rz = jn_zeros(1, 1)[0] / np.pi
+                cls._j1 = j1
+            except ValueError:
+                raise ImportError('AiryDisk2D model requires scipy > 0.11.')
+
         r = np.sqrt((x - x_0) ** 2 + (y - y_0) ** 2) / (radius / cls._rz)
         # Since r can be zero, we have to take care to treat that case
         # separately so as not to raise a numpy warning
@@ -1655,6 +1854,28 @@ class Moffat1D(Fittable1DModel):
     .. math::
 
         f(x) = A \\left(1 + \\frac{\\left(x - x_{0}\\right)^{2}}{\\gamma^{2}}\\right)^{- \\alpha}
+
+    Examples
+    --------
+    .. plot::
+        :include-source:
+
+        import numpy as np
+        import matplotlib.pyplot as plt
+
+        from astropy.modeling.models import Moffat1D
+
+        plt.figure()
+        s1 = Moffat1D()
+        r = np.arange(-5, 5, .01)
+
+        for factor in range(1, 4):
+            s1.amplitude = factor
+            s1.width = factor
+            plt.plot(r, s1(r), color=str(0.25 * factor), lw=2)
+
+        plt.axis([-5, 5, -1, 4])
+        plt.show()
     """
 
     amplitude = Parameter(default=1)
@@ -1772,7 +1993,7 @@ class Sersic2D(Fittable2DModel):
 
     .. math::
 
-        I(x,y) = I(r) = I_e\exp\left[-b_n\left(\frac{r}{r_{e}}\right)^{(1/n)}-1\right]
+        I(x,y) = I(r) = I_e\exp\left\{-b_n\left[\left(\frac{r}{r_{e}}\right)^{(1/n)}-1\right]\right\}
 
     The constant :math:`b_n` is defined such that :math:`r_e` contains half the total
     luminosity, and can be solved for numerically.
@@ -1822,23 +2043,17 @@ class Sersic2D(Fittable2DModel):
     theta = Parameter(default=0)
     _gammaincinv = None
 
-    def __init__(self, amplitude=amplitude.default, r_eff=r_eff.default,
-                 n=n.default, x_0=x_0.default, y_0=y_0.default, ellip=ellip.default,
-                 theta=theta.default, **kwargs):
-        try:
-            from scipy.special import gammaincinv
-            self.__class__._gammaincinv = gammaincinv
-        except ValueError:
-            raise ImportError("Sersic2D model requires scipy > 0.11.")
-
-        super(Sersic2D, self).__init__(
-            amplitude=amplitude, r_eff=r_eff, n=n, x_0=x_0, y_0=y_0,
-            ellip=ellip, theta=theta, **kwargs)
-
     @classmethod
     def evaluate(cls, x, y, amplitude, r_eff, n, x_0, y_0, ellip, theta):
         """Two dimensional Sersic profile function."""
 
+        if cls._gammaincinv is None:
+            try:
+                from scipy.special import gammaincinv
+                cls._gammaincinv = gammaincinv
+            except ValueError:
+                raise ImportError('Sersic2D model requires scipy > 0.11.')
+
         bn = cls._gammaincinv(2. * n, 0.5)
         a, b = r_eff, (1 - ellip) * r_eff
         cos_theta, sin_theta = np.cos(theta), np.sin(theta)
@@ -1847,15 +2062,3 @@ class Sersic2D(Fittable2DModel):
         z = np.sqrt((x_maj / a) ** 2 + (x_min / b) ** 2)
 
         return amplitude * np.exp(-bn * (z ** (1 / n) - 1))
-
-
-@deprecated('1.0', alternative='astropy.modeling.models.custom_model',
-            pending=True)
-def custom_model_1d(func, func_fit_deriv=None):
-    inputs, params = get_inputs_and_params(func)
-
-    if len(inputs) != 1:
-        raise ModelDefinitionError(
-            "All parameters must be keyword arguments")
-
-    return custom_model(func, fit_deriv=func_fit_deriv)
