@@ -7,13 +7,14 @@ Creates a common namespace for all pre-defined models.
 from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
-from .core import custom_model
+from .core import custom_model  # pylint: disable=W0611
 from .mappings import *
 from .projections import *
 from .rotations import *
 from .polynomial import *
 from .functional_models import *
 from .powerlaws import *
+from .tabular import *
 
 from ..extern.six import string_types
 
@@ -59,7 +60,7 @@ MODELS_WITH_CONSTRAINTS = [
     Gaussian1D, GaussianAbsorption1D, Gaussian2D,
     Linear1D, Lorentz1D, MexicanHat1D, MexicanHat2D,
     PowerLaw1D, Sersic1D, Sersic2D, Sine1D, Trapezoid1D, TrapezoidDisk2D,
-    Chebyshev1D, Chebyshev2D, Legendre2D, Legendre1D,
+    Chebyshev1D, Chebyshev2D, Hermite1D, Hermite2D, Legendre2D, Legendre1D,
     Polynomial1D, Polynomial2D, Voigt1D
 ]
 
