@@ -5,13 +5,13 @@ from __future__ import (absolute_import, unicode_literals, division,
 
 import numpy as np
 from numpy.testing import assert_allclose, assert_array_equal
-from .. import models
+from .. import models, InputParameterError
 from ...coordinates import Angle
 from .. import fitting
 from ...tests.helper import pytest
 
 try:
-    from scipy import optimize
+    from scipy import optimize  # pylint: disable=W0611
     HAS_SCIPY = True
 except ImportError:
     HAS_SCIPY = False
@@ -90,18 +90,40 @@ def test_Gaussian2DRotation():
     assert_allclose(value1, value2)
 
 
-def test_Redshift():
+def test_Gaussian2D_invalid_inputs():
+    x_stddev = 5.1
+    y_stddev = 3.3
+    theta = 10
+    cov_matrix = [[49., -16.], [-16., 9.]]
+
+    # first make sure the valid ones are OK
+    models.Gaussian2D()
+    models.Gaussian2D(x_stddev=x_stddev, y_stddev=y_stddev, theta=theta)
+    models.Gaussian2D(x_stddev=None, y_stddev=y_stddev, theta=theta)
+    models.Gaussian2D(x_stddev=x_stddev, y_stddev=None, theta=theta)
+    models.Gaussian2D(x_stddev=x_stddev, y_stddev=y_stddev, theta=None)
+    models.Gaussian2D(cov_matrix=cov_matrix)
+
+    with pytest.raises(InputParameterError):
+        models.Gaussian2D(x_stddev=0, cov_matrix=cov_matrix)
+    with pytest.raises(InputParameterError):
+        models.Gaussian2D(y_stddev=0, cov_matrix=cov_matrix)
+    with pytest.raises(InputParameterError):
+        models.Gaussian2D(theta=0, cov_matrix=cov_matrix)
+
+
+def test_RedshiftScaleFactor():
     """Like ``test_ScaleModel()``."""
 
     # Scale by a scalar
-    m = models.Redshift(0.4)
+    m = models.RedshiftScaleFactor(0.4)
     assert m(0) == 0
     assert_array_equal(m([1, 2]), [1.4, 2.8])
 
     assert_allclose(m.inverse(m([1, 2])), [1, 2])
 
     # Scale by a list
-    m = models.Redshift([-0.5, 0, 0.5], model_set_axis=0)
+    m = models.RedshiftScaleFactor([-0.5, 0, 0.5], n_models=3)
     assert_array_equal(m(0), 0)
     assert_array_equal(m([1, 2], model_set_axis=False),
                        [[0.5, 1], [1, 2], [1.5, 3]])
@@ -162,3 +184,23 @@ def test_Voigt1D():
     fitter = fitting.LevMarLSQFitter()
     voi_fit = fitter(voi_init, xarr, yarr)
     assert_allclose(voi_fit.param_sets, voi.param_sets)
+
+
+@pytest.mark.skipif("not HAS_SCIPY")
+def test_compound_models_with_class_variables():
+    models_2d = [models.AiryDisk2D, models.Sersic2D]
+    models_1d = [models.Sersic1D]
+
+    for model_2d in models_2d:
+        class CompoundModel2D(models.Const2D + model_2d):
+            pass
+        x, y = np.mgrid[:10, :10]
+        f = CompoundModel2D()(x, y)
+        assert f.shape == (10, 10)
+
+    for model_1d in models_1d:
+        class CompoundModel1D(models.Const1D + model_1d):
+            pass
+        x = np.arange(10)
+        f = CompoundModel1D()(x)
+        assert f.shape == (10,)
