@@ -23,14 +23,18 @@ from .. import fitting, models
 from ..core import FittableModel
 from ..polynomial import PolynomialBase
 from ...tests.helper import pytest
-
+from ...utils import minversion
+from ...extern.six.moves import zip
 
 try:
+    import scipy
     from scipy import optimize  # pylint: disable=W0611
     HAS_SCIPY = True
 except ImportError:
     HAS_SCIPY = False
 
+HAS_SCIPY_14 = HAS_SCIPY and minversion(scipy, "0.14")
+
 
 @pytest.mark.skipif('not HAS_SCIPY')
 def test_custom_model(amplitude=4, frequency=1):
@@ -364,15 +368,21 @@ class Fittable1DModelTester(object):
         except NotImplementedError:
             pytest.skip("Bounding_box is not defined for model.")
 
-        dx = np.diff(bbox) / 2
-        x1 = np.mgrid[slice(bbox[0], bbox[1] + 1)]
-        x2 = np.mgrid[slice(bbox[0] - dx, bbox[1] + dx + 1)]
+        if isinstance(model, models.Lorentz1D):
+            rtol = 0.01  # 1% agreement is enough due to very extended wings
+            ddx = 0.1  # Finer sampling to "integrate" flux for narrow peak
+        else:
+            rtol = 1e-7
+            ddx = 1
 
+        dx = np.diff(bbox) / 2
+        x1 = np.mgrid[slice(bbox[0], bbox[1] + 1, ddx)]
+        x2 = np.mgrid[slice(bbox[0] - dx, bbox[1] + dx + 1, ddx)]
         arr = model(x2)
         sub_arr = model(x1)
 
         # check for flux agreement
-        assert abs(arr.sum() - sub_arr.sum()) < arr.sum() * 1e-7
+        assert abs(arr.sum() - sub_arr.sum()) < arr.sum() * rtol
 
     @pytest.mark.skipif('not HAS_SCIPY')
     def test_fitter1D(self, model_class, test_parameters):
@@ -485,7 +495,7 @@ def test_ShiftModel():
     m = models.Shift([42, 43], n_models=2)
     utils.assert_equal(m(0), [42, 43])
     utils.assert_equal(m([1, 2], model_set_axis=False),
-                       [[ 43,  44], [ 44,  45]])
+                       [[43, 44], [44, 45]])
 
 
 def test_ScaleModel():
@@ -498,7 +508,7 @@ def test_ScaleModel():
     m = models.Scale([42, 43], n_models=2)
     utils.assert_equal(m(0), [0, 0])
     utils.assert_equal(m([1, 2], model_set_axis=False),
-                       [[ 42,  84], [ 43,  86]])
+                       [[42, 84], [43, 86]])
 
 
 def test_voigt_model():
@@ -516,3 +526,67 @@ def test_voigt_model():
 def test_model_instance_repr():
     m = models.Gaussian1D(1, 2, 3)
     assert repr(m) == '<Gaussian1D(amplitude=1.0, mean=2.0, stddev=3.0)>'
+
+
+@pytest.mark.skipif("not HAS_SCIPY_14")
+def test_tabular_interp_1d():
+    """
+    Test Tabular1D model.
+    """
+    points = np.arange(0, 5)
+    values = [1., 10, 2, 45, -3]
+    LookupTable = models.tabular_model(1)
+    model = LookupTable(points=points, lookup_table=values)
+    xnew = [0., .7, 1.4, 2.1, 3.9]
+    utils.assert_allclose(model(xnew), [1., 7.3, 6.8, 6.3, 1.8])
+    # Test evaluate without passing `points`.
+    model = LookupTable(lookup_table=values)
+    utils.assert_allclose(model(xnew), [1., 7.3, 6.8, 6.3, 1.8])
+    # Test bounds error.
+    with pytest.raises(ValueError):
+        model([0., .7, 1.4, 2.1, 3.9, 4.1])
+    # test extrapolation and fill value
+    model = LookupTable(lookup_table=values, bounds_error=False,
+                        fill_value=None)
+    utils.assert_allclose(model([0., .7, 1.4, 2.1, 3.9, 4.1]),
+                          [1., 7.3, 6.8, 6.3, 1.8, -7.8])
+
+
+@pytest.mark.skipif("not HAS_SCIPY_14")
+def test_tabular_interp_2d():
+    table = np.array([
+       [-0.04614432, -0.02512547, -0.00619557, 0.0144165, 0.0297525],
+       [-0.04510594, -0.03183369, -0.01118008, 0.01201388, 0.02496205],
+       [-0.05464094, -0.02804499, -0.00960086, 0.01134333, 0.02284104],
+       [-0.04879338, -0.02539565, -0.00440462, 0.01795145, 0.02122417],
+       [-0.03637372, -0.01630025, -0.00157902, 0.01649774, 0.01952131]])
+
+    points = (np.arange(0, 5), np.arange(0, 5))
+
+    xnew = np.array([0., .7, 1.4, 2.1, 3.9])
+    LookupTable = models.tabular_model(2)
+    model = LookupTable(points, table)
+    znew = model(xnew, xnew)
+    result = np.array(
+        [-0.04614432, -0.03450009, -0.02241028, -0.0069727, 0.01938675])
+    utils.assert_allclose(znew, result, atol=10**-7)
+
+    # test 2D arrays as input
+    a = np.arange(12).reshape((3, 4))
+    y, x = np.mgrid[:3, :4]
+    t = models.Tabular2D(lookup_table=a)
+    result = t(y, x)
+    utils.assert_allclose(a, result)
+
+    with pytest.raises(ValueError):
+        model = LookupTable(points=([1.2, 2.3], [1.2, 6.7], [3, 4]))
+
+
+@pytest.mark.skipif("not HAS_SCIPY_14")
+def test_tabular_nd():
+    a = np.arange(24).reshape((2, 3, 4))
+    x, y, z = np.mgrid[:2, :3, :4]
+    tab = models.tabular_model(3)
+    t = tab(lookup_table=a)
+    result = t(x, y, z)
+    utils.assert_allclose(a, result)
