@@ -6,35 +6,42 @@ from __future__ import (absolute_import, unicode_literals, division,
 from math import cos, sin
 import numpy as np
 from numpy.testing import utils
-from .. import models
 
+from .. import models
+from ...wcs import wcs
+from ...io import fits
+from ...utils.data import get_pkg_data_filename
 from ...tests.helper import pytest
 
 
-def test_RotateNative2Celestial():
-    lon, lat, lon_pole = 42, 43, 44
-    model = models.RotateNative2Celestial(lon, lat, lon_pole)
-    model.lon = model.lon + 1
-    model.lon_pole = model.lon_pole + 1
-    model.lat = model.lat + 1
-    assert model.lon == lon + 1
-    assert model.lat == lat + 1
-    assert model.lon_pole == lon_pole + 1
+@pytest.mark.parametrize(('inp'), [(0, 0), (4000, -20.56), (-2001.5, 45.9), (0, 90), (0, -90), (np.mgrid[:4, :6])])
+def test_against_wcslib(inp):
+    w = wcs.WCS()
+    crval = [ 202.4823228 , 47.17511893]
+    w.wcs.crval = crval
+    w.wcs.ctype=['RA---TAN', 'DEC--TAN']
+
+    lonpole = 180
+    tan = models.Pix2Sky_TAN()
+    n2c = models.RotateNative2Celestial(crval[0], crval[1], lonpole)
+    c2n = models.RotateCelestial2Native(crval[0], crval[1], lonpole)
+    m = tan | n2c
+    minv = c2n | tan.inverse
+
+    radec = w.wcs_pix2world(inp[0], inp[1], 1)
+    xy = w.wcs_world2pix(radec[0], radec[1], 1)
+
+    utils.assert_allclose(m(*inp), radec, atol=1e-12)
+    utils.assert_allclose(minv(*radec), xy, atol=1e-12)
 
 
-def test_native_celestial_native():
+@pytest.mark.parametrize(('inp'), [(0, 0), (40, -20.56), (21.5, 45.9)])
+def test_roundtrip_sky_rotaion(inp):
     lon, lat, lon_pole = 42, 43, 44
     n2c = models.RotateNative2Celestial(lon, lat, lon_pole)
     c2n = models.RotateCelestial2Native(lon, lat, lon_pole)
-
-    nnphi, ntheta = 33, 44
-    calpha, cdelta = n2c(nnphi, ntheta)
-    nnphi2, ntheta2 = c2n(calpha, cdelta)
-    utils.assert_allclose(nnphi2, nnphi)
-    utils.assert_allclose(ntheta2, ntheta)
-
-    assert n2c.inverse(nnphi, ntheta) == c2n(nnphi, ntheta)
-    assert c2n.inverse(nnphi, ntheta) == n2c(nnphi, ntheta)
+    utils.assert_allclose(n2c.inverse(*n2c(*inp)), inp, atol=1e-13)
+    utils.assert_allclose(c2n.inverse(*c2n(*inp)), inp, atol=1e-13)
 
 
 def test_native_celestial_lat90():
@@ -74,36 +81,6 @@ def test_euler_angle_rotations():
     utils.assert_allclose(model(*x), negy, atol=10**-12)
 
 
-def test_euler_against_sky():
-    """
-    Test sky transformations against euler angle rotations
-    """
-    # Write the Euler angles in terms of FITS WCS angles on the sky
-
-    # This corresponds to FITS WCS CRVAL1 = 5.63 deg
-    phi = 5.63 + 90
-    # This corresponds to FITS WCS CRVAL2 = -72.63 deg
-    theta = 90 - -72.63
-    # This corresponds to FITS WCS LONPOLE = 180 deg
-    psi = 180 - 90
-
-    # Create rotations between celestial and Native systems
-    c2n = models.RotateCelestial2Native(5.63, -72.63, 180)
-    n2c = models.RotateNative2Celestial(5.63, -72.63, 180)
-
-    # And the corresponding rotation in terms of Euler angles
-    # This corresponds to c2n since it's rotating coordinate systems,
-    # not vectors
-    model= models.EulerAngleRotation(phi, theta, -psi, 'zxz')
-
-    utils.assert_allclose(model(1, 23.1), c2n(1, 23.1))
-    ra, dec = model.inverse(1, 23.1)
-    if ra < 0:
-        ra += 360
-    utils.assert_allclose((ra, dec), n2c(1, 23.1))
-
-
-
 euler_axes_order = ['zxz', 'zyz', 'yzy', 'yxy', 'xyx', 'xzx']
 
 @pytest.mark.parametrize(('axes_order'), euler_axes_order)
@@ -147,4 +124,3 @@ def test_euler_angles(axes_order):
 
 
     utils.assert_allclose(mat.T, matrices[axes_order])#get_rotation_matrix(axes_order))
-
