@@ -11,9 +11,10 @@ from ..utils import (extract_array, add_array, subpixel_indices,
 from ...wcs import WCS
 from ...coordinates import SkyCoord
 from ... import units as u
+from ...extern.six.moves import zip
 
 try:
-    import skimage
+    import skimage  # pylint: disable=W0611
     HAS_SKIMAGE = True
 except ImportError:
     HAS_SKIMAGE = False
@@ -332,7 +333,8 @@ class TestCutout2D(object):
         wcs.wcs.cd = [[scale*np.cos(rho), -scale*np.sin(rho)],
                         [scale*np.sin(rho), scale*np.cos(rho)]]
         wcs.wcs.ctype = ['RA---TAN', 'DEC--TAN']
-        wcs.wcs.crval = [self.position.ra.value, self.position.dec.value]
+        wcs.wcs.crval = [self.position.ra.to(u.deg).value,
+                         self.position.dec.to(u.deg).value]
         wcs.wcs.crpix = [3, 3]
         self.wcs = wcs
 
