@@ -3,7 +3,7 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 import operator
 import numpy as np
-from ..extern.six.moves import zip
+from ..extern.six.moves import zip, range
 
 
 class MaxValue(object):
