@@ -15,29 +15,21 @@ class Conf(_config.ConfigNamespace):
     """
 
     jquery_url = _config.ConfigItem(
-        'https://code.jquery.com/jquery-1.11.3.min.js',
+        'https://code.jquery.com/jquery-3.1.1.min.js',
         'The URL to the jquery library.')
 
     datatables_url = _config.ConfigItem(
-        'https://cdn.datatables.net/1.10.9/js/jquery.dataTables.min.js',
+        'https://cdn.datatables.net/1.10.12/js/jquery.dataTables.min.js',
         'The URL to the jquery datatables library.')
 
     css_urls = _config.ConfigItem(
-        ['https://cdn.datatables.net/1.10.9/css/jquery.dataTables.css'],
+        ['https://cdn.datatables.net/1.10.12/css/jquery.dataTables.css'],
         'The URLs to the css file(s) to include.', cfgtype='list')
 
 
 conf = Conf()
 
 
-JQUERY_URL = _config.ConfigAlias(
-    '0.4', 'JQUERY_URL', 'jquery_url',
-    'astropy.table.jsviewer', 'astropy.table.jsviewer')
-
-DATATABLES_URL = _config.ConfigAlias(
-    '0.4', 'DATATABLES_URL', 'datatables_url',
-    'astropy.table.jsviewer', 'astropy.table.jsviewer')
-
 EXTERN_JS_DIR = abspath(join(dirname(extern.__file__), 'js'))
 EXTERN_CSS_DIR = abspath(join(dirname(extern.__file__), 'css'))
 
@@ -49,6 +41,7 @@ require.config({{paths: {{
 require(["datatables"], function(){{
     console.log("$('#{tid}').dataTable()");
     $('#{tid}').dataTable({{
+        "order": [],
         "iDisplayLength": {display_length},
         "aLengthMenu": {display_length_menu},
         "pagingType": "full_numbers"
@@ -60,9 +53,10 @@ require(["datatables"], function(){{
 HTML_JS_SCRIPT = """
 $(document).ready(function() {{
     $('#{tid}').dataTable({{
-     "iDisplayLength": {display_length},
-     "aLengthMenu": {display_length_menu},
-     "pagingType": "full_numbers"
+        "order": [],
+        "iDisplayLength": {display_length},
+        "aLengthMenu": {display_length_menu},
+        "pagingType": "full_numbers"
     }});
 }} );
 """
@@ -114,7 +108,7 @@ class JSViewer(object):
     @property
     def jquery_urls(self):
         if self._use_local_files:
-            return ['file://' + join(EXTERN_JS_DIR, 'jquery-1.11.3.min.js'),
+            return ['file://' + join(EXTERN_JS_DIR, 'jquery-3.1.1.min.js'),
                     'file://' + join(EXTERN_JS_DIR, 'jquery.dataTables.min.js')]
         else:
             return [conf.jquery_url, conf.datatables_url]
