@@ -9,12 +9,15 @@ from collections import OrderedDict
 import numpy as np
 
 from ...extern import six
+from ...extern.six.moves import cStringIO as StringIO
 from ... import units as u
 from ... import time
 from ... import coordinates
 from ... import table
 from ...utils.data_info import data_info_factory, dtype_info_name
 from ...utils.compat import NUMPY_LT_1_8
+from ..table_helpers import simple_table
+
 
 def test_table_info_attributes(table_types):
     """
@@ -55,7 +58,7 @@ def test_table_info_attributes(table_types):
     assert np.all(tinfo['class'] == [cls, cls, cls, cls, 'Time', 'SkyCoord'])
 
     # Test that repr(t.info) is same as t.info()
-    out = six.moves.cStringIO()
+    out = StringIO()
     t.info(out=out)
     assert repr(t.info) == out.getvalue()
 
@@ -71,7 +74,7 @@ def test_table_info_stats(table_types):
 
     # option = 'stats'
     masked = 'masked=True ' if t.masked else ''
-    out = six.moves.cStringIO()
+    out = StringIO()
     t.info('stats', out=out)
     table_header_line = '<{0} {1}length=4>'.format(t.__class__.__name__, masked)
     exp = [table_header_line,
@@ -92,7 +95,7 @@ def test_table_info_stats(table_types):
     assert np.all(tinfo['min'] == ['1', '1.0', '--', '1.0'])
     assert np.all(tinfo['max'] == ['2', '2.0', '--', '2.0'])
 
-    out = six.moves.cStringIO()
+    out = StringIO()
     t.info('stats', out=out)
     exp = [table_header_line,
            'name mean std min max',
@@ -106,7 +109,7 @@ def test_table_info_stats(table_types):
     # option = ['attributes', custom]
     custom = data_info_factory(names=['sum', 'first'],
                                funcs=[np.sum, lambda col: col[0]])
-    out = six.moves.cStringIO()
+    out = StringIO()
     tinfo = t.info(['attributes', custom], out=None)
     assert tinfo.colnames == ['name', 'dtype', 'shape', 'unit', 'format', 'description',
                               'class', 'sum', 'first', 'n_bad', 'length']
@@ -138,7 +141,7 @@ def test_data_info():
                                      ('length', 3)])
 
         # Test the console (string) version which omits trivial values
-        out = six.moves.cStringIO()
+        out = StringIO()
         c.info(out=out)
         exp = ['name = name',
                'dtype = float64',
@@ -194,7 +197,7 @@ def test_scalar_info():
 
 def test_empty_table():
     t = table.Table()
-    out = six.moves.cStringIO()
+    out = StringIO()
     t.info(out=out)
     exp = ['<Table length=0>', '<No columns>']
     assert out.getvalue().splitlines() == exp
@@ -222,7 +225,7 @@ def test_class_attribute():
     for table_cls, exp in ((table.Table, texp),
                            (table.QTable, qexp)):
         t = table_cls(vals)
-        out = six.moves.cStringIO()
+        out = StringIO()
         t.info(out=out)
         assert out.getvalue().splitlines() == exp
 
@@ -232,3 +235,12 @@ def test_ignore_warnings():
     with warnings.catch_warnings(record=True) as warns:
         t.info('stats', out=None)
         assert len(warns) == 0
+
+
+def test_no_deprecation_warning():
+    # regression test for #5459, where numpy deprecation warnings were
+    # emitted unnecessarily.
+    t = simple_table()
+    with warnings.catch_warnings(record=True) as warns:
+        t.info()
+        assert len(warns) == 0
