@@ -1,3 +1,4 @@
+# This Python file uses the following encoding: utf-8
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
 # TEST_UNICODE_LITERALS
@@ -8,7 +9,7 @@ from ...tests.helper import pytest
 from ... import table
 from ...table import Table
 from ...table.table_helpers import simple_table
-from ...extern.six import PY3
+from ...extern.six import PY2
 from ...utils import console
 
 BIG_WIDE_ARR = np.arange(2000, dtype=np.float64).reshape(100, 20)
@@ -39,9 +40,10 @@ class TestMultiD():
                          '<tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td></tr>',
                          '<tr><td>10 .. 20</td><td>30 .. 40</td><td>50 .. 60</td></tr>',
                          '</table>']
+        nbclass = table.conf.default_notebook_table_class
         assert t._repr_html_().splitlines() == [
             '&lt;{0} masked={1} length=2&gt;'.format(table_type.__name__, t.masked),
-            '<table id="table{tid}">'.format(tid=id(t)),
+            '<table id="table{id}" class="{nbclass}">'.format(id=id(t), nbclass=nbclass),
             '<thead><tr><th>col0 [2]</th><th>col1 [2]</th><th>col2 [2]</th></tr></thead>',
             '<thead><tr><th>int64</th><th>int64</th><th>int64</th></tr></thead>',
             '<tr><td>1 .. 2</td><td>3 .. 4</td><td>5 .. 6</td></tr>',
@@ -77,9 +79,10 @@ class TestMultiD():
                          '<tr><td>1</td><td>3</td><td>5</td></tr>',
                          '<tr><td>10</td><td>30</td><td>50</td></tr>',
                          '</table>']
+        nbclass = table.conf.default_notebook_table_class
         assert t._repr_html_().splitlines() == [
             '&lt;{0} masked={1} length=2&gt;'.format(table_type.__name__, t.masked),
-            '<table id="table{id}">'.format(id=id(t)),
+            '<table id="table{id}" class="{nbclass}">'.format(id=id(t), nbclass=nbclass),
             '<thead><tr><th>col0 [1,1]</th><th>col1 [1,1]</th><th>col2 [1,1]</th></tr></thead>',
             '<thead><tr><th>int64</th><th>int64</th><th>int64</th></tr></thead>',
             '<tr><td>1</td><td>3</td><td>5</td></tr>', u'<tr><td>10</td><td>30</td><td>50</td></tr>',
@@ -96,9 +99,10 @@ class TestMultiD():
 
 def test_html_escaping():
     t = table.Table([(str('<script>alert("gotcha");</script>'), 2, 3)])
+    nbclass = table.conf.default_notebook_table_class
     assert t._repr_html_().splitlines() == [
         '&lt;Table length=3&gt;',
-        '<table id="table{id}">'.format(id=id(t)),
+        '<table id="table{id}" class="{nbclass}">'.format(id=id(t), nbclass=nbclass),
         '<thead><tr><th>col0</th></tr></thead>',
         '<thead><tr><th>str33</th></tr></thead>',
         '<tr><td>&lt;script&gt;alert(&quot;gotcha&quot;);&lt;/script&gt;</td></tr>',
@@ -525,13 +529,15 @@ def test_pprint_npfloat32():
 
 def test_pprint_py3_bytes():
     """
-    Test for #1346.  Make sure a bytestring (dtype=S<N>) in Python 3 is printed
-    correctly (without the "b" prefix like b'string').
+    Test for #1346 and #4944. Make sure a bytestring (dtype=S<N>) in Python 3
+    is printed correctly (without the "b" prefix like b'string').
+    Also make sure special characters are printed in Python 2.
     """
-    val = bytes('val', encoding='utf-8') if PY3 else 'val'
-    dat = np.array([(val,)], dtype=[(str('col'), 'S3')])
+    val = str('val') if PY2 else bytes('val', encoding='utf-8')
+    blah = u'bläh'.encode('utf-8') if PY2 else bytes('bläh', encoding='utf-8')
+    dat = np.array([val, blah], dtype=[(str('col'), 'S10')])
     t = table.Table(dat)
-    assert t['col'].pformat() == ['col', '---', 'val']
+    assert t['col'].pformat() == ['col ', '----', ' val', u'bl\xe4h']
 
 
 def test_pprint_nameless_col():
