@@ -14,13 +14,13 @@ from numpy.testing import assert_allclose
 from ...extern import six
 from ...io import fits
 from ...tests.helper import (pytest, assert_follows_unicode_guidelines,
-                             ignore_warnings)
+                             ignore_warnings, catch_warnings)
 from ...utils.data import get_pkg_data_filename
 from ... import table
 from ... import units as u
 from .conftest import MaskedTable
 
-from ...extern.six.moves import zip as izip, cStringIO as StringIO
+from ...extern.six.moves import zip, range, cStringIO as StringIO
 
 try:
     with ignore_warnings(DeprecationWarning):
@@ -263,7 +263,7 @@ class TestNewFromColumns():
         t = table_types.Table(cols)
         assert np.all(t['a'].data == np.array([1, 2, 3]))
         assert np.all(t['b'].data == np.array([4, 5, 6], dtype=np.float32))
-        assert type(t['b'][1]) == np.float32
+        assert type(t['b'][1]) is np.float32
 
     def test_from_np_array(self, table_types):
         cols = [table_types.Column(name='a', data=np.array([1, 2, 3], dtype=np.int64),
@@ -272,8 +272,8 @@ class TestNewFromColumns():
         t = table_types.Table(cols)
         assert np.all(t['a'] == np.array([1, 2, 3], dtype=np.float64))
         assert np.all(t['b'] == np.array([4, 5, 6], dtype=np.float32))
-        assert type(t['a'][1]) == np.float64
-        assert type(t['b'][1]) == np.float32
+        assert type(t['a'][1]) is np.float64
+        assert type(t['b'][1]) is np.float32
 
     def test_size_mismatch(self, table_types):
         cols = [table_types.Column(name='a', data=[1, 2, 3]),
@@ -339,7 +339,7 @@ class TestColumnAccess():
     def test_itercols(self, table_types):
         names = ['a', 'b', 'c']
         t = table_types.Table([[1], [2], [3]], names=names)
-        for name, col in izip(names, t.itercols()):
+        for name, col in zip(names, t.itercols()):
             assert name == col.name
             assert isinstance(col, table_types.Column)
 
@@ -1097,7 +1097,7 @@ class TestConvertNumpyArray():
         np_data = np.array(d)
         if table_types.Table is not MaskedTable:
             assert np.all(np_data == d.as_array())
-        assert not np_data is d.as_array()
+        assert np_data is not d.as_array()
         assert d.colnames == list(np_data.dtype.names)
 
         np_data = np.array(d, copy=False)
@@ -1712,6 +1712,145 @@ def test_replace_column_qtable():
     assert t['a'].info.meta is None
     assert t['a'].info.format is None
 
+def test_replace_update_column_via_setitem():
+    """
+    Test table update like ``t['a'] = value``.  This leverages off the
+    already well-tested ``replace_column`` and in-place update
+    ``t['a'][:] = value``, so this testing is fairly light.
+    """
+    a = [1, 2] * u.m
+    b = [3, 4]
+    t = table.QTable([a, b], names=['a', 'b'])
+    assert isinstance(t['a'], u.Quantity)
+
+    # Inplace update
+    ta = t['a']
+    t['a'] = 5 * u.m
+    assert np.all(t['a'] == [5, 5] * u.m)
+    assert t['a'] is ta
+
+    # Replace
+    t['a'] = [5, 6]
+    assert np.all(t['a'] == [5, 6])
+    assert isinstance(t['a'], table.Column)
+    assert t['a'] is not ta
+
+
+def test_replace_update_column_via_setitem_warnings_normal():
+    """
+    Test warnings related to table replace change in #5556:
+    Normal warning-free replace
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t['a'] = 0  # in-place update
+            assert len(w) == 0
+
+            t['a'] = [10, 20, 30]  # replace column
+            assert len(w) == 0
+
+
+def test_replace_update_column_via_setitem_warnings_slice():
+    """
+    Test warnings related to table replace change in #5556:
+    Replace a slice, one warning.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t2 = t[:2]
+
+            t2['a'] = 0  # in-place slice update
+            assert np.all(t['a'] == [0, 0, 3])
+            assert len(w) == 0
+
+            t2['a'] = [10, 20]  # replace slice
+            assert len(w) == 1
+            assert "replaced column 'a' which looks like an array slice" in str(w[0].message)
+
+
+def test_replace_update_column_via_setitem_warnings_attributes():
+    """
+    Test warnings related to table replace change in #5556:
+    Lost attributes.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    t['a'].unit = 'm'
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t['a'] = [10, 20, 30]
+        assert len(w) == 1
+        assert "replaced column 'a' and column attributes ['unit']" in str(w[0].message)
+
+
+def test_replace_update_column_via_setitem_warnings_refcount():
+    """
+    Test warnings related to table replace change in #5556:
+    Reference count changes.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    ta = t['a']  # Generate an extra reference to original column
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings',
+                                 ['refcount', 'attributes', 'slice']):
+            t['a'] = [10, 20, 30]
+        assert len(w) == 1
+        assert "replaced column 'a' and the number of references" in str(w[0].message)
+
+
+def test_replace_update_column_via_setitem_warnings_always():
+    """
+    Test warnings related to table replace change in #5556:
+    Test 'always' setting that raises warning for any replace.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_warnings', ['always']):
+            t['a'] = 0  # in-place slice update
+            assert len(w) == 0
+
+            from inspect import currentframe, getframeinfo
+            frameinfo = getframeinfo(currentframe())
+            t['a'] = [10, 20, 30]  # replace column
+            assert len(w) == 1
+            assert "replaced column 'a'" == str(w[0].message)
+
+            # Make sure the warning points back to the user code line
+            assert w[0].lineno == frameinfo.lineno + 1
+            assert w[0].category is table.TableReplaceWarning
+            assert 'test_table' in w[0].filename
+
+
+def test_replace_update_column_via_setitem_replace_inplace():
+    """
+    Test the replace_inplace config option related to #5556.  In this
+    case no replace is done.
+    """
+    t = table.Table([[1, 2, 3], [4, 5, 6]], names=['a', 'b'])
+    ta = t['a']
+    t['a'].unit = 'm'
+
+    with catch_warnings() as w:
+        with table.conf.set_temp('replace_inplace', True):
+            with table.conf.set_temp('replace_warnings',
+                                     ['always', 'refcount', 'attributes', 'slice']):
+                t['a'] = 0  # in-place update
+                assert len(w) == 0
+                assert ta is t['a']
+
+                t['a'] = [10, 20, 30]  # normally replaces column, but not now
+                assert len(w) == 0
+                assert ta is t['a']
+                assert np.all(t['a'] == [10, 20, 30])
+
+
 def test_primary_key_is_inherited():
     """Test whether a new Table inherits the primary_key attribute from
     its parent Table. Issue #4672"""
@@ -1746,4 +1885,4 @@ def test_qtable_read_for_ipac_table_with_char_columns():
     out = StringIO()
     t1.write(out, format="ascii.ipac")
     t2 = table.QTable.read(out.getvalue(), format="ascii.ipac", guess=False)
-    assert t2["B"].unit == None
+    assert t2["B"].unit is None
