@@ -22,18 +22,16 @@ try:
     # Import pkg_resources to prevent it from issuing warnings upon being
     # imported from within py.test.  See
     # https://github.com/astropy/astropy/pull/537 for a detailed explanation.
-    import pkg_resources
+    import pkg_resources  # pylint: disable=W0611
 except ImportError:
     pass
 
-from .. import test
-from ..utils.exceptions import (AstropyWarning,
-                                AstropyDeprecationWarning,
+from ..utils.exceptions import (AstropyDeprecationWarning,
                                 AstropyPendingDeprecationWarning)
 
 
 # For backward-compatibility with affiliated packages
-from .runner import TestRunner
+from .runner import TestRunner  # pylint: disable=W0611
 
 __all__ = ['raises', 'enable_deprecations_as_exceptions', 'remote_data',
            'treat_deprecations_as_exceptions', 'catch_warnings',
@@ -48,19 +46,19 @@ if os.environ.get('ASTROPY_USE_SYSTEM_PYTEST') or '_pytest' in sys.modules:
 else:
     from ..extern import pytest as extern_pytest
 
-    if six.PY3:
+    if six.PY2:
+        exec("def do_exec_def(co, loc): exec co in loc\n")
+        extern_pytest.do_exec = do_exec_def
+
+        unpacked_sources = pickle.loads(
+            zlib.decompress(base64.decodestring(extern_pytest.sources)))
+    else:
         exec("def do_exec_def(co, loc): exec(co, loc)\n")
         extern_pytest.do_exec = do_exec_def
 
         unpacked_sources = extern_pytest.sources.encode("ascii")
         unpacked_sources = pickle.loads(
             zlib.decompress(base64.decodebytes(unpacked_sources)), encoding='utf-8')
-    elif six.PY2:
-        exec("def do_exec_def(co, loc): exec co in loc\n")
-        extern_pytest.do_exec = do_exec_def
-
-        unpacked_sources = pickle.loads(
-            zlib.decompress(base64.decodestring(extern_pytest.sources)))
 
     importer = extern_pytest.DictImporter(unpacked_sources)
     sys.meta_path.insert(0, importer)
@@ -126,30 +124,26 @@ def _save_coverage(cov, result, rootdir, testing_path):
 
     # The coverage report includes the full path to the temporary
     # directory, so we replace all the paths with the true source
-    # path. This means that the coverage line-by-line report will only
-    # be correct for Python 2 code (since the Python 3 code will be
-    # different in the build directory from the source directory as
-    # long as 2to3 is needed). Therefore we only do this fix for
-    # Python 2.x.
-    if six.PY2:
-        try:
-            # Coverage 4.0: _harvest_data has been renamed to get_data, the
-            # lines dict is private
-            cov.get_data()
-        except AttributeError:
-            # Coverage < 4.0
-            cov._harvest_data()
-            lines = cov.data.lines
-        else:
-            lines = cov.data._lines
+    # path. Note that this will not work properly for packages that still
+    # rely on 2to3.
+    try:
+        # Coverage 4.0: _harvest_data has been renamed to get_data, the
+        # lines dict is private
+        cov.get_data()
+    except AttributeError:
+        # Coverage < 4.0
+        cov._harvest_data()
+        lines = cov.data.lines
+    else:
+        lines = cov.data._lines
 
-        for key in lines.keys():
-            new_path = os.path.relpath(
-                os.path.realpath(key),
-                os.path.realpath(testing_path))
-            new_path = os.path.abspath(
-                os.path.join(rootdir, new_path))
-            lines[new_path] = lines.pop(key)
+    for key in list(lines.keys()):
+        new_path = os.path.relpath(
+            os.path.realpath(key),
+            os.path.realpath(testing_path))
+        new_path = os.path.abspath(
+            os.path.join(rootdir, new_path))
+        lines[new_path] = lines.pop(key)
 
     color_print('Saving coverage data in .coverage...', 'green')
     cov.save()
@@ -216,7 +210,9 @@ def treat_deprecations_as_exceptions():
     This completely resets the warning filters and any "already seen"
     warning state.
     """
-    # First, totally reset the warning state
+    # First, totally reset the warning state. The modules may change during
+    # this iteration thus we copy the original state to a list to iterate
+    # on. See https://github.com/astropy/astropy/pull/5513.
     for module in list(six.itervalues(sys.modules)):
         # We don't want to deal with six.MovedModules, only "real"
         # modules.
@@ -238,12 +234,12 @@ def treat_deprecations_as_exceptions():
     # before we turn the warnings into exceptions, we're golden.
     try:
         # A deprecated stdlib module used by py.test
-        import compiler
+        import compiler  # pylint: disable=W0611
     except ImportError:
         pass
 
     try:
-        import scipy
+        import scipy  # pylint: disable=W0611
     except ImportError:
         pass
 
@@ -257,15 +253,6 @@ def treat_deprecations_as_exceptions():
         warnings.filterwarnings("error", ".*", AstropyDeprecationWarning)
         warnings.filterwarnings("error", ".*", AstropyPendingDeprecationWarning)
 
-    if sys.version_info[:2] == (2, 6):
-        # py.test's warning.showwarning does not include the line argument
-        # on Python 2.6, so we need to explicitly ignore this warning.
-        warnings.filterwarnings(
-            "ignore",
-            r"functions overriding warnings\.showwarning\(\) must support "
-            r"the 'line' argument",
-            DeprecationWarning)
-
     if sys.version_info[:2] >= (3, 4):
         # py.test reads files with the 'U' flag, which is now
         # deprecated in Python 3.4.
@@ -535,7 +522,7 @@ def _unquantify_allclose_arguments(actual, desired, rtol, atol):
     rtol =  u.Quantity(rtol, subok=True, copy=False)
     try:
         rtol = rtol.to(u.dimensionless_unscaled)
-    except:
+    except Exception:
         raise u.UnitsError("`rtol` should be dimensionless")
 
     return actual.value, desired.value, rtol.value, atol.value
