@@ -9,7 +9,7 @@ from __future__ import (absolute_import, division, print_function,
 import __future__
 
 from ..extern import six
-from ..extern.six.moves import filter
+from ..extern.six.moves import filter, range
 
 import ast
 import doctest
@@ -23,22 +23,20 @@ import os
 import re
 import sys
 import types
+import argparse
+from collections import OrderedDict
 
 from ..config.paths import set_temp_config, set_temp_cache
-from .helper import (
-    pytest, treat_deprecations_as_exceptions,
-    enable_deprecations_as_exceptions,
-    ignore_warnings)
+from .helper import pytest, treat_deprecations_as_exceptions, ignore_warnings
+from .helper import enable_deprecations_as_exceptions  # pylint: disable=W0611
 from .disable_internet import turn_off_internet, turn_on_internet
-from .output_checker import AstropyOutputChecker, FIX, FLOAT_CMP
-from ..utils import OrderedDict
+from .output_checker import AstropyOutputChecker, FIX
 from ..utils.argparse import writeable_directory
 from ..utils.introspection import resolve_name
 
-# Needed for Python 2.6 compatibility
 try:
     import importlib.machinery as importlib_machinery
-except ImportError:
+except ImportError:  # Python 2.7
     importlib_machinery = None
 
 # these pytest hooks allow us to mark tests and run the marked tests with
@@ -46,8 +44,13 @@ except ImportError:
 
 
 def pytest_addoption(parser):
-    parser.addoption("--remote-data", action="store_true",
+
+    # The following means that if --remote-data is not specified, the default
+    # is 'none', but if it is specified without arguments (--remote-data), it
+    # defaults to '--remote-data=any'.
+    parser.addoption("--remote-data", nargs="?", const='any', default='none',
                      help="run tests with online data")
+
     parser.addoption("--open-files", action="store_true",
                      help="fail if any test leaves files open")
 
@@ -137,8 +140,10 @@ def pytest_configure(config):
 
     # Monkeypatch to deny access to remote resources unless explicitly told
     # otherwise
-    if not config.getoption('remote_data'):
-        turn_off_internet(verbose=config.option.verbose)
+
+    if config.getoption('remote_data') != 'any':
+        turn_off_internet(verbose=config.option.verbose,
+                          allow_astropy_data=config.getoption('remote_data') == 'astropy')
 
     doctest_plugin = config.pluginmanager.getplugin('doctest')
     if (doctest_plugin is None or config.option.doctestmodules or not
@@ -161,21 +166,24 @@ def pytest_configure(config):
         # handling __doctest_skip__) doesn't happen.
         def collect(self):
             if self.fspath.basename == "conftest.py":
-                module = self.config._conftest.importconftest(self.fspath)
+                try:
+                    module = self.config._conftest.importconftest(self.fspath)
+                except AttributeError:  # pytest >= 2.8.0
+                    module = self.config.pluginmanager._importconftest(self.fspath)
             else:
                 try:
                     module = self.fspath.pyimport()
                     # Just ignore searching modules that can't be imported when
                     # collecting doctests
                 except ImportError:
-                    raise StopIteration
+                    return
 
             # uses internal doctest module parsing mechanism
             finder = DocTestFinderPlus()
             runner = doctest.DebugRunner(verbose=False, optionflags=opts)
             for test in finder.find(module):
                 if test.examples:  # skip empty doctests
-                    if not config.getvalue("remote_data"):
+                    if config.getvalue("remote_data") != 'any':
                         for example in test.examples:
                             if example.options.get(REMOTE_DATA):
                                 example.options[doctest.SKIP] = True
@@ -191,8 +199,14 @@ def pytest_configure(config):
         def runtest(self):
             # satisfy `FixtureRequest` constructor...
             self.funcargs = {}
-            self._fixtureinfo = doctest_plugin.FuncFixtureInfo((), [], {})
-            fixture_request = doctest_plugin.FixtureRequest(self)
+            try:
+                self._fixtureinfo = doctest_plugin.FuncFixtureInfo((), [], {})
+                fixture_request = doctest_plugin.FixtureRequest(self)
+            except AttributeError:  # pytest >= 2.8.0
+                python_plugin = config.pluginmanager.getplugin('python')
+                self._fixtureinfo = python_plugin.FuncFixtureInfo((), [], {})
+                fixture_request = python_plugin.FixtureRequest(self)
+
             failed, tot = doctest.testfile(
                 str(self.fspath), module_relative=False,
                 optionflags=opts, parser=DocTestParserPlus(),
@@ -261,7 +275,7 @@ def pytest_configure(config):
                         not DocTestFinderPlus.check_required_modules(required)):
                         entry.options[doctest.SKIP] = True
 
-                    if (not config.getvalue('remote_data') and
+                    if (config.getvalue('remote_data') != 'any' and
                         entry.options.get(REMOTE_DATA)):
                         entry.options[doctest.SKIP] = True
 
@@ -295,9 +309,6 @@ class DoctestPlus(object):
         # Directories to ignore when adding doctests
         self._ignore_paths = []
 
-        if run_rst_doctests and six.PY3:
-            self._run_rst_doctests = False
-
     def pytest_ignore_collect(self, path, config):
         """Skip paths that match any of the doctest_norecursedirs patterns."""
 
@@ -355,10 +366,10 @@ class DoctestPlus(object):
             return self._doctest_module_item_cls(path, parent)
         elif self._run_rst_doctests and path.ext == '.rst':
             # Ignore generated .rst files
-            parts = bytes(path).split(os.path.sep)
-            if (path.basename.startswith(b'_') or
-                any(x.startswith(b'_') for x in parts) or
-                any(x == b'api' for x in parts)):
+            parts = str(path).split(os.path.sep)
+            if (path.basename.startswith('_') or
+                    any(x.startswith('_') for x in parts) or
+                    any(x == 'api' for x in parts)):
                 return None
 
             # TODO: Get better names on these items when they are
@@ -399,8 +410,8 @@ class DocTestFinderPlus(doctest.DocTestFinder):
                 name = obj.__name__
             else:
                 raise ValueError("DocTestFinder.find: name must be given "
-                                 "when obj.__name__ doesn't exist: %r" %
-                                 (type(obj),))
+                                 "when obj.__name__ doesn't exist: {!r}".format(
+                        (type(obj),)))
 
             def test_filter(test):
                 for pat in getattr(obj, '__doctest_skip__', []):
@@ -412,7 +423,7 @@ class DocTestFinderPlus(doctest.DocTestFinder):
                         return False
 
                 reqs = getattr(obj, '__doctest_requires__', {})
-                for pats, mods in list(six.iteritems(reqs)):
+                for pats, mods in six.iteritems(reqs):
                     if not isinstance(pats, tuple):
                         pats = (pats,)
                     for pat in pats:
@@ -476,10 +487,22 @@ def pytest_runtest_setup(item):
     if item.config.getvalue('open_files'):
         item.open_files = _get_open_file_list()
 
-    if ('remote_data' in item.keywords and
-            not item.config.getvalue("remote_data")):
-        pytest.skip("need --remote-data option to run")
+    remote_data = item.keywords.get('remote_data')
 
+    remote_data_config = item.config.getvalue("remote_data")
+
+    if remote_data is not None:
+
+        source = remote_data.kwargs.get('source', 'any')
+
+        if source not in ('astropy', 'any'):
+            raise ValueError("source should be 'astropy' or 'any'")
+
+        if remote_data_config == 'none':
+            pytest.skip("need --remote-data option to run")
+        elif remote_data_config == 'astropy':
+            if source == 'any':
+                pytest.skip("need --remote-data option to run")
 
 def pytest_runtest_teardown(item, nextitem):
     if hasattr(item, 'set_temp_cache'):
@@ -547,11 +570,14 @@ TESTED_VERSIONS = OrderedDict([('Astropy', __version__)])
 
 def pytest_report_header(config):
 
-    stdoutencoding = getattr(sys.stdout, 'encoding') or 'ascii'
+    try:
+        stdoutencoding = sys.stdout.encoding or 'ascii'
+    except AttributeError:
+        stdoutencoding = 'ascii'
 
     if six.PY2:
         args = [x.decode('utf-8') for x in config.args]
-    elif six.PY3:
+    else:
         args = config.args
 
     # TESTED_VERSIONS can contain the affiliated package version, too
@@ -622,10 +648,7 @@ def pytest_report_header(config):
     if opts:
         s += "Using Astropy options: {0}.\n".format(" ".join(opts))
 
-    if six.PY3 and (config.getini('doctest_rst') or config.option.doctest_rst):
-        s += "Running doctests in .rst files is not supported on Python 3.x\n"
-
-    if not six.PY3:
+    if six.PY2:
         s = s.encode(stdoutencoding, 'replace')
 
     return s
@@ -636,10 +659,10 @@ def pytest_pycollect_makemodule(path, parent):
     # from __future__ import unicode_literals
 
     # On Python 3, just do the regular thing that py.test does
-    if six.PY3:
-        return pytest.Module(path, parent)
-    elif six.PY2:
+    if six.PY2:
         return Pair(path, parent)
+    else:
+        return pytest.Module(path, parent)
 
 
 class Pair(pytest.File):
@@ -661,14 +684,12 @@ class Pair(pytest.File):
             e = sys.exc_info()[1]
             raise self.CollectError(
                 "import file mismatch:\n"
-                "imported module %r has this __file__ attribute:\n"
-                "  %s\n"
+                "imported module {!r} has this __file__ attribute:\n"
+                "  {}\n"
                 "which is not the same as the test file we want to collect:\n"
-                "  %s\n"
+                "  {}\n"
                 "HINT: remove __pycache__ / .pyc files and/or use a "
-                "unique basename for your test file modules"
-                % e.args
-            )
+                "unique basename for your test file modules".format(e.args))
 
         # Now get the file's content.
         with io.open(six.text_type(self.fspath), 'rb') as fd:
