@@ -19,7 +19,8 @@ from .core import UnitsError
 __all__ = ['parallax', 'spectral', 'spectral_density', 'doppler_radio',
            'doppler_optical', 'doppler_relativistic', 'mass_energy',
            'brightness_temperature', 'dimensionless_angles',
-           'logarithmic', 'temperature', 'temperature_energy']
+           'logarithmic', 'temperature', 'temperature_energy',
+           'pixel_scale', 'plate_scale']
 
 
 def dimensionless_angles():
@@ -36,7 +37,7 @@ def logarithmic():
     """Allow logarithmic units to be converted to dimensionless fractions"""
     return [
         (dimensionless_unscaled, function_units.dex,
-         lambda x: np.log10(x), lambda x: 10.**x)
+         np.log10, lambda x: 10.**x)
     ]
 
 
@@ -115,12 +116,21 @@ def spectral_density(wav, factor=None):
     h_cgs = _si.h.cgs.value  # erg * s
     hc = c_Aps * h_cgs
 
-    fla = cgs.erg / si.angstrom / si.cm ** 2 / si.s
-    fnu = cgs.erg / si.Hz / si.cm ** 2 / si.s
-    nufnu = cgs.erg / si.cm ** 2 / si.s
-    lafla = nufnu
-    photlam = astrophys.photon / (si.cm ** 2 * si.s * si.AA)
-    photnu = astrophys.photon / (si.cm ** 2 * si.s * si.Hz)
+    # flux density
+    f_la = cgs.erg / si.angstrom / si.cm ** 2 / si.s
+    f_nu = cgs.erg / si.Hz / si.cm ** 2 / si.s
+    nu_f_nu = cgs.erg / si.cm ** 2 / si.s
+    la_f_la = nu_f_nu
+    phot_f_la = astrophys.photon / (si.cm ** 2 * si.s * si.AA)
+    phot_f_nu = astrophys.photon / (si.cm ** 2 * si.s * si.Hz)
+
+    # luminosity density
+    L_nu = cgs.erg / si.s / si.Hz
+    L_la = cgs.erg / si.s / si.angstrom
+    nu_L_nu = cgs.erg / si.s
+    la_L_la = nu_L_nu
+    phot_L_la = astrophys.photon / (si.s * si.AA)
+    phot_L_nu = astrophys.photon / (si.s * si.Hz)
 
     def converter(x):
         return x * (wav.to(si.AA, spectral()).value ** 2 / c_Aps)
@@ -128,55 +138,81 @@ def spectral_density(wav, factor=None):
     def iconverter(x):
         return x / (wav.to(si.AA, spectral()).value ** 2 / c_Aps)
 
-    def converter_fnu_nufnu(x):
+    def converter_f_nu_to_nu_f_nu(x):
         return x * wav.to(si.Hz, spectral()).value
 
-    def iconverter_fnu_nufnu(x):
+    def iconverter_f_nu_to_nu_f_nu(x):
         return x / wav.to(si.Hz, spectral()).value
 
-    def converter_fla_lafla(x):
+    def converter_f_la_to_la_f_la(x):
         return x * wav.to(si.AA, spectral()).value
 
-    def iconverter_fla_lafla(x):
+    def iconverter_f_la_to_la_f_la(x):
         return x / wav.to(si.AA, spectral()).value
 
-    def converter_photlam_fla(x):
+    def converter_phot_f_la_to_f_la(x):
         return hc * x / wav.to(si.AA, spectral()).value
 
-    def iconverter_photlam_fla(x):
+    def iconverter_phot_f_la_to_f_la(x):
         return x * wav.to(si.AA, spectral()).value / hc
 
-    def converter_photlam_fnu(x):
+    def converter_phot_f_la_to_f_nu(x):
         return h_cgs * x * wav.to(si.AA, spectral()).value
 
-    def iconverter_photlam_fnu(x):
+    def iconverter_phot_f_la_to_f_nu(x):
         return x / (wav.to(si.AA, spectral()).value * h_cgs)
 
-    def converter_photlam_photnu(x):
+    def converter_phot_f_la_phot_f_nu(x):
         return x * wav.to(si.AA, spectral()).value ** 2 / c_Aps
 
-    def iconverter_photlam_photnu(x):
+    def iconverter_phot_f_la_phot_f_nu(x):
         return c_Aps * x / wav.to(si.AA, spectral()).value ** 2
 
-    converter_photnu_fnu = converter_photlam_fla
+    converter_phot_f_nu_to_f_nu = converter_phot_f_la_to_f_la
+    iconverter_phot_f_nu_to_f_nu = iconverter_phot_f_la_to_f_la
 
-    iconverter_photnu_fnu = iconverter_photlam_fla
-
-    def converter_photnu_fla(x):
+    def converter_phot_f_nu_to_f_la(x):
         return x * hc * c_Aps / wav.to(si.AA, spectral()).value ** 3
 
-    def iconverter_photnu_fla(x):
+    def iconverter_phot_f_nu_to_f_la(x):
         return x * wav.to(si.AA, spectral()).value ** 3 / (hc * c_Aps)
 
+    # for luminosity density
+    converter_L_nu_to_nu_L_nu = converter_f_nu_to_nu_f_nu
+    iconverter_L_nu_to_nu_L_nu = iconverter_f_nu_to_nu_f_nu
+    converter_L_la_to_la_L_la = converter_f_la_to_la_f_la
+    iconverter_L_la_to_la_L_la = iconverter_f_la_to_la_f_la
+
+    converter_phot_L_la_to_L_la = converter_phot_f_la_to_f_la
+    iconverter_phot_L_la_to_L_la = iconverter_phot_f_la_to_f_la
+    converter_phot_L_la_to_L_nu = converter_phot_f_la_to_f_nu
+    iconverter_phot_L_la_to_L_nu = iconverter_phot_f_la_to_f_nu
+    converter_phot_L_la_phot_L_nu = converter_phot_f_la_phot_f_nu
+    iconverter_phot_L_la_phot_L_nu = iconverter_phot_f_la_phot_f_nu
+    converter_phot_L_nu_to_L_nu = converter_phot_f_nu_to_f_nu
+    iconverter_phot_L_nu_to_L_nu = iconverter_phot_f_nu_to_f_nu
+    converter_phot_L_nu_to_L_la = converter_phot_f_nu_to_f_la
+    iconverter_phot_L_nu_to_L_la = iconverter_phot_f_nu_to_f_la
+
     return [
-        (fla, fnu, converter, iconverter),
-        (fnu, nufnu, converter_fnu_nufnu, iconverter_fnu_nufnu),
-        (fla, lafla, converter_fla_lafla, iconverter_fla_lafla),
-        (photlam, fla, converter_photlam_fla, iconverter_photlam_fla),
-        (photlam, fnu, converter_photlam_fnu, iconverter_photlam_fnu),
-        (photlam, photnu, converter_photlam_photnu, iconverter_photlam_photnu),
-        (photnu, fnu, converter_photnu_fnu, iconverter_photnu_fnu),
-        (photnu, fla, converter_photnu_fla, iconverter_photnu_fla)
+        # flux
+        (f_la, f_nu, converter, iconverter),
+        (f_nu, nu_f_nu, converter_f_nu_to_nu_f_nu, iconverter_f_nu_to_nu_f_nu),
+        (f_la, la_f_la, converter_f_la_to_la_f_la, iconverter_f_la_to_la_f_la),
+        (phot_f_la, f_la, converter_phot_f_la_to_f_la, iconverter_phot_f_la_to_f_la),
+        (phot_f_la, f_nu, converter_phot_f_la_to_f_nu, iconverter_phot_f_la_to_f_nu),
+        (phot_f_la, phot_f_nu, converter_phot_f_la_phot_f_nu, iconverter_phot_f_la_phot_f_nu),
+        (phot_f_nu, f_nu, converter_phot_f_nu_to_f_nu, iconverter_phot_f_nu_to_f_nu),
+        (phot_f_nu, f_la, converter_phot_f_nu_to_f_la, iconverter_phot_f_nu_to_f_la),
+        # luminosity
+        (L_la, L_nu, converter, iconverter),
+        (L_nu, nu_L_nu, converter_L_nu_to_nu_L_nu, iconverter_L_nu_to_nu_L_nu),
+        (L_la, la_L_la, converter_L_la_to_la_L_la, iconverter_L_la_to_la_L_la),
+        (phot_L_la, L_la, converter_phot_L_la_to_L_la, iconverter_phot_L_la_to_L_la),
+        (phot_L_la, L_nu, converter_phot_L_la_to_L_nu, iconverter_phot_L_la_to_L_nu),
+        (phot_L_la, phot_L_nu, converter_phot_L_la_phot_L_nu, iconverter_phot_L_la_phot_L_nu),
+        (phot_L_nu, L_nu, converter_phot_L_nu_to_L_nu, iconverter_phot_L_nu_to_L_nu),
+        (phot_L_nu, L_la, converter_phot_L_nu_to_L_la, iconverter_phot_L_nu_to_L_la),
     ]
 
 
@@ -450,11 +486,11 @@ def brightness_temperature(beam_area, disp):
         >>> bmaj = 15*u.arcsec
         >>> bmin = 15*u.arcsec
         >>> fwhm_to_sigma = 1./(8*np.log(2))**0.5
-        >>> beam_area = 2.*np.pi*(bmaj*bmin/fwhm_to_sigma**2)
+        >>> beam_area = 2.*np.pi*(bmaj*bmin*fwhm_to_sigma**2)
         >>> freq = 5*u.GHz
         >>> equiv = u.brightness_temperature(beam_area, freq)
         >>> u.Jy.to(u.K, equivalencies=equiv)  # doctest: +FLOAT_CMP
-        7.065788175060084
+        217.2658703625732
     """
     beam = beam_area.to(si.sr).value
     nu = disp.to(si.GHz, spectral())
@@ -481,15 +517,59 @@ def temperature():
         (si.K, deg_F, lambda x: (x - 273.15) * 1.8 + 32.0,
          lambda x: ((x - 32.0) / 1.8) + 273.15)]
 
+
 def temperature_energy():
     """Convert between Kelvin and keV(eV) to an equivalent amount."""
     return [
         (si.K, si.eV, lambda x: x / (_si.e.value / _si.k_B),
          lambda x: x * (_si.e.value / _si.k_B))]
 
+
 def assert_is_spectral_unit(value):
     try:
         value.to(si.Hz, spectral())
     except (AttributeError, UnitsError) as ex:
         raise UnitsError("The 'rest' value must be a spectral equivalent "
                          "(frequency, wavelength, or energy).")
+
+
+def pixel_scale(pixscale):
+    """
+    Convert between pixel distances (in units of ``pix``) and angular units,
+    given a particular ``pixscale``.
+
+    Parameters
+    ----------
+    pixscale : `~astropy.units.Quantity`
+        The pixel scale either in units of angle/pixel or pixel/angle.
+    """
+    if pixscale.unit.is_equivalent(si.arcsec/astrophys.pix):
+        pixscale_val = pixscale.to(si.radian/astrophys.pix).value
+    elif pixscale.unit.is_equivalent(astrophys.pix/si.arcsec):
+        pixscale_val = (1/pixscale).to(si.radian/astrophys.pix).value
+    else:
+        raise UnitsError("The pixel scale must be in angle/pixel or "
+                         "pixel/angle")
+
+    return [(astrophys.pix, si.radian, lambda px: px*pixscale_val, lambda rad: rad/pixscale_val)]
+
+
+def plate_scale(platescale):
+    """
+    Convert between lengths (to be interpreted as lengths in the focal plane)
+    and angular units with a specified ``platescale``.
+
+    Parameters
+    ----------
+    platescale : `~astropy.units.Quantity`
+        The pixel scale either in units of distance/pixel or distance/angle.
+    """
+    if platescale.unit.is_equivalent(si.arcsec/si.m):
+        platescale_val = platescale.to(si.radian/si.m).value
+    elif platescale.unit.is_equivalent(si.m/si.arcsec):
+        platescale_val = (1/platescale).to(si.radian/si.m).value
+    else:
+        raise UnitsError("The pixel scale must be in angle/distance or "
+                         "distance/angle")
+
+    return [(si.m, si.radian, lambda d: d*platescale_val, lambda rad: rad/platescale_val)]
