@@ -8,8 +8,7 @@ Handles the "Console" unit format.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from . import base
-from . import utils
+from . import base, core, utils
 
 
 class Console(base.Base):
@@ -25,8 +24,6 @@ class Console(base.Base):
       2.1798721*10^-18 ------
                         s^2
     """
-    def __init__(self):
-        pass
 
     _times = "*"
     _line = "-"
@@ -66,39 +63,38 @@ class Console(base.Base):
 
         return cls._times.join(parts)
 
-    def to_string(self, unit):
-        from .. import core
-
+    @classmethod
+    def to_string(cls, unit):
         if isinstance(unit, core.CompositeUnit):
             if unit.scale == 1:
                 s = ''
             else:
-                s = self.format_exponential_notation(unit.scale)
+                s = cls.format_exponential_notation(unit.scale)
 
             if len(unit.bases):
                 positives, negatives = utils.get_grouped_by_powers(
                     unit.bases, unit.powers)
                 if len(negatives):
                     if len(positives):
-                        positives = self._format_unit_list(positives)
+                        positives = cls._format_unit_list(positives)
                     else:
                         positives = '1'
-                    negatives = self._format_unit_list(negatives)
+                    negatives = cls._format_unit_list(negatives)
                     l = len(s)
                     r = max(len(positives), len(negatives))
                     f = "{{0:^{0}s}} {{1:^{1}s}}".format(l, r)
 
                     lines = [
                         f.format('', positives),
-                        f.format(s, self._line * r),
+                        f.format(s, cls._line * r),
                         f.format('', negatives)
                     ]
 
                     s = '\n'.join(lines)
                 else:
-                    positives = self._format_unit_list(positives)
+                    positives = cls._format_unit_list(positives)
                     s += positives
         elif isinstance(unit, core.NamedUnit):
-            s = self._get_unit_name(unit)
+            s = cls._get_unit_name(unit)
 
         return s
