@@ -70,9 +70,9 @@ class LogUnit(FunctionUnitBase):
         # u.dex, and u.dB are OK, i.e., other does not have to be LogUnit
         # (this will indirectly test whether other is a unit at all).
         try:
-            self._function_unit._to(getattr(other, 'function_unit', other))
+            getattr(other, 'function_unit', other)._to(self._function_unit)
         except AttributeError:
-            # if other is not a unit (_to cannot decompose).
+            # if other is not a unit (i.e., does not have _to).
             return NotImplemented
         except UnitsError:
             raise UnitsError("Can only add/subtract logarithmic units of"
@@ -245,7 +245,7 @@ class LogQuantity(FunctionQuantity):
         # Do calculation in-place using _function_view of array.
         function_view = self._function_view
         function_view += getattr(other, '_function_view', other)
-        self._full_unit = new_unit
+        self._set_unit(new_unit)
         return self
 
     def __sub__(self, other):
@@ -270,7 +270,7 @@ class LogQuantity(FunctionQuantity):
         # Do calculation in-place using _function_view of array.
         function_view = self._function_view
         function_view -= getattr(other, '_function_view', other)
-        self._full_unit = new_unit
+        self._set_unit(new_unit)
         return self
 
     # Could add __mul__ and __div__ and try interpreting other as a power,
