@@ -10,15 +10,18 @@ from __future__ import (absolute_import, unicode_literals, division,
                         print_function)
 
 # Standard library
+import re
 import numbers
 from fractions import Fraction
+import warnings
 
 import numpy as np
 
 # AstroPy
 from ..extern import six
-from .core import (Unit, dimensionless_unscaled, UnitBase, UnitsError,
-                   get_current_unit_registry, UnitConversionError)
+from ..extern.six.moves import zip
+from .core import (Unit, dimensionless_unscaled, get_current_unit_registry,
+                   UnitBase, UnitsError, UnitConversionError, UnitTypeError)
 from .format.latex import Latex
 from ..utils.compat import NUMPY_LT_1_8, NUMPY_LT_1_9
 from ..utils.compat.misc import override__dir__
@@ -27,7 +30,7 @@ from ..utils.data_info import ParentDtypeInfo
 from .. import config as _config
 
 
-__all__ = ["Quantity"]
+__all__ = ["Quantity", "SpecificTypeQuantity"]
 
 
 # We don't want to run doctests in the docstrings we inherit from Numpy
@@ -35,7 +38,7 @@ __doctest_skip__ = ['Quantity.*']
 
 
 _UNIT_NOT_INITIALISED = "(Unit not initialised)"
-
+_UFUNCS_FILTER_WARNINGS = {np.arcsin, np.arccos, np.arccosh, np.arctanh}
 
 class Conf(_config.ConfigNamespace):
     """
@@ -127,23 +130,45 @@ class QuantityInfo(ParentDtypeInfo):
     """
     attrs_from_parent = set(['dtype', 'unit'])  # dtype and unit taken from parent
     _supports_indexing = True
+    _represent_as_dict_attrs = ('value', 'unit')
 
     @staticmethod
     def default_format(val):
         return '{0.value:}'.format(val)
 
+    @staticmethod
+    def possible_string_format_functions(format_):
+        """Iterate through possible string-derived format functions.
+
+        A string can either be a format specifier for the format built-in,
+        a new-style format string, or an old-style format string.
+
+        This method is overridden in order to suppress printing the unit
+        in each row since it is already at the top in the column header.
+        """
+        yield lambda format_, val: format(val.value, format_)
+        yield lambda format_, val: format_.format(val.value)
+        yield lambda format_, val: format_ % val.value
+
+    def _construct_from_dict(self, map):
+        # Need to pop value because different Quantity subclasses use
+        # different first arg name for the value.  :-(
+        value = map.pop('value')
+        return self._parent_cls(value, **map)
+
 
 @six.add_metaclass(InheritDocstrings)
 class Quantity(np.ndarray):
-    """ A `~astropy.units.Quantity` represents a number with some associated unit.
+    """A `~astropy.units.Quantity` represents a number with some associated unit.
 
     Parameters
     ----------
-    value : number, `~numpy.ndarray`, `Quantity` object, or sequence of `Quantity` objects.
+    value : number, `~numpy.ndarray`, `Quantity` object (sequence), str
         The numerical value of this quantity in the units given by unit.  If a
         `Quantity` or sequence of them (or any other valid object with a
         ``unit`` attribute), creates a new `Quantity` object, converting to
-        `unit` units as needed.
+        `unit` units as needed.  If a string, it is converted to a number or
+        `Quantity`, depending on whether a unit is present.
 
     unit : `~astropy.units.UnitBase` instance, str
         An object that represents the unit associated with the input value.
@@ -185,11 +210,24 @@ class Quantity(np.ndarray):
     TypeError
         If the unit provided is not either a :class:`~astropy.units.Unit`
         object or a parseable string unit.
+
+    Notes
+    -----
+    Quantities can also be created by multiplying a number or array with a
+    :class:`~astropy.units.Unit`. See http://docs.astropy.org/en/latest/units/
+
     """
     # Need to set a class-level default for _equivalencies, or
     # Constants can not initialize properly
     _equivalencies = []
 
+    # Default unit for initialization; can be overridden by subclasses,
+    # possibly to `None` to indicate there is no default unit.
+    _default_unit = dimensionless_unscaled
+
+    # Ensures views have an undefined unit.
+    _unit = None
+
     __array_priority__ = 10000
 
     def __new__(cls, value, unit=None, dtype=None, copy=True, order=None,
@@ -220,24 +258,45 @@ class Quantity(np.ndarray):
             return np.array(value, dtype=dtype, copy=copy, order=order,
                             subok=True, ndmin=ndmin)
 
-        # Maybe list/tuple of Quantity? short-circuit array for speed
-        if (not isinstance(value, np.ndarray) and isiterable(value) and
-            all(isinstance(v, Quantity) for v in value)):
-            # Convert all quantities to the same unit.
-            if unit is None:
-                unit = value[0].unit
-            value = [q.to(unit).value for q in value]
-            value_unit = unit  # signal below that conversion has been done
-            copy = False  # copy already made
+        # Maybe str, or list/tuple of Quantity? If so, this may set value_unit.
+        # To ensure array remains fast, we short-circuit it.
+        value_unit = None
+        if not isinstance(value, np.ndarray):
+            if isinstance(value, six.string_types):
+                # The first part of the regex string matches any integer/float;
+                # the second parts adds possible trailing .+-, which will break
+                # the float function below and ensure things like 1.2.3deg
+                # will not work.
+                v = re.match(r'\s*[+-]?((\d+\.?\d*)|(\.\d+))([eE][+-]?\d+)?'
+                             r'[.+-]?', value)
+                try:
+                    value = float(v.group())
+                except Exception:
+                    raise TypeError('Cannot parse "{0}" as a {1}. It does not '
+                                    'start with a number.'
+                                    .format(value, cls.__name__))
+                unit_string = v.string[v.end():].strip()
+                if unit_string:
+                    value_unit = Unit(unit_string)
+                    if unit is None:
+                        unit = value_unit  # signal no conversion needed below.
 
-        else:
+            elif (isiterable(value) and len(value) > 0 and
+                  all(isinstance(v, Quantity) for v in value)):
+                # Convert all quantities to the same unit.
+                if unit is None:
+                    unit = value[0].unit
+                value = [q.to(unit).value for q in value]
+                value_unit = unit  # signal below that conversion has been done
+
+        if value_unit is None:
             # If the value has a `unit` attribute and if not None
             # (for Columns with uninitialized unit), treat it like a quantity.
             value_unit = getattr(value, 'unit', None)
             if value_unit is None:
                 # Default to dimensionless for no (initialized) unit attribute.
                 if unit is None:
-                    unit = dimensionless_unscaled
+                    unit = cls._default_unit
                 value_unit = unit  # signal below that no conversion is needed
             else:
                 try:
@@ -270,7 +329,7 @@ class Quantity(np.ndarray):
             value = value.astype(np.float)
 
         value = value.view(cls)
-        value._unit = value_unit
+        value._set_unit(value_unit)
         if unit is value_unit:
             return value
         else:
@@ -279,7 +338,11 @@ class Quantity(np.ndarray):
             return value.to(unit)
 
     def __array_finalize__(self, obj):
-        self._unit = getattr(obj, '_unit', None)
+        # If our unit is not set and obj has a valid one, use it.
+        if self._unit is None:
+            unit = getattr(obj, '_unit', None)
+            if unit is not None:
+                self._set_unit(unit)
 
         # Copy info if the original had `info` defined.  Because of the way the
         # DataInfo works, `'info' in obj.__dict__` is False until the
@@ -352,29 +415,31 @@ class Quantity(np.ndarray):
                                         args[0].__class__.__name__,
                                         args[1].__class__.__name__))
 
-        # In the case of np.power, the unit itself needs to be modified by an
-        # amount that depends on one of the input values, so we need to treat
-        # this as a special case.
-        # TODO: find a better way to deal with this case
-        if function is np.power and result_unit != dimensionless_unscaled:
-
-            if units[1] is None:
-                p = args[1]
+        # In the case of np.power and np.float_power, the unit itself needs to
+        # be modified by an amount that depends on one of the input values,
+        # so we need to treat this as a special case.
+        # TODO: find a better way to deal with this.
+        if result_unit is False:
+            if units[0] is None or units[0] == dimensionless_unscaled:
+                result_unit = dimensionless_unscaled
             else:
-                p = args[1].to(dimensionless_unscaled).value
-
-            try:
-                result_unit = result_unit ** p
-            except ValueError as exc:
-                # Changing the unit does not work for, e.g., array-shaped
-                # power, but this is OK if we're (scaled) dimensionless.
-                try:
-                    converters[0] = units[0]._get_converter(
-                        dimensionless_unscaled)
-                except UnitConversionError:
-                    raise exc
+                if units[1] is None:
+                    p = args[1]
                 else:
-                    result_unit = dimensionless_unscaled
+                    p = args[1].to(dimensionless_unscaled).value
+
+                try:
+                    result_unit = units[0] ** p
+                except ValueError as exc:
+                    # Changing the unit does not work for, e.g., array-shaped
+                    # power, but this is OK if we're (scaled) dimensionless.
+                    try:
+                        converters[0] = units[0]._get_converter(
+                            dimensionless_unscaled)
+                    except UnitConversionError:
+                        raise exc
+                    else:
+                        result_unit = dimensionless_unscaled
 
         # We now prepare the output object
         if self is obj:
@@ -390,7 +455,7 @@ class Quantity(np.ndarray):
                                 .format(function.__name__, type(self)))
 
             if self.__quantity_subclass__(result_unit)[0] is not type(self):
-                raise TypeError(
+                raise UnitTypeError(
                     "Cannot store output with unit '{0}' from {1} function "
                     "in {2} instance.  Use {3} instance instead."
                     .format(result_unit, function.__name__, type(self),
@@ -450,12 +515,23 @@ class Quantity(np.ndarray):
             # ensure we remember the scales we need
             result._converters = converters
 
+            if function in _UFUNCS_FILTER_WARNINGS:
+                # Filter out RuntimeWarning's caused by the ufunc being called on
+                # the unscaled quantity first (e.g., np.arcsin(15*u.pc/u.kpc))
+                self._catch_warnings = warnings.catch_warnings()
+                self._catch_warnings.__enter__()
+                warnings.filterwarnings('ignore',
+                                        message='invalid value encountered in',
+                                        category=RuntimeWarning)
+
         # unit output will get (setting _unit could prematurely change input
         # if obj is self, which happens for in-place operations; see above)
         result._result_unit = result_unit
+
         return result
 
     def __array_wrap__(self, obj, context=None):
+
         if context is None:
             # Methods like .squeeze() created a new `ndarray` and then call
             # __array_wrap__ to turn the array into self's subclass.
@@ -476,6 +552,10 @@ class Quantity(np.ndarray):
                 converters = obj._converters
                 del obj._converters
 
+                if hasattr(self, '_catch_warnings'):
+                    self._catch_warnings.__exit__()
+                    del self._catch_warnings
+
                 # For in-place operations, input will get overwritten with
                 # junk. To avoid that, we hid it in a new object in
                 # __array_prepare__ and retrieve it here.
@@ -535,11 +615,12 @@ class Quantity(np.ndarray):
                     obj = self._new_view(out, result_unit)
 
             if result_unit is None:  # return a plain array
-                obj = obj.view(np.ndarray)
+                return obj.view(np.ndarray)
+            elif obj is self:  # all OK now, so set unit.
+                obj._set_unit(result_unit)
+                return obj
             else:
-                obj._unit = result_unit
-
-        return obj
+                return obj
 
     def __deepcopy__(self, memo):
         # If we don't define this, ``copy.deepcopy(quantity)`` will
@@ -564,52 +645,84 @@ class Quantity(np.ndarray):
         """
         return Quantity, True
 
-    def _new_view(self, obj, unit=None):
+    def _new_view(self, obj=None, unit=None):
         """
-        Create a Quantity view of obj, and set the unit
+        Create a Quantity view of some array-like input, and set the unit
 
-        By default, return a view of ``obj`` of the same class as ``self``
-        and with the unit passed on, or that of ``self``.  Subclasses can
-        override the type of class used with ``__quantity_subclass__``, and
-        can ensure other properties of ``self`` are copied using
-        `__array_finalize__`.
+        By default, return a view of ``obj`` of the same class as ``self`` and
+        with the same unit.  Subclasses can override the type of class for a
+        given unit using ``__quantity_subclass__``, and can ensure properties
+        other than the unit are copied using ``__array_finalize__``.
+
+        If the given unit defines a ``_quantity_class`` of which ``self``
+        is not an instance, a view using this class is taken.
 
         Parameters
         ----------
-        obj : ndarray or scalar
+        obj : ndarray or scalar, optional
             The array to create a view of.  If obj is a numpy or python scalar,
-            it will be converted to an array scalar.
+            it will be converted to an array scalar.  By default, ``self``
+            is converted.
 
-        unit : `UnitBase`, or anything convertible to a :class:`~astropy.units.Unit`, or `None`
+        unit : `UnitBase`, or anything convertible to a :class:`~astropy.units.Unit`, optional
             The unit of the resulting object.  It is used to select a
-            subclass, and explicitly assigned to the view if not `None`.
-            If `None` (default), the unit is set by `__array_finalize__`
-            to self._unit.
+            subclass, and explicitly assigned to the view if given.
+            If not given, the subclass and unit will be that of ``self``.
 
         Returns
         -------
         view : Quantity subclass
         """
-        # python and numpy scalars cannot be viewed as arrays and thus not as
-        # Quantity either; turn them into zero-dimensional arrays
-        # (These are turned back into scalar in `.value`)
-        if not isinstance(obj, np.ndarray):
-            obj = np.array(obj)
-
+        # Determine the unit and quantity subclass that we need for the view.
         if unit is None:
-            subclass = self.__class__
+            unit = self.unit
+            quantity_subclass = self.__class__
         else:
             unit = Unit(unit)
-            subclass, subok = self.__quantity_subclass__(unit)
-            if subok:
-                subclass = self.__class__
+            quantity_subclass = getattr(unit, '_quantity_class', Quantity)
+            if isinstance(self, quantity_subclass):
+                quantity_subclass, subok = self.__quantity_subclass__(unit)
+                if subok:
+                    quantity_subclass = self.__class__
 
-        view = obj.view(subclass)
+        # We only want to propagate information from ``self`` to our new view,
+        # so obj should be a regular array.  By using ``np.array``, we also
+        # convert python and numpy scalars, which cannot be viewed as arrays
+        # and thus not as Quantity either, to zero-dimensional arrays.
+        # (These are turned back into scalar in `.value`)
+        if obj is None:
+            obj = self.view(np.ndarray)
+        else:
+            obj = np.array(obj, copy=False)
+
+        # Take the view, set the unit, and update possible other properties
+        # such as ``info``, ``wrap_angle`` in `Longitude`, etc.
+        view = obj.view(quantity_subclass)
+        view._set_unit(unit)
         view.__array_finalize__(self)
-        if unit is not None:
-            view._unit = unit
         return view
 
+    def _set_unit(self, unit):
+        """Set the unit.
+
+        This is used anywhere the unit is set or modified, i.e., in the
+        initilizer, in ``__imul__`` and ``__itruediv__`` for in-place
+        multiplication and division by another unit, as well as in
+        ``__array_finalize__`` for wrapping up views.  For Quantity, it just
+        sets the unit, but subclasses can override it to check that, e.g.,
+        a unit is consistent.
+        """
+        if not isinstance(unit, UnitBase):
+            # Trying to go through a string ensures that, e.g., Magnitudes with
+            # dimensionless physical unit become Quantity with units of mag.
+            unit = Unit(str(unit), parse_strict='silent')
+            if not isinstance(unit, UnitBase):
+                raise UnitTypeError(
+                    "{0} instances require {1} units, not {2} instances."
+                    .format(type(self).__name__, UnitBase, type(unit)))
+
+        self._unit = unit
+
     def __reduce__(self):
         # patch to pickle Quantity objects (ndarray subclasses), see
         # http://www.mail-archive.com/numpy-discussion@scipy.org/msg02446.html
@@ -673,9 +786,6 @@ class Quantity(np.ndarray):
 
         return self._unit
 
-    # this ensures that if we do a view, __repr__ and __str__ do not balk
-    _unit = None
-
     @property
     def equivalencies(self):
         """
@@ -817,7 +927,7 @@ class Quantity(np.ndarray):
         """In-place multiplication between `Quantity` objects and others."""
 
         if isinstance(other, (UnitBase, six.string_types)):
-            self._unit = other * self.unit
+            self._set_unit(other * self.unit)
             return self
 
         return super(Quantity, self).__imul__(other)
@@ -844,7 +954,7 @@ class Quantity(np.ndarray):
         """Inplace division between `Quantity` objects and other objects."""
 
         if isinstance(other, (UnitBase, six.string_types)):
-            self._unit = self.unit / other
+            self._set_unit(self.unit / other)
             return self
 
         return super(Quantity, self).__itruediv__(other)
@@ -945,7 +1055,7 @@ class Quantity(np.ndarray):
         potential for ambiguity otherwise.
         """
         return True
-    if six.PY3:
+    if not six.PY2:
         __bool__ = __nonzero__
 
     def __len__(self):
@@ -976,7 +1086,7 @@ class Quantity(np.ndarray):
         try:
             assert self.unit.is_unity()
             return self.value.__index__()
-        except:
+        except Exception:
             raise TypeError('Only integer dimensionless scalar quantities '
                             'can be converted to a Python index')
 
@@ -1288,19 +1398,18 @@ class Quantity(np.ndarray):
                 out.__quantity_subclass__(unit)[0] is type(out)):
                 # Set out to ndarray view to prevent calling __array_prepare__.
                 kwargs['out'] = out.view(np.ndarray)
-
             else:
                 ok_class =  (out.__quantity_subclass__(out, unit)[0]
                              if isinstance(out, Quantity) else Quantity)
-                raise TypeError("out cannot be assigned to a {0} instance; "
-                                "use a {1} instance instead.".format(
-                                    out.__class__, ok_class))
+                raise UnitTypeError("out cannot be assigned to a {0} instance; "
+                                    "use a {1} instance instead.".format(
+                                        out.__class__, ok_class))
 
         value = function(self.view(np.ndarray), *args, **kwargs)
         if out is None:
             return self._new_view(value, unit)
         else:
-            out._unit = unit
+            out._set_unit(unit)
             return out
 
     def clip(self, a_min, a_max, out=None):
@@ -1430,3 +1539,47 @@ class Quantity(np.ndarray):
         """
         out_array = np.insert(self.value, obj, self._to_own_unit(values), axis)
         return self._new_view(out_array)
+
+
+class SpecificTypeQuantity(Quantity):
+    """Superclass for Quantities of specific physical type.
+
+    Subclasses of these work just like :class:`~astropy.units.Quantity`, except
+    that they are for specific physical types (and may have methods that are
+    only appropriate for that type).  Astropy examples are
+    :class:`~astropy.coordinates.Angle` and
+    :class:`~astropy.coordinates.Distance`
+
+    At a minimum, subclasses should set ``_equivalent_unit`` to the unit
+    associated with the physical type.
+    """
+    # The unit for the specific physical type.  Instances can only be created
+    # with units that are equivalent to this.
+    _equivalent_unit = None
+
+    # The default unit used for views.  Even with `None`, views of arrays
+    # without units are possible, but will have an uninitalized unit.
+    _unit = None
+
+    # Default unit for initialization through the constructor.
+    _default_unit = None
+
+    # ensure that we get precedence over our superclass.
+    __array_priority__ = Quantity.__array_priority__ + 10
+
+    def __quantity_subclass__(self, unit):
+        if unit.is_equivalent(self._equivalent_unit):
+            return type(self), True
+        else:
+            return super(SpecificTypeQuantity,
+                         self).__quantity_subclass__(unit)[0], False
+
+    def _set_unit(self, unit):
+        if unit is None or not unit.is_equivalent(self._equivalent_unit):
+            raise UnitTypeError(
+                "{0} instances require units equivalent to '{1}'"
+                .format(type(self).__name__, self._equivalent_unit) +
+                (", but no unit was given." if unit is None else
+                 ", so cannot set it to '{0}'.".format(unit)))
+
+        super(SpecificTypeQuantity, self)._set_unit(unit)
