@@ -14,7 +14,7 @@ from numpy.testing.utils import assert_allclose
 # LOCAL
 from ... import units as u
 from ... import constants
-from ...tests.helper import pytest
+from ...tests.helper import pytest, assert_quantity_allclose
 
 
 def test_dimensionless_angles():
@@ -291,12 +291,23 @@ def test_spectral4(in_val, in_unit):
 
 
 def test_spectraldensity2():
+    # flux density
     flambda = u.erg / u.angstrom / u.cm ** 2 / u.s
     fnu = u.erg / u.Hz / u.cm ** 2 / u.s
 
     a = flambda.to(fnu, 1, u.spectral_density(u.Quantity(3500, u.AA)))
     assert_allclose(a, 4.086160166177361e-12)
 
+    # luminosity density
+    llambda = u.erg / u.angstrom / u.s
+    lnu = u.erg / u.Hz / u.s
+
+    a = llambda.to(lnu, 1, u.spectral_density(u.Quantity(3500, u.AA)))
+    assert_allclose(a, 4.086160166177361e-12)
+
+    a = lnu.to(llambda, 1, u.spectral_density(u.Quantity(3500, u.AA)))
+    assert_allclose(a, 2.44728537142857e11)
+
 
 def test_spectraldensity3():
     # Define F_nu in Jy
@@ -345,6 +356,8 @@ def test_spectraldensity4():
     flux_flam = [3.9135e-14, 4.0209e-14, 3.9169e-14]
     flux_fnu = [3.20735792e-25, 3.29903646e-25, 3.21727226e-25]
     flux_jy = [3.20735792e-2, 3.29903646e-2, 3.21727226e-2]
+    flux_stmag = [12.41858665, 12.38919182, 12.41764379]
+    flux_abmag = [12.63463143, 12.60403221, 12.63128047]
 
     # PHOTLAM <--> FLAM
     assert_allclose(photlam.to(
@@ -382,6 +395,61 @@ def test_spectraldensity4():
     assert_allclose(flam.to(
         photnu, flux_flam, u.spectral_density(wave)), flux_photnu, rtol=1e-6)
 
+    # PHOTLAM <--> STMAG
+    assert_allclose(photlam.to(
+        u.STmag, flux_photlam, u.spectral_density(wave)), flux_stmag, rtol=1e-6)
+    assert_allclose(u.STmag.to(
+        photlam, flux_stmag, u.spectral_density(wave)), flux_photlam, rtol=1e-6)
+
+    # PHOTLAM <--> ABMAG
+    assert_allclose(photlam.to(
+        u.ABmag, flux_photlam, u.spectral_density(wave)), flux_abmag, rtol=1e-6)
+    assert_allclose(u.ABmag.to(
+        photlam, flux_abmag, u.spectral_density(wave)), flux_photlam, rtol=1e-6)
+
+def test_spectraldensity5():
+    """ Test photon luminosity density conversions. """
+    L_la = u.erg / (u.s * u.AA)
+    L_nu = u.erg / (u.s * u.Hz)
+    phot_L_la = u.photon / (u.s * u.AA)
+    phot_L_nu = u.photon / (u.s * u.Hz)
+
+    wave = u.Quantity([4956.8, 4959.55, 4962.3], u.AA)
+    flux_phot_L_la = [9.7654e-3, 1.003896e-2, 9.78473e-3]
+    flux_phot_L_nu = [8.00335589e-14, 8.23668949e-14, 8.03700310e-14]
+    flux_L_la = [3.9135e-14, 4.0209e-14, 3.9169e-14]
+    flux_L_nu = [3.20735792e-25, 3.29903646e-25, 3.21727226e-25]
+
+    # PHOTLAM <--> FLAM
+    assert_allclose(phot_L_la.to(
+        L_la, flux_phot_L_la, u.spectral_density(wave)), flux_L_la, rtol=1e-6)
+    assert_allclose(L_la.to(
+        phot_L_la, flux_L_la, u.spectral_density(wave)), flux_phot_L_la, rtol=1e-6)
+
+    # PHOTLAM <--> FNU
+    assert_allclose(phot_L_la.to(
+        L_nu, flux_phot_L_la, u.spectral_density(wave)), flux_L_nu, rtol=1e-6)
+    assert_allclose(L_nu.to(
+        phot_L_la, flux_L_nu, u.spectral_density(wave)), flux_phot_L_la, rtol=1e-6)
+
+    # PHOTLAM <--> PHOTNU
+    assert_allclose(phot_L_la.to(
+        phot_L_nu, flux_phot_L_la, u.spectral_density(wave)), flux_phot_L_nu, rtol=1e-6)
+    assert_allclose(phot_L_nu.to(
+        phot_L_la, flux_phot_L_nu, u.spectral_density(wave)), flux_phot_L_la, rtol=1e-6)
+
+    # PHOTNU <--> FNU
+    assert_allclose(phot_L_nu.to(
+        L_nu, flux_phot_L_nu, u.spectral_density(wave)), flux_L_nu, rtol=1e-6)
+    assert_allclose(L_nu.to(
+        phot_L_nu, flux_L_nu, u.spectral_density(wave)), flux_phot_L_nu, rtol=1e-6)
+
+    # PHOTNU <--> FLAM
+    assert_allclose(phot_L_nu.to(
+        L_la, flux_phot_L_nu, u.spectral_density(wave)), flux_L_la, rtol=1e-6)
+    assert_allclose(L_la.to(
+        phot_L_nu, flux_L_la, u.spectral_density(wave)), flux_phot_L_nu, rtol=1e-6)
+
 
 def test_equivalent_units():
     from .. import imperial
@@ -390,7 +458,8 @@ def test_equivalent_units():
         units_set = set(units)
         match = set(
             [u.M_e, u.M_p, u.g, u.kg, u.solMass, u.t, u.u, u.M_earth,
-             u.M_jup, imperial.oz, imperial.lb, imperial.st, imperial.ton])
+             u.M_jup, imperial.oz, imperial.lb, imperial.st, imperial.ton,
+             imperial.slug])
         assert units_set == match
 
     r = repr(units)
@@ -401,7 +470,8 @@ def test_equivalent_units2():
     units = set(u.Hz.find_equivalent_units(u.spectral()))
     match = set(
         [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr,
-         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k])
+         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k, u.earthRad,
+         u.jupiterRad])
     assert units == match
 
     from .. import imperial
@@ -412,13 +482,14 @@ def test_equivalent_units2():
              imperial.cal, u.cm, u.eV, u.erg, imperial.ft, imperial.fur,
              imperial.inch, imperial.kcal, u.lyr, u.m, imperial.mi,
              imperial.mil, u.micron, u.pc, u.solRad, imperial.yd, u.Bq, u.Ci,
-             imperial.nmi, u.k])
+             imperial.nmi, u.k, u.earthRad, u.jupiterRad])
         assert units == match
 
     units = set(u.Hz.find_equivalent_units(u.spectral()))
     match = set(
         [u.AU, u.Angstrom, u.Hz, u.J, u.Ry, u.cm, u.eV, u.erg, u.lyr,
-         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k])
+         u.m, u.micron, u.pc, u.solRad, u.Bq, u.Ci, u.k, u.earthRad,
+         u.jupiterRad])
     assert units == match
 
 
@@ -563,3 +634,37 @@ def test_compose_equivalencies():
             break
     else:
         assert False, "Didn't find speed in compose results"
+
+
+def test_pixel_scale():
+    pix = 75*u.pix
+    asec = 30*u.arcsec
+
+    pixscale = 0.4*u.arcsec/u.pix
+    pixscale2 = 2.5*u.pix/u.arcsec
+
+    assert_quantity_allclose(pix.to(u.arcsec, u.pixel_scale(pixscale)), asec)
+    assert_quantity_allclose(pix.to(u.arcmin, u.pixel_scale(pixscale)), asec)
+
+    assert_quantity_allclose(pix.to(u.arcsec, u.pixel_scale(pixscale2)), asec)
+    assert_quantity_allclose(pix.to(u.arcmin, u.pixel_scale(pixscale2)), asec)
+
+    assert_quantity_allclose(asec.to(u.pix, u.pixel_scale(pixscale)), pix)
+    assert_quantity_allclose(asec.to(u.pix, u.pixel_scale(pixscale2)), pix)
+
+
+def test_plate_scale():
+    mm = 1.5*u.mm
+    asec = 30*u.arcsec
+
+    platescale = 20*u.arcsec/u.mm
+    platescale2 = 0.05*u.mm/u.arcsec
+
+    assert_quantity_allclose(mm.to(u.arcsec, u.plate_scale(platescale)), asec)
+    assert_quantity_allclose(mm.to(u.arcmin, u.plate_scale(platescale)), asec)
+
+    assert_quantity_allclose(mm.to(u.arcsec, u.plate_scale(platescale2)), asec)
+    assert_quantity_allclose(mm.to(u.arcmin, u.plate_scale(platescale2)), asec)
+
+    assert_quantity_allclose(asec.to(u.mm, u.plate_scale(platescale)), mm)
+    assert_quantity_allclose(asec.to(u.mm, u.plate_scale(platescale2)), mm)
