@@ -5,7 +5,8 @@ import numpy as np
 
 from ... import units as u
 from ...tests.helper import pytest
-from ...utils.compat import NUMPY_LT_1_7, NUMPY_LT_1_8, NUMPY_LT_1_9_1
+from ...utils.compat import (NUMPY_LT_1_8, NUMPY_LT_1_9_1,
+                             NUMPY_LT_1_10, NUMPY_LT_1_10_4)
 
 
 class TestQuantityArrayCopy(object):
@@ -144,14 +145,8 @@ class TestQuantityStatsFuncs(object):
 
     # For 1.7 <= Numpy < 1.9.1, inplace causes the variance to be stored instead
     # of the standard deviation; https://github.com/numpy/numpy/issues/5240
-    @pytest.mark.xfail("NUMPY_LT_1_9_1")
+    @pytest.mark.xfail(NUMPY_LT_1_9_1, reason="Numpy 1.9.1 or later is required")
     def test_std_inplace(self):
-
-        # For Numpy < 1.7, the test segfaults.  Hence, the xfail decorator does
-        # not suffice: py.test will run the test anyway to see if it works.
-        if NUMPY_LT_1_7:
-            pytest.xfail()
-
         q1 = np.array([1., 2.]) * u.m
         qi = 1.5 * u.s
         np.std(q1, out=qi)
@@ -162,12 +157,6 @@ class TestQuantityStatsFuncs(object):
         assert np.var(q1) == 0.25 * u.m ** 2
 
     def test_var_inplace(self):
-
-        # For Numpy < 1.7, the test segfaults.  Hence, we cannot use the xfail
-        # decorator since py.test will run the test anyway to see if it works.
-        if NUMPY_LT_1_7:
-            pytest.xfail()
-
         q1 = np.array([1., 2.]) * u.m
         qi = 1.5 * u.s
         np.var(q1, out=qi)
@@ -297,7 +286,7 @@ class TestQuantityStatsFuncs(object):
         assert np.all(q2.nansum(0) == np.array([1., 5., 10.]) * u.s)
         assert np.all(np.nansum(q2, 0) == np.array([1., 5., 10.]) * u.s)
 
-    @pytest.mark.xfail("NUMPY_LT_1_8")
+    @pytest.mark.xfail(NUMPY_LT_1_8, reason="Numpy 1.8 or later is required")
     def test_nansum_inplace(self):
 
         q1 = np.array([1., 2., np.nan]) * u.m
@@ -370,7 +359,8 @@ class TestQuantityStatsFuncs(object):
         assert q3.value == np.dot(q1.value, q2.value)
         assert q3.unit == u.m * u.s
 
-    @pytest.mark.xfail
+    @pytest.mark.xfail(NUMPY_LT_1_10_4,
+                       reason="Numpy 1.10.4 or later is required")
     def test_trace_func(self):
 
         q = np.array([[1.,2.],[3.,4.]]) * u.m
