@@ -1,11 +1,14 @@
 # The purpose of these tests are to ensure that calling ufuncs with quantities
 # returns quantities with the right units, or raises exceptions.
 
+import warnings
+
 import numpy as np
 from numpy.testing.utils import assert_allclose
 
 from ... import units as u
 from ...tests.helper import pytest, raises
+from ...extern.six.moves import zip
 
 
 class TestUfuncCoverage(object):
@@ -61,6 +64,14 @@ class TestQuantityTrigonometricFuncs(object):
         assert exc.value.args[0] == ("Can only apply 'arcsin' function to "
                                      "dimensionless quantities")
 
+    def test_arcsin_no_warning_on_unscaled_quantity(self):
+        a = 15 * u.kpc
+        b = 27 * u.pc
+
+        with warnings.catch_warnings():
+            warnings.filterwarnings('error')
+            np.arcsin(b/a)
+
     def test_cos_scalar(self):
         q = np.cos(np.pi / 3. * u.radian)
         assert q.unit == u.dimensionless_unscaled
@@ -288,6 +299,15 @@ class TestQuantityMathFuncs(object):
         assert np.all(np.power(np.arange(4.) * u.m, 0.) ==
                       1. * u.dimensionless_unscaled)
 
+    # float_power only introduced in numpy 1.12
+    @pytest.mark.skipif("not hasattr(np, 'float_power')")
+    def test_float_power_array(self):
+        assert np.all(np.float_power(np.array([1., 2., 3.]) * u.m, 3.)
+                      == np.array([1., 8., 27.]) * u.m ** 3)
+        # regression check on #1696
+        assert np.all(np.float_power(np.arange(4.) * u.m, 0.) ==
+                      1. * u.dimensionless_unscaled)
+
     @raises(ValueError)
     def test_power_array_array(self):
         np.power(4. * u.m, [2., 4.])
@@ -296,6 +316,27 @@ class TestQuantityMathFuncs(object):
     def test_power_array_array2(self):
         np.power([2., 4.] * u.m, [2., 4.])
 
+    def test_power_array_array3(self):
+        # Identical unit fractions are converted automatically to dimensionless
+        # and should be allowed as base for np.power: #4764
+        q = [2., 4.] * u.m / u.m
+        powers = [2., 4.]
+        res = np.power(q, powers)
+        assert np.all(res.value == q.value ** powers)
+        assert res.unit == u.dimensionless_unscaled
+        # The same holds for unit fractions that are scaled dimensionless.
+        q2 = [2., 4.] * u.m / u.cm
+        # Test also against different types of exponent
+        for cls in (list, tuple, np.array, np.ma.array, u.Quantity):
+            res2 = np.power(q2, cls(powers))
+            assert np.all(res2.value == q2.to(1).value ** powers)
+            assert res2.unit == u.dimensionless_unscaled
+        # Though for single powers, we keep the composite unit.
+        res3 = q2 ** 2
+        assert np.all(res3.value == q2.value ** 2)
+        assert res3.unit == q2.unit ** 2
+        assert np.all(res3 == q2 ** [2, 2])
+
     def test_power_invalid(self):
         with pytest.raises(TypeError) as exc:
             np.power(3., 4. * u.m)
