@@ -5,12 +5,12 @@ Back port of Python 3.3's function signature tools from the inspect module,
 modified to be compatible with Python 2.6, 2.7 and 3.2+.
 """
 from __future__ import absolute_import, division, print_function
+
 import itertools
 import functools
 import re
 import types
-
-from .odict import OrderedDict
+from collections import OrderedDict
 
 __all__ = ['BoundArguments', 'Parameter', 'Signature', 'signature']
 
