@@ -71,13 +71,10 @@ else:
         # Just define a dummy class
         class PyreadlineConsole(object): pass
 
-from ..config import ConfigAlias
 from ..extern import six
 from ..extern.six.moves import range
 from .. import conf
-from .. import units as u
 
-from .decorators import deprecated
 from .misc import isiterable
 
 
@@ -86,12 +83,6 @@ __all__ = [
     'ProgressBar', 'Spinner', 'print_code_line', 'ProgressBarOrSpinner',
     'terminal_size']
 
-
-# Only use color by default on Windows if IPython is installed.
-USE_COLOR = ConfigAlias(
-    '0.4', 'USE_COLOR', 'use_color', 'astropy.utils.console', 'astropy')
-
-
 _DEFAULT_ENCODING = 'utf-8'
 
 
@@ -128,7 +119,7 @@ def _get_stdout(stderr=False):
 
 def isatty(file):
     """
-    Returns `True` if `file` is a tty.
+    Returns `True` if ``file`` is a tty.
 
     Most built-in Python file-like objects have an `isatty` member,
     but some user-defined types may not, so this assumes those are not
@@ -173,8 +164,10 @@ def terminal_size(file=None):
             lines -= 6
         if width > 10:
             width -= 1
+        if lines <= 0 or width <= 0:
+            raise Exception('unable to get terminal size')
         return (lines, width)
-    except:
+    except Exception:
         try:
             # see if POSIX standard variables will work
             return (int(os.environ.get('LINES')),
@@ -350,7 +343,7 @@ def color_print(*args, **kwargs):
             # Some file objects support writing unicode sensibly on some Python
             # versions; if this fails try creating a writer using the locale's
             # preferred encoding. If that fails too give up.
-            if not six.PY3 and isinstance(msg, bytes):
+            if six.PY2 and isinstance(msg, bytes):
                 msg = _decode_preferred_encoding(msg)
 
             write = _write_with_fallback(msg, write, file)
@@ -359,7 +352,7 @@ def color_print(*args, **kwargs):
     else:
         for i in range(0, len(args), 2):
             msg = args[i]
-            if not six.PY3 and isinstance(msg, bytes):
+            if six.PY2 and isinstance(msg, bytes):
                 # Support decoding bytes to unicode on Python 2; use the
                 # preferred encoding for the locale (which is *sometimes*
                 # sensible)
@@ -372,7 +365,7 @@ def strip_ansi_codes(s):
     """
     Remove ANSI color codes from the string.
     """
-    return re.sub('\033\[([0-9]+)(;[0-9]+)*m', '', s)
+    return re.sub('\033\\[([0-9]+)(;[0-9]+)*m', '', s)
 
 
 def human_time(seconds):
@@ -447,6 +440,9 @@ def human_file_size(size):
         A human-friendly representation of the size of the file
     """
     if hasattr(size, 'unit'):
+        # Import units only if necessary because the import takes a
+        # significant time [#4649]
+        from .. import units as u
         size = size.to(u.byte).value
 
     suffixes = ' kMGTPEZY'
@@ -499,7 +495,7 @@ class ProgressBar(six.Iterator):
 
         file : writable file-like object, optional
             The file to write the progress bar to.  Defaults to
-            `sys.stdout`.  If `file` is not a tty (as determined by
+            `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any, or special case hacks
             to detect the IPython console), the progress bar will be
             completely silent.
@@ -634,7 +630,7 @@ class ProgressBar(six.Iterator):
         write(' {0:>4s}/{1:>4s}'.format(
             human_file_size(value),
             self._human_total))
-        write(' ({0:>6s}%)'.format('{0:.2f}'.format(frac * 100.0)))
+        write(' ({:>6.2%})'.format(frac))
         write(prefix)
         if t is not None:
             write(human_time(t))
@@ -664,9 +660,9 @@ class ProgressBar(six.Iterator):
             self._widget.value = 0
 
         # Calculate percent completion, and update progress bar
-        percent = (value/self._total) * 100
-        self._widget.value = percent
-        self._widget.description =' ({0:>6s}%)'.format('{0:.2f}'.format(percent))
+        frac = (value/self._total)
+        self._widget.value = frac * 100
+        self._widget.description =' ({:>6.2%})'.format(frac)
 
 
     def _silent_update(self, value=None):
@@ -700,7 +696,7 @@ class ProgressBar(six.Iterator):
 
         file : writeable file-like object, optional
             The file to write the progress bar to.  Defaults to
-            `sys.stdout`.  If `file` is not a tty (as determined by
+            `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any), the scrollbar will
             be completely silent.
 
@@ -766,7 +762,7 @@ class Spinner(object):
 
         file : writeable file-like object, optional
             The file to write the spinner to.  Defaults to
-            `sys.stdout`.  If `file` is not a tty (as determined by
+            `sys.stdout`.  If ``file`` is not a tty (as determined by
             calling its `isatty` member, if any, or special case hacks
             to detect the IPython console), the spinner will be
             completely silent.
@@ -894,7 +890,7 @@ class ProgressBarOrSpinner(object):
 
         file : writable file-like object, optional
             The file to write the to.  Defaults to `sys.stdout`.  If
-            `file` is not a tty (as determined by calling its `isatty`
+            ``file`` is not a tty (as determined by calling its `isatty`
             member, if any), only ``msg`` will be displayed: the
             `ProgressBar` or `Spinner` will be silent.
         """
@@ -1033,12 +1029,12 @@ class Getch(object):
 
 class _GetchUnix(object):
     def __init__(self):
-        import tty
-        import sys
+        import tty  # pylint: disable=W0611
+        import sys  # pylint: disable=W0611
 
         # import termios now or else you'll get the Unix
         # version on the Mac
-        import termios
+        import termios  # pylint: disable=W0611
 
     def __call__(self):
         import sys
@@ -1056,7 +1052,7 @@ class _GetchUnix(object):
 
 class _GetchWindows(object):
     def __init__(self):
-        import msvcrt
+        import msvcrt  # pylint: disable=W0611
 
     def __call__(self):
         import msvcrt
