@@ -7,18 +7,19 @@ from __future__ import print_function
 
 import functools
 import inspect
-import sys
 import textwrap
 import types
 import warnings
 
 from .codegen import make_function_with_signature
-from .exceptions import (AstropyDeprecationWarning,
+from .exceptions import (AstropyDeprecationWarning, AstropyUserWarning,
                          AstropyPendingDeprecationWarning)
 from ..extern import six
+from ..extern.six.moves import zip
 
 
-__all__ = ['deprecated', 'deprecated_attribute', 'classproperty',
+__all__ = ['classproperty', 'deprecated', 'deprecated_attribute',
+           'deprecated_renamed_argument', 'format_doc',
            'lazyproperty', 'sharedmethod', 'wraps']
 
 
@@ -78,9 +79,9 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         if not old_doc:
             old_doc = ''
         old_doc = textwrap.dedent(old_doc).strip('\n')
-        new_doc = (('\n.. deprecated:: %(since)s'
-                    '\n    %(message)s\n\n' %
-                    {'since': since, 'message': message.strip()}) + old_doc)
+        new_doc = (('\n.. deprecated:: {since}'
+                    '\n    {message}\n\n'.format(
+                    **{'since': since, 'message': message.strip()})) + old_doc)
         if not old_doc:
             # This is to prevent a spurious 'unexpected unindent' warning from
             # docutils when the original docstring was blank.
@@ -93,24 +94,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         the function object.
         """
         if isinstance(func, method_types):
-            try:
-                func = func.__func__
-            except AttributeError:
-                # classmethods in Python2.6 and below lack the __func__
-                # attribute so we need to hack around to get it
-                method = func.__get__(None, object)
-                if isinstance(method, types.FunctionType):
-                    # For staticmethods anyways the wrapped object is just a
-                    # plain function (not a bound method or anything like that)
-                    func = method
-                elif hasattr(method, '__func__'):
-                    func = method.__func__
-                elif hasattr(method, 'im_func'):
-                    func = method.im_func
-                else:
-                    # Nothing we can do really...  just return the original
-                    # classmethod, etc.
-                    return func
+            func = func.__func__
         return func
 
     def deprecate_function(func, message):
@@ -140,7 +124,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         # functools.wraps on it, but we normally don't care.
         # This crazy way to get the type of a wrapper descriptor is
         # straight out of the Python 3.3 inspect module docs.
-        if type(func) != type(str.__dict__['__add__']):
+        if type(func) is not type(str.__dict__['__add__']):  # nopep8
             deprecated_func = functools.wraps(func)(deprecated_func)
 
         deprecated_func.__doc__ = deprecate_doc(
@@ -198,21 +182,21 @@ def deprecated(since, message='', name='', alternative='', pending=False,
             name = get_function(obj).__name__
 
         altmessage = ''
-        if not message or type(message) == type(deprecate):
+        if not message or type(message) is type(deprecate):
             if pending:
-                message = ('The %(func)s %(obj_type)s will be deprecated in a '
+                message = ('The {func} {obj_type} will be deprecated in a '
                            'future version.')
             else:
-                message = ('The %(func)s %(obj_type)s is deprecated and may '
+                message = ('The {func} {obj_type} is deprecated and may '
                            'be removed in a future version.')
             if alternative:
-                altmessage = '\n        Use %s instead.' % alternative
+                altmessage = '\n        Use {} instead.'.format(alternative)
 
-        message = ((message % {
+        message = ((message.format(**{
             'func': name,
             'name': name,
             'alternative': alternative,
-            'obj_type': obj_type_name}) +
+            'obj_type': obj_type_name})) +
             altmessage)
 
         if isinstance(obj, type):
@@ -220,7 +204,7 @@ def deprecated(since, message='', name='', alternative='', pending=False,
         else:
             return deprecate_function(obj, message)
 
-    if type(message) == type(deprecate):
+    if type(message) is type(deprecate):
         return deprecate(message)
 
     return deprecate
@@ -289,6 +273,229 @@ def deprecated_attribute(name, since, message=None, alternative=None,
     return property(get, set, delete)
 
 
+def deprecated_renamed_argument(old_name, new_name, since,
+                                arg_in_kwargs=False, relax=False):
+    """Deprecate a _renamed_ function argument.
+
+    The decorator assumes that the argument with the ``old_name`` was removed
+    from the function signature and the ``new_name`` replaced it at the
+    **same position** in the signature.  If the ``old_name`` argument is
+    given when calling the decorated function the decorator will catch it and
+    issue a deprecation warning and pass it on as ``new_name`` argument.
+
+    Parameters
+    ----------
+    old_name : str or list/tuple thereof
+        The old name of the argument.
+
+    new_name : str or list/tuple thereof
+        The new name of the argument.
+
+    since : str or number or list/tuple thereof
+        The release at which the old argument became deprecated.
+
+    arg_in_kwargs : bool or list/tuple thereof, optional
+        If the argument is not a named argument (for example it
+        was meant to be consumed by ``**kwargs``) set this to
+        ``True``.  Otherwise the decorator will throw an Exception
+        if the ``new_name`` cannot be found in the signature of
+        the decorated function.
+        Default is ``False``.
+
+    relax : bool or list/tuple thereof, optional
+        If ``False`` a ``TypeError`` is raised if both ``new_name`` and
+        ``old_name`` are given.  If ``True`` the value for ``new_name`` is used
+        and a Warning is issued.
+        Default is ``False``.
+
+    Raises
+    ------
+    TypeError
+        If the new argument name cannot be found in the function
+        signature and arg_in_kwargs was False or if it is used to
+        deprecate the name of the ``*args``-, ``**kwargs``-like arguments.
+        At runtime such an Error is raised if both the new_name
+        and old_name were specified when calling the function and
+        "relax=False".
+
+    Notes
+    -----
+    The decorator should be applied to a function where the **name**
+    of an argument was changed but it applies the same logic.
+
+    .. warning::
+        If ``old_name`` is a list or tuple the ``new_name`` and ``since`` must
+        also be a list or tuple with the same number of entries. ``relax`` and
+        ``arg_in_kwarg`` can be a single bool (applied to all) or also a
+        list/tuple with the same number of entries like ``new_name``, etc.
+
+    Examples
+    --------
+    The deprecation warnings are not shown in the following examples.
+
+    To deprecate a positional or keyword argument::
+
+        >>> from astropy.utils.decorators import deprecated_renamed_argument
+        >>> @deprecated_renamed_argument('sig', 'sigma', '1.0')
+        ... def test(sigma):
+        ...     return sigma
+
+        >>> test(2)
+        2
+        >>> test(sigma=2)
+        2
+        >>> test(sig=2)
+        2
+
+    To deprecate an argument catched inside the ``**kwargs`` the
+    ``arg_in_kwargs`` has to be set::
+
+        >>> @deprecated_renamed_argument('sig', 'sigma', '1.0',
+        ...                             arg_in_kwargs=True)
+        ... def test(**kwargs):
+        ...     return kwargs['sigma']
+
+        >>> test(sigma=2)
+        2
+        >>> test(sig=2)
+        2
+
+    By default providing the new and old keyword will lead to an Exception. If
+    a Warning is desired set the ``relax`` argument::
+
+        >>> @deprecated_renamed_argument('sig', 'sigma', '1.0', relax=True)
+        ... def test(sigma):
+        ...     return sigma
+
+        >>> test(sig=2)
+        2
+
+    It is also possible to replace multiple arguments. The ``old_name``,
+    ``new_name`` and ``since`` have to be `tuple` or `list` and contain the
+    same number of entries::
+
+        >>> @deprecated_renamed_argument(['a', 'b'], ['alpha', 'beta'],
+        ...                              ['1.0', 1.2])
+        ... def test(alpha, beta):
+        ...     return alpha, beta
+
+        >>> test(a=2, b=3)
+        (2, 3)
+
+    In this case ``arg_in_kwargs`` and ``relax`` can be a single value (which
+    is applied to all renamed arguments) or must also be a `tuple` or `list`
+    with values for each of the arguments.
+
+    .. warning::
+        This decorator needs to access the original signature of the decorated
+        function. Therefore this decorator must be the **first** decorator on
+        any function if it needs to work for Python before version 3.4.
+    """
+    cls_iter = (list, tuple)
+    if isinstance(old_name, cls_iter):
+        n = len(old_name)
+        # Assume that new_name and since are correct (tuple/list with the
+        # appropriate length) in the spirit of the "consenting adults". But the
+        # optional parameters may not be set, so if these are not iterables
+        # wrap them.
+        if not isinstance(arg_in_kwargs, cls_iter):
+            arg_in_kwargs = [arg_in_kwargs] * n
+        if not isinstance(relax, cls_iter):
+            relax = [relax] * n
+    else:
+        # To allow a uniform approach later on, wrap all arguments in lists.
+        n = 1
+        old_name = [old_name]
+        new_name = [new_name]
+        since = [since]
+        arg_in_kwargs = [arg_in_kwargs]
+        relax = [relax]
+
+    def decorator(function):
+        # Lazy import to avoid cyclic imports
+        from .compat.funcsigs import signature
+
+        # The named arguments of the function.
+        arguments = signature(function).parameters
+        keys = list(arguments.keys())
+        position = [None] * n
+
+        for i in range(n):
+            # Determine the position of the argument.
+            if new_name[i] in arguments:
+                param = arguments[new_name[i]]
+                # There are several possibilities now:
+
+                # 1.) Positional or keyword argument:
+                if param.kind == param.POSITIONAL_OR_KEYWORD:
+                    position[i] = keys.index(new_name[i])
+
+                # 2.) Keyword only argument (Python 3 only):
+                elif param.kind == param.KEYWORD_ONLY:
+                    # These cannot be specified by position.
+                    position[i] = None
+
+                # 3.) positional-only argument, varargs, varkwargs or some
+                #     unknown type:
+                else:
+                    raise TypeError('cannot replace argument "{0}" of kind {1}'
+                                    '.'.format(new_name[i], repr(param.kind)))
+
+            # In case the argument is not found in the list of arguments
+            # the only remaining possibility is that it should be catched
+            # by some kind of **kwargs argument.
+            # This case has to be explicitly specified, otherwise throw
+            # an exception!
+            elif arg_in_kwargs[i]:
+                position[i] = None
+            else:
+                raise TypeError('"{}" was not specified in the function '
+                                'signature. If it was meant to be part of '
+                                '"**kwargs" then set "arg_in_kwargs" to "True"'
+                                '.'.format(new_name[i]))
+
+        @functools.wraps(function)
+        def wrapper(*args, **kwargs):
+            for i in range(n):
+                # The only way to have oldkeyword inside the function is
+                # that it is passed as kwarg because the oldkeyword
+                # parameter was renamed to newkeyword.
+                if old_name[i] in kwargs:
+                    value = kwargs.pop(old_name[i])
+                    warnings.warn('"{0}" was deprecated in version {1} '
+                                  'and will be removed in a future version. '
+                                  'Use argument "{2}" instead.'
+                                  ''.format(old_name[i], since[i],
+                                            new_name[i]),
+                                  AstropyDeprecationWarning)
+
+                    # Check if the newkeyword was given as well.
+                    newarg_in_args = (position[i] is not None and
+                                      len(args) > position[i])
+                    newarg_in_kwargs = new_name[i] in kwargs
+
+                    if newarg_in_args or newarg_in_kwargs:
+                        # If both are given print a Warning if relax is True or
+                        # raise an Exception is relax is False.
+                        if relax[i]:
+                            warnings.warn('"{0}" and "{1}" keywords were set. '
+                                          'Using the value of "{1}".'
+                                          ''.format(old_name[i], new_name[i]),
+                                          AstropyUserWarning)
+                        else:
+                            raise TypeError('cannot specify both "{}" and "{}"'
+                                            '.'.format(old_name[i],
+                                                       new_name[i]))
+                    else:
+                        # If the new argument isn't specified just pass the old
+                        # one with the name of the new argument to the function
+                        kwargs[new_name[i]] = value
+            return function(*args, **kwargs)
+
+        return wrapper
+    return decorator
+
+
 # TODO: This can still be made to work for setters by implementing an
 # accompanying metaclass that supports it; we just don't need that right this
 # second
@@ -493,8 +700,12 @@ class lazyproperty(property):
     the ``print`` statement is not executed.  Only the return value from the
     first access off ``complicated_property`` is returned.
 
-    If a setter for this property is defined, it will still be possible to
-    manually update the value of the property, if that capability is desired.
+    By default, a setter and deleter are used which simply overwrite and
+    delete, respectively, the value stored in ``__dict__``. Any user-specified
+    setter or deleter is executed before executing these default actions.
+    The one exception is that the default setter is not run if the user setter
+    already sets the new value in ``__dict__`` and returns that value and the
+    returned value is not ``None``.
 
     Adapted from the recipe at
     http://code.activestate.com/recipes/363602-lazy-property-evaluation
@@ -585,38 +796,8 @@ class sharedmethod(classmethod):
         this implements the Example.identify classmethod
     """
 
-    if sys.version_info[:2] < (2, 7):
-        # Workaround for Python 2.6 which does not have classmethod.__func__
-        @property
-        def __func__(self):
-            try:
-                meth = classmethod.__get__(self, self.__obj__,
-                                           self.__objtype__)
-            except AttributeError:
-                # self.__obj__ not set when called from __get__, but then it
-                # doesn't matter anyways
-                meth = classmethod.__get__(self, None, object)
-            return meth.__func__
-
-        def __getobjwrapper(orig_get):
-            """
-            Used to temporarily set/unset self.__obj__ and self.__objtype__
-            for use by __func__.
-            """
-            def __get__(self, obj, objtype=None):
-                self.__obj__ = obj
-                self.__objtype__ = objtype
-
-                try:
-                    return orig_get(self, obj, objtype)
-                finally:
-                    del self.__obj__
-                    del self.__objtype__
-
-            return __get__
-    else:
-        def __getobjwrapper(func):
-            return func
+    def __getobjwrapper(func):
+        return func
 
     @__getobjwrapper
     def __get__(self, obj, objtype=None):
@@ -641,7 +822,7 @@ class sharedmethod(classmethod):
 
     del __getobjwrapper
 
-    if six.PY3:
+    if not six.PY2:
         # The 'instancemethod' type of Python 2 and the method type of
         # Python 3 have slightly different constructors
         @staticmethod
@@ -684,11 +865,12 @@ def wraps(wrapped, assigned=functools.WRAPPER_ASSIGNMENTS,
     return wrapper
 
 
-if isinstance(wraps.__doc__, six.string_types):
+if (isinstance(wraps.__doc__, six.string_types) and
+        wraps.__doc__ is not None and functools.wraps.__doc__ is not None):
     wraps.__doc__ += functools.wraps.__doc__
 
 
-if six.PY3:
+if not six.PY2:
     def _get_function_args_internal(func):
         """
         Utility function for `wraps`.
@@ -751,3 +933,231 @@ def _get_function_args(func, exclude_args=()):
                 all_args[arg_type] = None
 
     return all_args
+
+
+def format_doc(docstring, *args, **kwargs):
+    """
+    Replaces the docstring of the decorated object and then formats it.
+
+    The formatting works like :meth:`str.format` and if the decorated object
+    already has a docstring this docstring can be included in the new
+    documentation if you use the ``{__doc__}`` placeholder.
+    Its primary use is for reusing a *long* docstring in multiple functions
+    when it is the same or only slightly different between them.
+
+    Parameters
+    ----------
+    docstring : str or object or None
+        The docstring that will replace the docstring of the decorated
+        object. If it is an object like a function or class it will
+        take the docstring of this object. If it is a string it will use the
+        string itself. One special case is if the string is ``None`` then
+        it will use the decorated functions docstring and formats it.
+
+    args :
+        passed to :meth:`str.format`.
+
+    kwargs :
+        passed to :meth:`str.format`. If the function has a (not empty)
+        docstring the original docstring is added to the kwargs with the
+        keyword ``'__doc__'``.
+
+    Raises
+    ------
+    ValueError
+        If the ``docstring`` (or interpreted docstring if it was ``None``
+        or not a string) is empty.
+
+    IndexError, KeyError
+        If a placeholder in the (interpreted) ``docstring`` was not filled. see
+        :meth:`str.format` for more information.
+
+    Notes
+    -----
+    Using this decorator allows, for example Sphinx, to parse the
+    correct docstring.
+
+    Examples
+    --------
+
+    Replacing the current docstring is very easy::
+
+        >>> from astropy.utils.decorators import format_doc
+        >>> @format_doc('''Perform num1 + num2''')
+        ... def add(num1, num2):
+        ...     return num1+num2
+        ...
+        >>> help(add) # doctest: +SKIP
+        Help on function add in module __main__:
+        <BLANKLINE>
+        add(num1, num2)
+            Perform num1 + num2
+
+    sometimes instead of replacing you only want to add to it::
+
+        >>> doc = '''
+        ...       {__doc__}
+        ...       Parameters
+        ...       ----------
+        ...       num1, num2 : Numbers
+        ...       Returns
+        ...       -------
+        ...       result: Number
+        ...       '''
+        >>> @format_doc(doc)
+        ... def add(num1, num2):
+        ...     '''Perform addition.'''
+        ...     return num1+num2
+        ...
+        >>> help(add) # doctest: +SKIP
+        Help on function add in module __main__:
+        <BLANKLINE>
+        add(num1, num2)
+            Perform addition.
+            Parameters
+            ----------
+            num1, num2 : Numbers
+            Returns
+            -------
+            result : Number
+
+    in case one might want to format it further::
+
+        >>> doc = '''
+        ...       Perform {0}.
+        ...       Parameters
+        ...       ----------
+        ...       num1, num2 : Numbers
+        ...       Returns
+        ...       -------
+        ...       result: Number
+        ...           result of num1 {op} num2
+        ...       {__doc__}
+        ...       '''
+        >>> @format_doc(doc, 'addition', op='+')
+        ... def add(num1, num2):
+        ...     return num1+num2
+        ...
+        >>> @format_doc(doc, 'subtraction', op='-')
+        ... def subtract(num1, num2):
+        ...     '''Notes: This one has additional notes.'''
+        ...     return num1-num2
+        ...
+        >>> help(add) # doctest: +SKIP
+        Help on function add in module __main__:
+        <BLANKLINE>
+        add(num1, num2)
+            Perform addition.
+            Parameters
+            ----------
+            num1, num2 : Numbers
+            Returns
+            -------
+            result : Number
+                result of num1 + num2
+        >>> help(subtract) # doctest: +SKIP
+        Help on function subtract in module __main__:
+        <BLANKLINE>
+        subtract(num1, num2)
+            Perform subtraction.
+            Parameters
+            ----------
+            num1, num2 : Numbers
+            Returns
+            -------
+            result : Number
+                result of num1 - num2
+            Notes : This one has additional notes.
+
+    These methods can be combined an even taking the docstring from another
+    object is possible as docstring attribute. You just have to specify the
+    object::
+
+        >>> @format_doc(add)
+        ... def another_add(num1, num2):
+        ...     return num1 + num2
+        ...
+        >>> help(another_add) # doctest: +SKIP
+        Help on function another_add in module __main__:
+        <BLANKLINE>
+        another_add(num1, num2)
+            Perform addition.
+            Parameters
+            ----------
+            num1, num2 : Numbers
+            Returns
+            -------
+            result : Number
+                result of num1 + num2
+
+    But be aware that this decorator *only* formats the given docstring not
+    the strings passed as ``args`` or ``kwargs`` (not even the original
+    docstring)::
+
+        >>> @format_doc(doc, 'addition', op='+')
+        ... def yet_another_add(num1, num2):
+        ...    '''This one is good for {0}.'''
+        ...    return num1 + num2
+        ...
+        >>> help(yet_another_add) # doctest: +SKIP
+        Help on function yet_another_add in module __main__:
+        <BLANKLINE>
+        yet_another_add(num1, num2)
+            Perform addition.
+            Parameters
+            ----------
+            num1, num2 : Numbers
+            Returns
+            -------
+            result : Number
+                result of num1 + num2
+            This one is good for {0}.
+
+    To work around it you could specify the docstring to be ``None``::
+
+        >>> @format_doc(None, 'addition')
+        ... def last_add_i_swear(num1, num2):
+        ...    '''This one is good for {0}.'''
+        ...    return num1 + num2
+        ...
+        >>> help(last_add_i_swear) # doctest: +SKIP
+        Help on function last_add_i_swear in module __main__:
+        <BLANKLINE>
+        last_add_i_swear(num1, num2)
+            This one is good for addition.
+
+    Using it with ``None`` as docstring allows to use the decorator twice
+    on an object to first parse the new docstring and then to parse the
+    original docstring or the ``args`` and ``kwargs``.
+    """
+    def set_docstring(obj):
+        if docstring is None:
+            # None means: use the objects __doc__
+            doc = obj.__doc__
+            # Delete documentation in this case so we don't end up with
+            # awkwardly self-inserted docs.
+            obj.__doc__ = None
+        elif isinstance(docstring, six.string_types):
+            # String: use the string that was given
+            doc = docstring
+        else:
+            # Something else: Use the __doc__ of this
+            doc = docstring.__doc__
+
+        if not doc:
+            # In case the docstring is empty it's probably not what was wanted.
+            raise ValueError('docstring must be a string or containing a '
+                             'docstring that is not empty.')
+
+        # If the original has a not-empty docstring append it to the format
+        # kwargs.
+        kwargs['__doc__'] = obj.__doc__ or ''
+        if six.PY2 and isinstance(obj, type):
+            # For python 2 we must create a subclass because there the __doc__
+            # is not mutable for objects.
+            obj = type(obj.__name__, (obj,), {'__doc__': doc.format(*args, **kwargs),
+                                              '__module__': obj.__module__})
+        else:
+            obj.__doc__ = doc.format(*args, **kwargs)
+        return obj
+    return set_docstring
