@@ -4,12 +4,15 @@ from __future__ import (absolute_import, division, print_function,
 
 import json
 import os
+from datetime import datetime
+import locale
 
 import numpy as np
 
 from .. import data, misc
 from ...tests.helper import remote_data
 from ...extern import six
+from ...tests.helper import pytest
 
 
 def test_isiterable():
@@ -20,6 +23,12 @@ def test_isiterable():
     assert misc.isiterable(np.array([1, 2, 3])) is True
 
 
+def test_signal_number_to_name_no_failure():
+    # Regression test for #5340: ensure signal_number_to_name throws no
+    # AttributeError (it used ".iteritems()" which was removed in Python3).
+    misc.signal_number_to_name(0)
+
+
 @remote_data
 def test_api_lookup():
     strurl = misc.find_api_page('astropy.utils.misc', 'dev', False, timeout=3)
@@ -68,3 +77,40 @@ def test_inherit_docstrings():
     if Base.__call__.__doc__ is not None:
         # TODO: Maybe if __doc__ is None this test should be skipped instead?
         assert Subclass.__call__.__doc__ == "FOO"
+
+
+def test_set_locale():
+    # First, test if the required locales are available
+    current = locale.setlocale(locale.LC_ALL)
+    try:
+        locale.setlocale(locale.LC_ALL, str('en_US'))
+        locale.setlocale(locale.LC_ALL, str('de_DE'))
+    except locale.Error as e:
+        pytest.skip('Locale error: {}'.format(e))
+    finally:
+        locale.setlocale(locale.LC_ALL, current)
+
+    date = datetime(2000, 10, 1, 0, 0, 0)
+    day_mon = date.strftime('%a, %b')
+
+    with misc.set_locale('en_US'):
+        assert date.strftime('%a, %b') == 'Sun, Oct'
+
+    with misc.set_locale('de_DE'):
+        assert date.strftime('%a, %b') == 'So, Okt'
+
+    # Back to original
+    assert date.strftime('%a, %b') == day_mon
+
+    with misc.set_locale(current):
+        assert date.strftime('%a, %b') == day_mon
+
+
+def test_check_broadcast():
+    assert misc.check_broadcast((10, 1), (3,)) == (10, 3)
+    assert misc.check_broadcast((10, 1), (3,), (4, 1, 1, 3)) == (4, 1, 10, 3)
+    with pytest.raises(ValueError):
+        misc.check_broadcast((10, 2), (3,))
+
+    with pytest.raises(ValueError):
+        misc.check_broadcast((10, 1), (3,), (4, 1, 2, 3))
