@@ -5,18 +5,18 @@ from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 from ..extern import six
+from ..extern.six.moves import range
 
 # STDLIB
 import time
 import warnings
-from collections import Iterable
+from collections import Iterable, OrderedDict
 from functools import partial, wraps
 
 # THIRD-PARTY
 import numpy as np
 
 # LOCAL
-from . import OrderedDict
 from .. import units as u
 from .. import log
 from .. import modeling
@@ -126,7 +126,7 @@ class RunTimePredictor(object):
     pow(10, 810)
     10000000000...
 
-    Fit a straight line assuming :math:`\\textnormal{arg}^{1}` relationship
+    Fit a straight line assuming :math:`\\text{arg}^{1}` relationship
     (coefficients are returned):
 
     >>> p.do_fit()  # doctest: +SKIP
@@ -225,7 +225,7 @@ class RunTimePredictor(object):
         ----------
         model : `astropy.modeling.Model`
             Model for the expected trend of run time (Y-axis)
-            w.r.t. :math:`\\textnormal{arg}^{\\textnormal{power}}` (X-axis).
+            w.r.t. :math:`\\text{arg}^{\\text{power}}` (X-axis).
             If `None`, will use `~astropy.modeling.polynomial.Polynomial1D`
             with ``degree=1``.
 
