@@ -1,9 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from .stretch import *
+from .hist import *
 from .interval import *
+from .mpl_normalize import *
+from .mpl_style import *
+from .stretch import *
 from .transform import *
 from .ui import *
-from .mpl_style import *
-from .hist import *
 from .units import *
+from .lupton_rgb import *
