@@ -7,7 +7,7 @@ matplotlib plotting style.
 It is mostly here to allow a consistent plotting style in tutorials,
 but can be used to prepare any matplotlib figure.
 
-Using a matplotlib version > 1.4 you can do::
+Using a matplotlib version >= 1.5 you can do::
 
     >>> import matplotlib.pyplot as plt
     >>> from astropy.visualization import astropy_mpl_style
@@ -31,8 +31,14 @@ example), you should reset the matplotlib settings to the library defaults
     >>> mpl.rcdefaults()
     >>> mpl.rcParams.update(astropy_mpl_style)
 """
+from ..utils import minversion
+# This returns False if matplotlib cannot be imported
+MATPLOTLIB_GE_1_5 = minversion('matplotlib', '1.5')
 
 
+__all__ = ['astropy_mpl_style_1', 'astropy_mpl_style',
+           'astropy_mpl_docs_style']
+
 astropy_mpl_style_1 = {
 
     # Lines
@@ -61,13 +67,6 @@ astropy_mpl_style_1 = {
     'axes.labelsize': 'large',
     'axes.labelcolor': 'k',
     'axes.axisbelow': True,
-    'axes.color_cycle': ['#348ABD',   # blue
-                         '#7A68A6',   # purple
-                         '#A60628',   # red
-                         '#467821',   # green
-                         '#CF4457',   # pink
-                         '#188487',   # turquoise
-                         '#E24A33'],  # orange
 
     # Ticks
     'xtick.major.size': 0,
@@ -96,6 +95,23 @@ astropy_mpl_style_1 = {
     # Other
     'savefig.dpi': 72,
 }
+color_cycle = ['#348ABD',   # blue
+               '#7A68A6',   # purple
+               '#A60628',   # red
+               '#467821',   # green
+               '#CF4457',   # pink
+               '#188487',   # turquoise
+               '#E24A33']  # orange
+
+if MATPLOTLIB_GE_1_5:
+    # This is a dependency of matplotlib, so should be present.
+    from cycler import cycler
+    astropy_mpl_style_1['axes.prop_cycle'] = cycler('color', color_cycle)
+
+else:
+    astropy_mpl_style_1['axes.color_cycle'] = color_cycle
+
+
 '''
 Version 1 astropy plotting style for matplotlib.
 
@@ -114,7 +130,7 @@ astropy_mpl_docs_style = astropy_mpl_style_1.copy()
 The style used in the astropy documentation.
 '''
 
-astropy_mpl_docs_style['axes.color_cycle'] = [
+color_cycle_docs = [
     '#E24A33',   # orange
     '#348ABD',   # blue
     '#467821',   # green
@@ -124,4 +140,13 @@ astropy_mpl_docs_style['axes.color_cycle'] = [
     '#188487'    # turquoise
 ]
 
+if MATPLOTLIB_GE_1_5:
+    astropy_mpl_docs_style['axes.prop_cycle'] = cycler('color',
+                                                       color_cycle_docs)
+else:
+    astropy_mpl_docs_style['axes.color_cycle'] = color_cycle_docs
+
 astropy_mpl_docs_style['axes.grid'] = False
+astropy_mpl_docs_style['figure.figsize'] = (6, 6)
+astropy_mpl_docs_style['savefig.facecolor'] = 'none'
+astropy_mpl_docs_style['savefig.bbox'] = 'tight'
