@@ -1,9 +1,9 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-.. _wcslib: http://www.atnf.csiro.au/~mcalabre/WCS/
-.. _FITS WCS standard: http://fits.gsfc.nasa.gov/fits_wcs.html
+.. _wcslib: http://www.atnf.csiro.au/people/mcalabre/WCS/wcslib/index.html
 .. _distortion paper: http://www.atnf.csiro.au/people/mcalabre/WCS/dcs_20040422.pdf
 .. _SIP: http://irsa.ipac.caltech.edu/data/SPITZER/docs/files/spitzer/shupeADASS.pdf
+.. _FITS WCS standard: http://fits.gsfc.nasa.gov/fits_wcs.html
 
 `astropy.wcs` contains utilities for managing World Coordinate System
 (WCS) transformations in FITS files.  These transformations map the
