@@ -18,6 +18,7 @@
 #include <wcsmath.h>
 #include <wcsprintf.h>
 #include <wcsunits.h>
+#include <tab.h>
 
 #include "astropy_wcs/isnan.h"
 #include "astropy_wcs/distortion.h"
@@ -63,6 +64,13 @@ convert_rejections_to_warnings() {
   PyObject *wcs_module = NULL;
   PyObject *FITSFixedWarning = NULL;
   int status = -1;
+  char delimiter;
+
+#ifdef HAVE_WCSLIB_VERSION
+  delimiter = ',';
+#else
+  delimiter = ':';
+#endif
 
   if (wcsprintf_buf()[0] == 0) {
     return 0;
@@ -105,7 +113,7 @@ convert_rejections_to_warnings() {
     /* For the second line, remove everything up to and including the
        first colon */
     for (; *src != 0; ++src) {
-      if (*src == ':') {
+      if (*src == delimiter) {
         ++src;
         break;
       }
@@ -205,18 +213,20 @@ PyWcsprm_init(
   int            keysel        = -1;
   PyObject*      colsel        = Py_None;
   PyArrayObject* colsel_array  = NULL;
+  int*           colsel_data  = NULL;
   int*           colsel_ints   = NULL;
+  int            warnings      = 1;
   int            nreject       = 0;
   int            nwcs          = 0;
   struct wcsprm* wcs           = NULL;
   int            i             = 0;
   const char*    keywords[]    = {"header", "key", "relax", "naxis", "keysel",
-                                  "colsel", NULL};
+                                  "colsel", "warnings", NULL};
 
   if (!PyArg_ParseTupleAndKeywords(
-          args, kwds, "|OsOiiO:WCSBase.__init__",
+          args, kwds, "|OsOiiOi:WCSBase.__init__",
           (char **)keywords, &header_obj, &key, &relax_obj, &naxis, &keysel,
-          &colsel)) {
+          &colsel, &warnings)) {
     return -1;
   }
 
@@ -291,8 +301,6 @@ PyWcsprm_init(
             "relax must be True, False or an integer.");
         return -1;
       }
-      /* Mask out any invalid flags */
-      relax &= WCSHDR_all;
     }
 
     if (!is_valid_alt_key(key)) {
@@ -316,7 +324,7 @@ PyWcsprm_init(
 
     if (colsel != Py_None) {
       colsel_array = (PyArrayObject*) PyArray_ContiguousFromAny(
-        colsel, 1, 1, PyArray_INT);
+        colsel, 1, 1, NPY_INT);
       if (colsel_array == NULL) {
         return -1;
       }
@@ -331,8 +339,9 @@ PyWcsprm_init(
       }
 
       colsel_ints[0] = (int)PyArray_DIM(colsel_array, 0);
+      colsel_data = (int *)PyArray_DATA(colsel_array);
       for (i = 0; i < colsel_ints[0]; ++i) {
-        colsel_ints[i+1] = colsel_array->data[i];
+        colsel_ints[i+1] = colsel_data[i];
       }
 
       Py_DECREF(colsel_array);
@@ -369,15 +378,13 @@ PyWcsprm_init(
 
     if (status != 0) {
       free(colsel_ints);
-      PyErr_SetString(
-          PyExc_MemoryError,
-          "Memory allocation error.");
+      wcshdr_err_to_python_exc(status);
       return -1;
     }
 
     wcsvfree(&nwcs, &wcs);
 
-    if (convert_rejections_to_warnings()) {
+    if (warnings && convert_rejections_to_warnings()) {
       free(colsel_ints);
       return -1;
     }
@@ -407,9 +414,7 @@ PyWcsprm_init(
     free(colsel_ints);
 
     if (status != 0) {
-      PyErr_SetString(
-          PyExc_MemoryError,
-          "Memory allocation error.");
+      wcshdr_err_to_python_exc(status);
       return -1;
     }
 
@@ -477,6 +482,7 @@ PyWcsprm_bounds_check(
       bounds |= 1;
   }
 
+  wcsprm_python2c(&self->x);
   wcsbchk(&self->x, bounds);
 
   Py_RETURN_NONE;
@@ -528,18 +534,19 @@ PyWcsprm_find_all_wcs(
   PyObject*      relax_obj     = NULL;
   int            relax         = 0;
   int            keysel        = 0;
+  int            warnings      = 1;
   int            nreject       = 0;
   int            nwcs          = 0;
   struct wcsprm* wcs           = NULL;
   PyObject*      result        = NULL;
   PyWcsprm*      subresult     = NULL;
   int            i             = 0;
-  const char*    keywords[]    = {"header", "relax", "keysel", NULL};
+  const char*    keywords[]    = {"header", "relax", "keysel", "warnings", NULL};
   int            status        = -1;
 
   if (!PyArg_ParseTupleAndKeywords(
-          args, kwds, "O|Oi:find_all_wcs",
-          (char **)keywords, &header_obj, &relax_obj, &keysel)) {
+          args, kwds, "O|Oii:find_all_wcs",
+          (char **)keywords, &header_obj, &relax_obj, &keysel, &warnings)) {
     return NULL;
   }
 
@@ -575,9 +582,6 @@ PyWcsprm_find_all_wcs(
           "relax must be True, False or an integer.");
       return NULL;
     }
-
-    /* Mask out any invalid flags */
-    relax &= WCSHDR_all;
   }
 
   /* Call the header parser twice, the first time to get warnings
@@ -610,15 +614,13 @@ PyWcsprm_find_all_wcs(
   Py_END_ALLOW_THREADS
 
   if (status != 0) {
-    PyErr_SetString(
-        PyExc_MemoryError,
-        "Memory allocation error.");
+    wcshdr_err_to_python_exc(status);
     return NULL;
   }
 
   wcsvfree(&nwcs, &wcs);
 
-  if (convert_rejections_to_warnings()) {
+  if (warnings && convert_rejections_to_warnings()) {
     return NULL;
   }
 
@@ -647,9 +649,7 @@ PyWcsprm_find_all_wcs(
   Py_END_ALLOW_THREADS
 
   if (status != 0) {
-    PyErr_SetString(
-        PyExc_MemoryError,
-        "Memory allocation error.");
+    wcshdr_err_to_python_exc(status);
     return NULL;
   }
 
@@ -788,7 +788,7 @@ PyWcsprm_cylfix(
 
   if (naxis_obj != NULL && naxis_obj != Py_None) {
     naxis_array = (PyArrayObject*)PyArray_ContiguousFromAny(
-        naxis_obj, 1, 1, PyArray_INT);
+        naxis_obj, 1, 1, NPY_INT);
     if (naxis_array == NULL) {
       return NULL;
     }
@@ -891,7 +891,7 @@ PyWcsprm_fix(
 
   if (naxis_obj != NULL && naxis_obj != Py_None) {
     naxis_array = (PyArrayObject*)PyArray_ContiguousFromAny(
-        naxis_obj, 1, 1, PyArray_INT);
+        naxis_obj, 1, 1, NPY_INT);
     if (naxis_array == NULL) {
       return NULL;
     }
@@ -1094,7 +1094,7 @@ PyWcsprm_mix(
   }
 
   world = (PyArrayObject*)PyArray_ContiguousFromAny
-    (world_obj, PyArray_DOUBLE, 1, 1);
+    (world_obj, NPY_DOUBLE, 1, 1);
   if (world == NULL) {
     PyErr_SetString(
         PyExc_TypeError,
@@ -1111,7 +1111,7 @@ PyWcsprm_mix(
   }
 
   pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny
-    (pixcrd_obj, PyArray_DOUBLE, 1, 1);
+    (pixcrd_obj, NPY_DOUBLE, 1, 1);
   if (pixcrd == NULL) {
     PyErr_SetString(
         PyExc_TypeError,
@@ -1148,19 +1148,19 @@ PyWcsprm_mix(
    */
   naxis = (Py_ssize_t)self->x.naxis;
   phi = (PyArrayObject*)PyArray_SimpleNew
-    (1, &naxis, PyArray_DOUBLE);
+    (1, &naxis, NPY_DOUBLE);
   if (phi == NULL) {
     goto exit;
   }
 
   theta = (PyArrayObject*)PyArray_SimpleNew
-    (1, &naxis, PyArray_DOUBLE);
+    (1, &naxis, NPY_DOUBLE);
   if (theta == NULL) {
     goto exit;
   }
 
   imgcrd = (PyArrayObject*)PyArray_SimpleNew
-    (1, &naxis, PyArray_DOUBLE);
+    (1, &naxis, NPY_DOUBLE);
   if (imgcrd == NULL) {
     goto exit;
   }
@@ -1249,7 +1249,7 @@ PyWcsprm_p2s(
   naxis = self->x.naxis;
 
   pixcrd = (PyArrayObject*)PyArray_ContiguousFromAny
-    (pixcrd_obj, PyArray_DOUBLE, 2, 2);
+    (pixcrd_obj, NPY_DOUBLE, 2, 2);
   if (pixcrd == NULL) {
     return NULL;
   }
@@ -1265,31 +1265,31 @@ PyWcsprm_p2s(
   /* Now we allocate a bunch of numpy arrays to store the results in.
    */
   imgcrd = (PyArrayObject*)PyArray_SimpleNew(
-      2, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+      2, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (imgcrd == NULL) {
     goto exit;
   }
 
   phi = (PyArrayObject*)PyArray_SimpleNew(
-      1, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+      1, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (phi == NULL) {
     goto exit;
   }
 
   theta = (PyArrayObject*)PyArray_SimpleNew(
-      1, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+      1, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (theta == NULL) {
     goto exit;
   }
 
   world = (PyArrayObject*)PyArray_SimpleNew(
-      2, PyArray_DIMS(pixcrd), PyArray_DOUBLE);
+      2, PyArray_DIMS(pixcrd), NPY_DOUBLE);
   if (world == NULL) {
     goto exit;
   }
 
   stat = (PyArrayObject*)PyArray_SimpleNew(
-      1, PyArray_DIMS(pixcrd), PyArray_INT);
+      1, PyArray_DIMS(pixcrd), NPY_INT);
   if (stat == NULL) {
     goto exit;
   }
@@ -1391,7 +1391,7 @@ PyWcsprm_s2p(
   naxis = self->x.naxis;
 
   world = (PyArrayObject*)PyArray_ContiguousFromAny(
-      world_obj, PyArray_DOUBLE, 2, 2);
+      world_obj, NPY_DOUBLE, 2, 2);
   if (world == NULL) {
     return NULL;
   }
@@ -1408,31 +1408,31 @@ PyWcsprm_s2p(
    * results in.
    */
   phi = (PyArrayObject*)PyArray_SimpleNew(
-      1, PyArray_DIMS(world), PyArray_DOUBLE);
+      1, PyArray_DIMS(world), NPY_DOUBLE);
   if (phi == NULL) {
     goto exit;
   }
 
   theta = (PyArrayObject*)PyArray_SimpleNew(
-      1, PyArray_DIMS(world), PyArray_DOUBLE);
+      1, PyArray_DIMS(world), NPY_DOUBLE);
   if (phi == NULL) {
     goto exit;
   }
 
   imgcrd = (PyArrayObject*)PyArray_SimpleNew(
-      2, PyArray_DIMS(world), PyArray_DOUBLE);
+      2, PyArray_DIMS(world), NPY_DOUBLE);
   if (theta == NULL) {
     goto exit;
   }
 
   pixcrd = (PyArrayObject*)PyArray_SimpleNew(
-      2, PyArray_DIMS(world), PyArray_DOUBLE);
+      2, PyArray_DIMS(world), NPY_DOUBLE);
   if (pixcrd == NULL) {
     goto exit;
   }
 
   stat = (PyArrayObject*)PyArray_SimpleNew(
-      1, PyArray_DIMS(world), PyArray_INT);
+      1, PyArray_DIMS(world), NPY_INT);
   if (stat == NULL) {
     goto exit;
   }
@@ -1859,6 +1859,7 @@ PyWcsprm_sub(
   status = wcssub(1, &self->x, &nsub, axes, &py_dest_wcs->x);
   wcsprm_c2python(&self->x);
   if (PyWcsprm_cset(py_dest_wcs, 0)) {
+    status = -1;
     goto exit;
   }
   wcsprm_c2python(&py_dest_wcs->x);
@@ -2031,8 +2032,6 @@ PyWcsprm_set_alt(
 
   strncpy(self->x.alt, value_string, 2);
 
-  note_change(self);
-
   return 0;
 }
 
@@ -2195,8 +2194,6 @@ PyWcsprm_set_cname(
     return -1;
   }
 
-  note_change(self);
-
   return set_str_list("cname", value, (Py_ssize_t)self->x.naxis, 0, self->x.cname);
 }
 
@@ -2230,8 +2227,6 @@ PyWcsprm_set_colax(
 
   naxis = (Py_ssize_t)self->x.naxis;
 
-  note_change(self);
-
   return set_int_array("colax", value, 1, &naxis, self->x.colax);
 }
 
@@ -2249,8 +2244,6 @@ PyWcsprm_set_colnum(
     PyObject* value,
     /*@unused@*/ void* closure) {
 
-  note_change(self);
-
   return set_int("colnum", value, &self->x.colnum);
 }
 
@@ -2284,8 +2277,6 @@ PyWcsprm_set_crder(
 
   naxis = (Py_ssize_t)self->x.naxis;
 
-  note_change(self);
-
   return set_double_array("crder", value, 1, &naxis, self->x.crder);
 }
 
@@ -2441,8 +2432,6 @@ PyWcsprm_set_csyer(
 
   naxis = (Py_ssize_t)self->x.naxis;
 
-  note_change(self);
-
   return set_double_array("csyer", value, 1, &naxis, self->x.csyer);
 }
 
@@ -2543,8 +2532,6 @@ PyWcsprm_set_dateavg(
     return -1;
   }
 
-  note_change(self);
-
   /* TODO: Verify that this looks like a date string */
 
   return set_string("dateavg", value, self->x.dateavg, 72);
@@ -2572,8 +2559,6 @@ PyWcsprm_set_dateobs(
     return -1;
   }
 
-  note_change(self);
-
   return set_string("dateobs", value, self->x.dateobs, 72);
 }
 
@@ -2596,8 +2581,6 @@ PyWcsprm_set_equinox(
     return 0;
   }
 
-  note_change(self);
-
   return set_double("equinox", value, &self->x.equinox);
 }
 
@@ -2741,8 +2724,6 @@ PyWcsprm_set_mjdavg(
     PyObject* value,
     /*@unused@*/ void* closure) {
 
-  note_change(self);
-
   if (value == NULL) {
     self->x.mjdavg = (double)NPY_NAN;
     return 0;
@@ -2797,8 +2778,6 @@ PyWcsprm_set_name(
     return -1;
   }
 
-  note_change(self);
-
   return set_string("name", value, self->x.wcsname, 72);
 }
 
@@ -2836,8 +2815,6 @@ PyWcsprm_set_obsgeo(
     return -1;
   }
 
-  note_change(self);
-
   if (value == NULL) {
     self->x.obsgeo[0] = NPY_NAN;
     self->x.obsgeo[1] = NPY_NAN;
@@ -2991,8 +2968,6 @@ PyWcsprm_set_radesys(
     return -1;
   }
 
-  note_change(self);
-
   return set_string("radesys", value, self->x.radesys, 72);
 }
 
@@ -3074,8 +3049,6 @@ PyWcsprm_set_specsys(
     return -1;
   }
 
-  note_change(self);
-
   return set_string("specsys", value, self->x.specsys, 72);
 }
 
@@ -3128,8 +3101,6 @@ PyWcsprm_set_ssyssrc(
     return -1;
   }
 
-  note_change(self);
-
   return set_string("ssyssrc", value, self->x.ssyssrc, 72);
 }
 
@@ -3209,8 +3180,6 @@ PyWcsprm_set_velangl(
     return 0;
   }
 
-  note_change(self);
-
   return set_double("velangl", value, &self->x.velangl);
 }
 
@@ -3233,11 +3202,31 @@ PyWcsprm_set_velosys(
     return 0;
   }
 
-  note_change(self);
-
   return set_double("velosys", value, &self->x.velosys);
 }
 
+static PyObject*
+PyWcsprm_get_velref(
+    PyWcsprm* self,
+    /*@unused@*/ void* closure) {
+
+  return get_int("velref", self->x.velref);
+}
+
+static int
+PyWcsprm_set_velref(
+    PyWcsprm* self,
+    PyObject* value,
+    /*@unused@*/ void* closure) {
+
+  if (value == NULL) { /* deletion */
+    self->x.velref = 0;
+    return 0;
+  }
+
+  return set_int("velref", value, &self->x.velref);
+}
+
 /* static PyObject* */
 /* PyWcsprm_get_wtb( */
 /*     PyWcsprm* self, */
@@ -3290,8 +3279,6 @@ PyWcsprm_set_zsource(
     return 0;
   }
 
-  note_change(self);
-
   return set_double("zsource", value, &self->x.zsource);
 }
 
@@ -3345,6 +3332,7 @@ static PyGetSetDef PyWcsprm_getset[] = {
   {"theta0", (getter)PyWcsprm_get_theta0, (setter)PyWcsprm_set_theta0, (char *)doc_theta0},
   {"velangl", (getter)PyWcsprm_get_velangl, (setter)PyWcsprm_set_velangl, (char *)doc_velangl},
   {"velosys", (getter)PyWcsprm_get_velosys, (setter)PyWcsprm_set_velosys, (char *)doc_velosys},
+  {"velref", (getter)PyWcsprm_get_velref, (setter)PyWcsprm_set_velref, (char *)doc_velref},
   /* {"wtb", (getter)PyWcsprm_get_wtb, NULL, (char *)doc_tab}, */
   {"zsource", (getter)PyWcsprm_get_zsource, (setter)PyWcsprm_set_zsource, (char *)doc_zsource},
   {NULL}
@@ -3463,12 +3451,28 @@ _setup_wcsprm_type(
     CONSTANT(WCSHDR_PIXLIST)   ||
     CONSTANT(WCSHDR_none)      ||
     CONSTANT(WCSHDR_all)       ||
+    CONSTANT(WCSHDR_reject)    ||
+#ifdef WCSHDR_strict
+    CONSTANT(WCSHDR_strict)    ||
+#endif
     CONSTANT(WCSHDR_CROTAia)   ||
     CONSTANT(WCSHDR_EPOCHa)    ||
     CONSTANT(WCSHDR_VELREFa)   ||
     CONSTANT(WCSHDR_CD00i00j)  ||
     CONSTANT(WCSHDR_PC00i00j)  ||
     CONSTANT(WCSHDR_PROJPn)    ||
+#ifdef WCSHDR_CD0i_0ja
+    CONSTANT(WCSHDR_CD0i_0ja)  ||
+#endif
+#ifdef WCSHDR_PC0i_0ja
+    CONSTANT(WCSHDR_PC0i_0ja)  ||
+#endif
+#ifdef WCSHDR_PV0i_0ma
+    CONSTANT(WCSHDR_PV0i_0ma)  ||
+#endif
+#ifdef WCSHDR_PS0i_0ma
+    CONSTANT(WCSHDR_PS0i_0ma)  ||
+#endif
     CONSTANT(WCSHDR_RADECSYS)  ||
     CONSTANT(WCSHDR_VSOURCE)   ||
     CONSTANT(WCSHDR_DOBSn)     ||
@@ -3485,6 +3489,13 @@ _setup_wcsprm_type(
     CONSTANT(WCSHDO_CRPXna)    ||
     CONSTANT(WCSHDO_CNAMna)    ||
     CONSTANT(WCSHDO_WCSNna)    ||
+    CONSTANT(WCSHDO_P12)       ||
+    CONSTANT(WCSHDO_P13)       ||
+    CONSTANT(WCSHDO_P14)       ||
+    CONSTANT(WCSHDO_P15)       ||
+    CONSTANT(WCSHDO_P16)       ||
+    CONSTANT(WCSHDO_P17)       ||
+    CONSTANT(WCSHDO_EFMT)      ||
     CONSTANT(WCSCOMPARE_ANCILLARY) ||
     CONSTANT(WCSCOMPARE_TILING) ||
     CONSTANT(WCSCOMPARE_CRPIX));
