@@ -23,6 +23,7 @@ from ...utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
 from ...utils.misc import NumpyRNGContext
 from ...io import fits
+from ...extern.six.moves import range
 
 
 # test_maps() is a generator
@@ -69,8 +70,8 @@ def test_maps():
 
     if len(hdr_file_list) != n_data_files:
         assert False, (
-            "test_maps has wrong number data files: found %d, expected "
-            " %d" % (len(hdr_file_list), n_data_files))
+            "test_maps has wrong number data files: found {}, expected "
+            " {}".format(len(hdr_file_list), n_data_files))
         # b.t.w.  If this assert happens, py.test reports one more test
         # than it would have otherwise.
 
@@ -114,8 +115,8 @@ def test_spectra():
 
     if len(hdr_file_list) != n_data_files:
         assert False, (
-            "test_spectra has wrong number data files: found %d, expected "
-            " %d" % (len(hdr_file_list), n_data_files))
+            "test_spectra has wrong number data files: found {}, expected "
+            " {}".format(len(hdr_file_list), n_data_files))
         # b.t.w.  If this assert happens, py.test reports one more test
         # than it would have otherwise.
 
@@ -175,7 +176,7 @@ def test_pix2world():
         assert len(caught_warnings) == 1
 
     n = 3
-    pixels = (np.arange(n)*np.ones((2, n))).T
+    pixels = (np.arange(n) * np.ones((2, n))).T
     result = ww.wcs_pix2world(pixels, 0, ra_dec_order=True)
 
     # Catch #2791
@@ -186,13 +187,13 @@ def test_pix2world():
     answer = np.array([[0.00024976, 0.00023018],
                        [0.00023043, -0.00024997]])
 
-    assert np.all(np.abs(ww.wcs.pc-answer) < close_enough)
+    assert np.all(np.abs(ww.wcs.pc - answer) < close_enough)
 
-    answer = np.array([[ 202.39265216,   47.17756518],
-                       [ 202.39335826,   47.17754619],
-                       [ 202.39406436,   47.1775272 ]])
+    answer = np.array([[202.39265216, 47.17756518],
+                       [202.39335826, 47.17754619],
+                       [202.39406436, 47.1775272]])
 
-    assert  np.all(np.abs(result-answer) < close_enough)
+    assert np.all(np.abs(result - answer) < close_enough)
 
 
 def test_load_fits_path():
@@ -259,37 +260,37 @@ def test_3d_shapes():
 def test_preserve_shape():
     w = wcs.WCS(naxis=2)
 
-    x = np.random.random((2,3,4))
-    y = np.random.random((2,3,4))
+    x = np.random.random((2, 3, 4))
+    y = np.random.random((2, 3, 4))
 
     xw, yw = w.wcs_pix2world(x, y, 1)
 
-    assert xw.shape == (2,3,4)
-    assert yw.shape == (2,3,4)
+    assert xw.shape == (2, 3, 4)
+    assert yw.shape == (2, 3, 4)
 
     xp, yp = w.wcs_world2pix(x, y, 1)
 
-    assert xp.shape == (2,3,4)
-    assert yp.shape == (2,3,4)
+    assert xp.shape == (2, 3, 4)
+    assert yp.shape == (2, 3, 4)
 
 
 def test_broadcasting():
     w = wcs.WCS(naxis=2)
 
-    x = np.random.random((2,3,4))
+    x = np.random.random((2, 3, 4))
     y = 1
 
     xp, yp = w.wcs_world2pix(x, y, 1)
 
-    assert xp.shape == (2,3,4)
-    assert yp.shape == (2,3,4)
+    assert xp.shape == (2, 3, 4)
+    assert yp.shape == (2, 3, 4)
 
 
 def test_shape_mismatch():
     w = wcs.WCS(naxis=2)
 
-    x = np.random.random((2,3,4))
-    y = np.random.random((3,2,4))
+    x = np.random.random((2, 3, 4))
+    y = np.random.random((3, 2, 4))
 
     with pytest.raises(ValueError) as exc:
         xw, yw = w.wcs_pix2world(x, y, 1)
@@ -424,8 +425,12 @@ def test_validate():
     with catch_warnings():
         results = wcs.validate(get_pkg_data_filename("data/validate.fits"))
         results_txt = repr(results)
-        if wcs._wcs.__version__[0] == '5':
-            filename = 'data/validate.5.0.txt'
+        version = wcs._wcs.__version__
+        if version[0] == '5':
+            if version >= '5.13':
+                filename = 'data/validate.5.13.txt'
+            else:
+                filename = 'data/validate.5.0.txt'
         else:
             filename = 'data/validate.txt'
         with open(get_pkg_data_filename(filename), "r") as fd:
@@ -451,7 +456,7 @@ def test_all_world2pix(fname=None, ext=0,
     # Open test FITS file:
     if fname is None:
         fname = get_pkg_data_filename('data/j94f05bgq_flt.fits')
-        ext = ('SCI',1)
+        ext = ('SCI', 1)
     if not os.path.isfile(fname):
         raise IOError("Input file '{:s}' to 'test_all_world2pix' not found."
                       .format(fname))
@@ -466,8 +471,8 @@ def test_all_world2pix(fname=None, ext=0,
     # Assume that CRPIX is at the center of the image and that the image has
     # a power-of-2 number of pixels along each axis. Only use the central
     # 1/64 for this testing purpose:
-    naxesi_l = list((7./16*crpix).astype(np.int))
-    naxesi_u = list((9./16*crpix).astype(np.int))
+    naxesi_l = list((7. / 16 * crpix).astype(np.int))
+    naxesi_u = list((9. / 16 * crpix).astype(np.int))
 
     # Generate integer indices of pixels (image grid):
     img_pix = np.dstack([i.flatten() for i in
@@ -478,8 +483,8 @@ def test_all_world2pix(fname=None, ext=0,
         rnd_pix = np.random.rand(random_npts, ncoord)
 
     # Scale random data to cover the central part of the image
-    mwidth = 2 * (crpix * 1./8)
-    rnd_pix = crpix - 0.5*mwidth + (mwidth-1) * rnd_pix
+    mwidth = 2 * (crpix * 1. / 8)
+    rnd_pix = crpix - 0.5 * mwidth + (mwidth - 1) * rnd_pix
 
     # Reference pixel coordinates in image coordinate system (CS):
     test_pix = np.append(img_pix, rnd_pix, axis=0)
@@ -546,7 +551,7 @@ def test_all_world2pix(fname=None, ext=0,
           "Run time: {2}\n"
           .format(meanerr, maxerr, runtime_end - runtime_begin))
 
-    assert(maxerr < 2.0*tolerance)
+    assert(maxerr < 2.0 * tolerance)
 
 
 def test_scamp_sip_distortion_parameters():
@@ -590,8 +595,28 @@ def test_footprint_to_file(tmpdir):
                  'CTYPE2': 'DEC--ZPN', 'CRUNIT2': 'deg',
                  'CRPIX2': 3.0453999e+03, 'CRVAL2': 4.388538000000e+01,
                  'PV2_1': 1., 'PV2_3': 220.})
-    # Just check that this doesn't raise an exception:
-    w.footprint_to_file(str(tmpdir.join('test.txt')))
+
+    testfile = str(tmpdir.join('test.txt'))
+    w.footprint_to_file(testfile)
+
+    with open(testfile, 'r') as f:
+        lines = f.readlines()
+
+    assert len(lines) == 4
+    assert lines[2] == 'ICRS\n'
+    assert 'color=green' in lines[3]
+
+    w.footprint_to_file(testfile, coordsys='FK5', color='red')
+
+    with open(testfile, 'r') as f:
+        lines = f.readlines()
+
+    assert len(lines) == 4
+    assert lines[2] == 'FK5\n'
+    assert 'color=red' in lines[3]
+
+    with pytest.raises(ValueError):
+        w.footprint_to_file(testfile, coordsys='FOO')
 
 
 def test_validate_faulty_wcs():
@@ -640,10 +665,10 @@ def test_calc_footprint_1():
     w = wcs.WCS(fits)
 
     axes = (1000, 1051)
-    ref = np.array([[ 202.39314493,   47.17753352],
-                    [ 202.71885939,   46.94630488],
-                    [ 202.94631893,   47.15855022],
-                    [ 202.72053428,   47.37893142]])
+    ref = np.array([[202.39314493, 47.17753352],
+                    [202.71885939, 46.94630488],
+                    [202.94631893, 47.15855022],
+                    [202.72053428, 47.37893142]])
     footprint = w.calc_footprint(axes=axes)
     assert_allclose(footprint, ref)
 
@@ -654,10 +679,10 @@ def test_calc_footprint_2():
     w = wcs.WCS(fits)
 
     axes = (1000, 1051)
-    ref = np.array([[ 202.39265216,   47.17756518],
-                    [ 202.7469062 ,   46.91483312],
-                    [ 203.11487481,   47.14359319],
-                    [ 202.76092671,   47.40745948]])
+    ref = np.array([[202.39265216, 47.17756518],
+                    [202.7469062, 46.91483312],
+                    [203.11487481, 47.14359319],
+                    [202.76092671, 47.40745948]])
     footprint = w.calc_footprint(axes=axes, undistort=False)
     assert_allclose(footprint, ref)
 
@@ -834,6 +859,7 @@ def test_hst_wcs():
     assert w.sip.bp_order == 0
     assert_array_equal(w.sip.crpix, [2048., 1024.])
     wcs.WCS(hdulist[1].header, hdulist)
+    hdulist.close()
 
 
 def test_list_naxis():
@@ -856,6 +882,7 @@ def test_list_naxis():
     w = wcs.WCS(content, naxis=['spectral'])
     assert w.naxis == 0
     assert w.wcs.naxis == 0
+    hdulist.close()
 
 
 def test_sip_broken():
@@ -864,3 +891,188 @@ def test_sip_broken():
     hdr = get_pkg_data_contents("data/sip-broken.hdr")
 
     w = wcs.WCS(hdr)
+
+
+def test_no_truncate_crval():
+    """
+    Regression test for https://github.com/astropy/astropy/issues/4612
+    """
+    w = wcs.WCS(naxis=3)
+    w.wcs.crval = [50, 50, 2.12345678e11]
+    w.wcs.cdelt = [1e-3, 1e-3, 1e8]
+    w.wcs.ctype = ['RA---TAN', 'DEC--TAN', 'FREQ']
+    w.wcs.set()
+
+    header = w.to_header()
+    for ii in range(3):
+        assert header['CRVAL{0}'.format(ii + 1)] == w.wcs.crval[ii]
+        assert header['CDELT{0}'.format(ii + 1)] == w.wcs.cdelt[ii]
+
+
+def test_no_truncate_crval_try2():
+    """
+    Regression test for https://github.com/astropy/astropy/issues/4612
+    """
+    w = wcs.WCS(naxis=3)
+    w.wcs.crval = [50, 50, 2.12345678e11]
+    w.wcs.cdelt = [1e-5, 1e-5, 1e5]
+    w.wcs.ctype = ['RA---SIN', 'DEC--SIN', 'FREQ']
+    w.wcs.cunit = ['deg', 'deg', 'Hz']
+    w.wcs.crpix = [1, 1, 1]
+    w.wcs.restfrq = 2.34e11
+    w.wcs.set()
+
+    header = w.to_header()
+    for ii in range(3):
+        assert header['CRVAL{0}'.format(ii + 1)] == w.wcs.crval[ii]
+        assert header['CDELT{0}'.format(ii + 1)] == w.wcs.cdelt[ii]
+
+
+def test_no_truncate_crval_p17():
+    """
+    Regression test for https://github.com/astropy/astropy/issues/5162
+    """
+    w = wcs.WCS(naxis=2)
+    w.wcs.crval = [50.1234567890123456, 50.1234567890123456]
+    w.wcs.cdelt = [1e-3, 1e-3]
+    w.wcs.ctype = ['RA---TAN', 'DEC--TAN']
+    w.wcs.set()
+
+    header = w.to_header()
+    assert header['CRVAL1'] != w.wcs.crval[0]
+    assert header['CRVAL2'] != w.wcs.crval[1]
+    header = w.to_header(relax=wcs.WCSHDO_P17)
+    assert header['CRVAL1'] == w.wcs.crval[0]
+    assert header['CRVAL2'] == w.wcs.crval[1]
+
+
+def test_no_truncate_using_compare():
+    """
+    Regression test for https://github.com/astropy/astropy/issues/4612
+
+    This one uses WCS.wcs.compare and some slightly different values
+    """
+    w = wcs.WCS(naxis=3)
+    w.wcs.crval = [2.409303333333E+02, 50, 2.12345678e11]
+    w.wcs.cdelt = [1e-3, 1e-3, 1e8]
+    w.wcs.ctype = ['RA---TAN', 'DEC--TAN', 'FREQ']
+    w.wcs.set()
+    w2 = wcs.WCS(w.to_header())
+    w.wcs.compare(w2.wcs)
+
+
+def test_passing_ImageHDU():
+    """
+    Passing ImageHDU or PrimaryHDU and comparing it with
+    wcs initialized from header. For #4493.
+    """
+    path = get_pkg_data_filename('data/validate.fits')
+    hdulist = fits.open(path)
+    wcs_hdu = wcs.WCS(hdulist[0])
+    wcs_header = wcs.WCS(hdulist[0].header)
+    assert wcs_hdu.wcs.compare(wcs_header.wcs)
+    wcs_hdu = wcs.WCS(hdulist[1])
+    wcs_header = wcs.WCS(hdulist[1].header)
+    assert wcs_hdu.wcs.compare(wcs_header.wcs)
+    hdulist.close()
+
+
+def test_inconsistent_sip():
+    """
+    Test for #4814
+    """
+    hdr = get_pkg_data_contents("data/sip-broken.hdr")
+    w = wcs.WCS(hdr)
+    newhdr = w.to_header(relax=None)
+    # CTYPE should not include "-SIP" if relax is None
+    wnew = wcs.WCS(newhdr)
+    assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
+    newhdr = w.to_header(relax=False)
+    assert('A_0_2' not in newhdr)
+    # CTYPE should not include "-SIP" if relax is False
+    wnew = wcs.WCS(newhdr)
+    assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
+    newhdr = w.to_header(key="C")
+    assert('A_0_2' not in newhdr)
+    # Test writing header with a different key
+    wnew = wcs.WCS(newhdr, key='C')
+    assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
+    newhdr = w.to_header(key=" ")
+    # Test writing a primary WCS to header
+    wnew = wcs.WCS(newhdr)
+    assert all(not ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
+    # Test that "-SIP" is kept into CTYPE if relax=True and
+    # "-SIP" was in the original header
+    newhdr = w.to_header(relax=True)
+    wnew = wcs.WCS(newhdr)
+    assert all(ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
+    assert('A_0_2' in newhdr)
+    # Test that SIP coefficients are also written out.
+    assert wnew.sip is not None
+    ########## broken header ###########
+    # Test that "-SIP" is added to CTYPE if relax=True and
+    # "-SIP" was not in the original header but SIP coefficients
+    # are present.
+    w = wcs.WCS(hdr)
+    w.wcs.ctype = ['RA---TAN', 'DEC--TAN']
+    newhdr = w.to_header(relax=True)
+    wnew = wcs.WCS(newhdr)
+    assert all(ctyp.endswith('-SIP') for ctyp in wnew.wcs.ctype)
+
+
+def test_bounds_check():
+    """Test for #4957"""
+    w = wcs.WCS(naxis=2)
+    w.wcs.ctype = ["RA---CAR","DEC--CAR"]
+    w.wcs.cdelt = [10,10]
+    w.wcs.crval = [-90,90]
+    w.wcs.crpix = [1,1]
+    w.wcs.bounds_check(False, False)
+    ra, dec = w.wcs_pix2world(300,0,0)
+    assert_allclose(ra, -180)
+    assert_allclose(dec, -30)
+
+
+def test_naxis():
+    w = wcs.WCS(naxis=2)
+    w.wcs.crval = [1, 1]
+    w.wcs.cdelt = [0.1, 0.1]
+    w.wcs.crpix = [1, 1]
+    w._naxis = [1000, 500]
+
+    assert w._naxis1 == 1000
+    assert w._naxis2 == 500
+
+    w._naxis1 = 99
+    w._naxis2 = 59
+    assert w._naxis == [99, 59]
+
+
+def test_sip_with_altkey():
+    """
+    Test that when creating a WCS object using a key, CTYPE with
+    that key is looked at and not the primary CTYPE.
+    fix for #5443.
+    """
+    with fits.open(get_pkg_data_filename('data/sip.fits')) as f:
+        w = wcs.WCS(f[0].header)
+    # create a header with two WCSs.
+    h1 = w.to_header(relax=True, key='A')
+    h2 = w.to_header(relax=False)
+    h1['CTYPE1A'] = "RA---SIN-SIP"
+    h1['CTYPE2A'] = "DEC--SIN-SIP"
+    h1.update(h2)
+    w = wcs.WCS(h1, key='A')
+    assert (w.wcs.ctype == np.array(['RA---SIN-SIP', 'DEC--SIN-SIP'])).all()
+
+
+def test_to_fits_1():
+    """
+    Test to_fits() with LookupTable distortion.
+    """
+    fits_name = get_pkg_data_filename('data/dist.fits')
+    w = wcs.WCS(fits_name)
+    wfits = w.to_fits()
+    assert isinstance(wfits, fits.HDUList)
+    assert isinstance(wfits[0], fits.PrimaryHDU)
+    assert isinstance(wfits[1], fits.ImageHDU)
