@@ -51,16 +51,16 @@ def test_cd():
     w = _wcs.Wcsprm()
     w.cd = [[1, 0], [0, 1]]
     assert w.cd.dtype == np.float
-    assert w.has_cd() == True
+    assert w.has_cd() is True
     assert_array_equal(w.cd, [[1, 0], [0, 1]])
     del w.cd
-    assert w.has_cd() == False
+    assert w.has_cd() is False
 
 
 @raises(AttributeError)
 def test_cd_missing():
     w = _wcs.Wcsprm()
-    assert w.has_cd() == False
+    assert w.has_cd() is False
     w.cd
 
 
@@ -68,9 +68,9 @@ def test_cd_missing():
 def test_cd_missing2():
     w = _wcs.Wcsprm()
     w.cd = [[1, 0], [0, 1]]
-    assert w.has_cd() == True
+    assert w.has_cd() is True
     del w.cd
-    assert w.has_cd() == False
+    assert w.has_cd() is False
     w.cd
 
 
@@ -179,16 +179,16 @@ def test_crota():
     w = _wcs.Wcsprm()
     w.crota = [1, 0]
     assert w.crota.dtype == np.float
-    assert w.has_crota() == True
+    assert w.has_crota() is True
     assert_array_equal(w.crota, [1, 0])
     del w.crota
-    assert w.has_crota() == False
+    assert w.has_crota() is False
 
 
 @raises(AttributeError)
 def test_crota_missing():
     w = _wcs.Wcsprm()
-    assert w.has_crota() == False
+    assert w.has_crota() is False
     w.crota
 
 
@@ -196,9 +196,9 @@ def test_crota_missing():
 def test_crota_missing2():
     w = _wcs.Wcsprm()
     w.crota = [1, 0]
-    assert w.has_crota() == True
+    assert w.has_crota() is True
     del w.crota
-    assert w.has_crota() == False
+    assert w.has_crota() is False
     w.crota
 
 
